@@ -106,12 +106,6 @@ static gboolean
 cong_app_private_load_displayspecs (CongApp *app,
 				    GtkWindow *toplevel_window);
 
-static void 
-register_plugin (CongApp *app,
-		 const gchar *id,
-		 CongPluginCallbackRegister register_callback,
-		 CongPluginCallbackConfigure configure_callback);
-
 static void 
 cong_app_private_load_plugins (CongApp *app);
 
@@ -119,6 +113,12 @@ static void
 cong_app_private_insert_element_init (CongApp *app);
 
 /* Exported function definitions: */
+/**
+ * cong_app_singleton:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongApp*
 cong_app_singleton (void)
 {
@@ -127,6 +127,13 @@ cong_app_singleton (void)
 	return the_singleton_app;
 }
 
+/**
+ * cong_app_construct_singleton:
+ * @argc:
+ * @argv:
+ *
+ * TODO: Write me
+ */
 void
 cong_app_construct_singleton (int   argc,
 			      char *argv[])
@@ -137,6 +144,11 @@ cong_app_construct_singleton (int   argc,
 					 argv);
 }
 
+/**
+ * cong_app_destroy_singleton:
+ *
+ * TODO: Write me
+ */
 void
 cong_app_destroy_singleton(void)
 {
@@ -147,6 +159,13 @@ cong_app_destroy_singleton(void)
 	the_singleton_app = NULL;
 }
 
+/**
+ * cong_app_post_init_hack:
+ * @app:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 int
 cong_app_post_init_hack (CongApp *app)
 {
@@ -274,6 +293,14 @@ debug_target_list (GtkClipboard *clipboard,
 }
 #endif
 
+/**
+ * convert_ucs2_to_utf8:
+ * @data:
+ * @length:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gchar*
 convert_ucs2_to_utf8 (guchar *data,
 		      gint length)
@@ -285,6 +312,14 @@ convert_ucs2_to_utf8 (guchar *data,
 				NULL);
 }
 
+/**
+ * convert_utf8_string:
+ * @data:
+ * @length:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gchar*
 convert_utf8_string (guchar *data, 
 		     gint length)
@@ -296,6 +331,15 @@ convert_utf8_string (guchar *data,
 				     (gssize)length);
 }
 
+/**
+ * convert_text_xml:
+ * @data:
+ * @length:
+ *
+ * This function is not currently implemented
+ *
+ * Returns: %NULL
+ */
 gchar*
 convert_text_xml (guchar *data, 
 		  gint length)
@@ -305,6 +349,14 @@ convert_text_xml (guchar *data,
 	return NULL;
 }
 
+/**
+ * convert_text_html:
+ * @data:
+ * @length:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gchar*
 convert_text_html (guchar *data, 
 		   gint length)
@@ -320,6 +372,15 @@ convert_text_html (guchar *data,
 #endif
 }
 
+/**
+ * convert_text_plain:
+ * @data:
+ * @length:
+ *
+ * This function is not currently implemented
+ *
+ * Returns: %NULL
+ */
 gchar*
 convert_text_plain (guchar *data, 
 		    gint length)
@@ -329,6 +390,15 @@ convert_text_plain (guchar *data,
 	return NULL;
 }
 
+/**
+ * convert_application_xhtml_plus_xml:
+ * @data:
+ * @length:
+ *
+ * This function is not currently implemented
+ *
+ * Returns: %NULL
+ */
 gchar*
 convert_application_xhtml_plus_xml (guchar *data, 
 				    gint length)
@@ -338,6 +408,15 @@ convert_application_xhtml_plus_xml (guchar *data,
 	return NULL;
 }
 
+/**
+ * convert_application_rtf:
+ * @data:
+ * @length:
+ *
+ * This function is not currently implemented
+ *
+ * Returns: %NULL
+ */
 gchar*
 convert_application_rtf (guchar *data, 
 			 gint length)
@@ -383,6 +462,15 @@ debug_well_known_targets (GtkClipboard *clipboard)
 #endif
 
 /* This is a simple copy-and-paste of gtk_clipboard_wait_for_targets, which was added to GTK in version 2.4: */
+/**
+ * cong_eel_gtk_clipboard_wait_for_targets:
+ * @clipboard:
+ * @targets:
+ * @n_targets:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_eel_gtk_clipboard_wait_for_targets (GtkClipboard  *clipboard, 
 					 GdkAtom      **targets,
@@ -435,6 +523,20 @@ generate_source_fragment_from_selection_doc (xmlDocPtr xml_doc,
 	return NULL;
 }
 
+/**
+ * cong_app_get_clipboard_xml_source:
+ * @app: the #CongApp
+ * @selection: should be either GDK_SELECTION_CLIPBOARD or GDK_SELECTION_PRIMARY
+ * @target_doc: the #CongDocument into which the source is to be pasted
+ * 
+ * This function interrogates the appropriate clipboard and attempts to get the content
+ * in the best possible format for the target document.  It may attempt to do conversions,
+ * for example, converting &lt;li&gt; elements into &lt;listitem&gt; when pasting HTML into a DocBook document.
+ *
+ * Returns: a newly-allocated UTF-8 fragment of XML source, wrapped in a &lt;placeholder&gt; element,
+ * suitable for parsing and adding to the document (which must then be freed using g_free) or NULL
+ * Returns: 
+ */
 gchar*
 cong_app_get_clipboard_xml_source (CongApp *app,
 				   GdkAtom selection,
@@ -683,8 +785,22 @@ regenerate_selection (CongApp *app,
 	}
 }
 
-
-
+/**
+ * cong_app_set_clipboard_from_xml_fragment:
+ * @app: the #CongApp
+ * @selection: should be either GDK_SELECTION_CLIPBOARD or GDK_SELECTION_PRIMARY
+ * @xml_fragment: a fragment of XML source
+ * @source_doc: the #CongDocument from which the source has been cut
+ * 
+ * This function takes the XML source and attempts to place it into the appropriate clipboard.  It will attempt to make it 
+ * available in a number of formats, and in the best possible way for each format.  It may attempt to do conversions when it does 
+ * this, e.g. generating pretty versions of bulleted lists for text, or converting to an HTML representation
+ * where appropriate.
+ *
+ * The XML form of the source is not converted, but is wrapped in an &lt;xml-fragment&gt; top-level element and given a DOCTYPE declaration if available in the source document.
+ * So it should be well-formed, but not valid.
+ * Haven't yet specified what happens to entities.
+ */
 void
 cong_app_set_clipboard_from_xml_fragment (CongApp *app,
 					  GdkAtom selection,
@@ -725,6 +841,13 @@ cong_app_set_clipboard_from_xml_fragment (CongApp *app,
 	/* emit signals? */
 }
 
+/**
+ * cong_app_get_gnome_program:
+ * @app:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 GnomeProgram*
 cong_app_get_gnome_program (CongApp *app)
 {
@@ -733,6 +856,13 @@ cong_app_get_gnome_program (CongApp *app)
 	return PRIVATE(app)->gnome_program;
 }
 
+/**
+ * cong_app_get_tooltips:
+ * @app:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 GtkTooltips*
 cong_app_get_tooltips (CongApp *app)
 {
@@ -741,9 +871,17 @@ cong_app_get_tooltips (CongApp *app)
 	return PRIVATE(app)->tooltips;
 }
 
+/**
+ * cong_app_get_font:
+ * @app:
+ * @role:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongFont*
 cong_app_get_font (CongApp *app,
-		   enum CongFontRole role)
+		   CongFontRole role)
 {
 	g_return_val_if_fail (app, NULL);
 	g_return_val_if_fail (role<CONG_FONT_ROLE_NUM, NULL);
@@ -751,6 +889,13 @@ cong_app_get_font (CongApp *app,
 	return PRIVATE(app)->fonts[role];
 }
 
+/**
+ * cong_app_get_plugin_manager:
+ * @app:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongPluginManager*
 cong_app_get_plugin_manager (CongApp *app)
 {
@@ -759,6 +904,13 @@ cong_app_get_plugin_manager (CongApp *app)
 	return PRIVATE(app)->plugin_manager;
 }
 
+/**
+ * cong_app_get_dispspec_registry:
+ * @app:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongDispspecRegistry*
 cong_app_get_dispspec_registry (CongApp *app)
 {
@@ -767,6 +919,13 @@ cong_app_get_dispspec_registry (CongApp *app)
 	return PRIVATE(app)->ds_registry;
 }
 
+/**
+ * cong_app_get_gconf_client:
+ * @app:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 GConfClient*
 cong_app_get_gconf_client (CongApp *app)
 {
@@ -775,6 +934,13 @@ cong_app_get_gconf_client (CongApp *app)
 	return PRIVATE(app)->gconf_client;
 }
 
+/**
+ * cong_app_get_language_list:
+ * @app:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 const GList*
 cong_app_get_language_list (CongApp *app)
 {
@@ -783,6 +949,29 @@ cong_app_get_language_list (CongApp *app)
 	return PRIVATE(app)->language_list;
 }
 
+static gboolean
+handle_cmdline_args (gpointer data)
+{
+	poptContext ctx;
+	const gchar **startup_files;
+	gchar *uri;
+	int i;
+
+	ctx = data;
+	startup_files = poptGetArgs (ctx);
+
+	if (startup_files) {
+		for (i = 0; startup_files [i]; ++i) {
+		       uri = gnome_vfs_make_uri_from_shell_arg (startup_files[i]);
+		       open_document_do (uri, NULL);
+		       g_free(uri);
+		   }
+	}
+	
+	poptFreeContext (ctx);
+
+	return FALSE;
+}
 
 
 /* Internal function definitions: */
@@ -791,20 +980,29 @@ cong_app_new (int   argc,
 	      char *argv[])
 {
 	CongApp *app;
+	GValue value = { 0 };
+	poptContext ctx;
 
 	app = g_new0(CongApp,1);
 	app->private = g_new0(CongAppPrivate,1);
 
+
 	/* Set up the GnomeProgram: */
 	PRIVATE(app)->gnome_program = gnome_program_init (PACKAGE_NAME, PACKAGE_VERSION,
 							  LIBGNOMEUI_MODULE,
 							  argc,argv,
 							  GNOME_PARAM_HUMAN_READABLE_NAME,
 							  _("XML Editor"),
-							  GNOME_PARAM_APP_DATADIR, PKGDATADIR,
-							  /* GSt: bugzilla # 128544 reminder */
+							  GNOME_PROGRAM_STANDARD_PROPERTIES,
 							  NULL);
 
+	g_value_init (&value, G_TYPE_POINTER);
+	g_object_get_property (G_OBJECT (PRIVATE (app)->gnome_program), GNOME_PARAM_POPT_CONTEXT,
+			       &value);
+	ctx = g_value_get_pointer (&value);
+	g_value_unset (&value);
+	g_idle_add (handle_cmdline_args, ctx);
+
 	/* Set up usage of GConf: */
 	PRIVATE(app)->gconf_client = gconf_client_get_default();
 	gconf_client_add_dir (PRIVATE(app)->gconf_client,
@@ -833,7 +1031,7 @@ cong_app_new (int   argc,
 	{
 		gchar *stylesheet_file = gnome_program_locate_file (PRIVATE(app)->gnome_program,
 								    GNOME_FILE_DOMAIN_APP_DATADIR,
-								    "stylesheets/selection-to-text.xsl",
+								    "conglomerate/stylesheets/selection-to-text.xsl",
 								    FALSE,
 								    NULL);
 
@@ -915,7 +1113,7 @@ cong_app_private_load_displayspecs (CongApp *app,
 	{
 		gchar* xds_directory = gnome_program_locate_file (PRIVATE(app)->gnome_program,
 								  GNOME_FILE_DOMAIN_APP_DATADIR,
-								  "dispspecs",
+								  "conglomerate/dispspecs",
 								  FALSE,
 								  NULL);
 
@@ -946,96 +1144,10 @@ cong_app_private_load_displayspecs (CongApp *app,
 	return TRUE;
 }
 
-static void 
-register_plugin (CongApp *app,
-		 const gchar *id,
-		 CongPluginCallbackRegister register_callback,
-		 CongPluginCallbackConfigure configure_callback)
-{
-	g_return_if_fail(id);
-	g_return_if_fail(register_callback);
-
-	g_assert(PRIVATE(app)->plugin_manager);
-
-	cong_plugin_manager_register(PRIVATE(app)->plugin_manager,
-				     id,
-				     register_callback, 
-				     configure_callback);
-}
-
-
-
 static void 
 cong_app_private_load_plugins (CongApp *app)
 {
-	/* For the moment, there aren't any actual plugins; instead we fake it. */
-	register_plugin(app,"admonition",
-			plugin_admonition_plugin_register,
-			plugin_admonition_plugin_configure);
-
-	register_plugin(app,"docbook",
-			plugin_docbook_plugin_register,
-			plugin_docbook_plugin_configure);
-
-	register_plugin(app,"empty",
-			plugin_empty_plugin_register,
-			plugin_empty_plugin_configure);
-
-	register_plugin(app,"fo",
-			plugin_fo_plugin_register,
-			plugin_fo_plugin_configure);
-
-	register_plugin(app,"lists",
-			plugin_lists_plugin_register,
-			plugin_lists_plugin_configure);
-
-	register_plugin(app,"random",
-			plugin_random_plugin_register,
-			plugin_random_plugin_configure);
-
-	register_plugin(app,"sgml",
-			plugin_sgml_plugin_register,
-			plugin_sgml_plugin_configure);
-
-	register_plugin(app,"tests",
-			plugin_tests_plugin_register,
-			plugin_tests_plugin_configure);
-
-	register_plugin(app,"validate",
-			plugin_validate_plugin_register,
-			plugin_validate_plugin_configure);
-
-	register_plugin(app,"website",
-			plugin_website_plugin_register,
-			plugin_website_plugin_configure);
-
-	register_plugin(app,"xsl",
-			plugin_xsl_plugin_register,
-			plugin_xsl_plugin_configure);
-
-	register_plugin(app,"convert-case",
-			plugin_convert_case_plugin_register,
-			plugin_convert_case_plugin_configure);
-
-	register_plugin(app,"cleanup-source",
-			plugin_cleanup_source_plugin_register,
-			plugin_cleanup_source_plugin_configure);
-
-	register_plugin(app,"dtd",
-			plugin_dtd_plugin_register,
-			plugin_dtd_plugin_configure);
-
-	register_plugin(app,"paragraph",
-			plugin_paragraph_plugin_register,
-			plugin_paragraph_plugin_configure);
-
-	register_plugin(app,"save-dispspec",
-			plugin_save_dispspec_plugin_register,
-			plugin_save_dispspec_plugin_configure);
-
-	register_plugin(app,"templates",
-			plugin_templates_plugin_register,
-			plugin_templates_plugin_configure);
+	cong_fake_plugin_hook_register_the_whole_shebang (app);
 }
 
 static void 
@@ -1051,11 +1163,3 @@ cong_app_private_insert_element_init (CongApp *app)
 	gdk_gc_set_foreground(app->insert_element_gc, &gcol);
 #endif
 }
-
-
-
-
-
-
-
-
