@@ -26,6 +26,46 @@
 #include "cong-enum-mapping.h"
 #include "cong-ui-hooks.h"
 
+struct CongDispspecElementHeaderInfo
+{
+	gchar *xpath; /* if present, this is the XPath to use when determining the title of the tag */
+};
+
+struct CongDispspecElement
+{
+	CongDispspec *ds;
+
+	/* Local name within namespace (if any); must be non-NULL */
+	gchar *local_name;
+
+	GHashTable *hash_of_language_to_user_name;
+	GHashTable *hash_of_language_to_short_desc;
+
+	gchar *icon_name;
+	GdkPixbuf *icon16;
+	CongWhitespaceHandling whitespace;
+
+	CongElementType type;
+	gboolean collapseto;
+
+#if NEW_LOOK
+	GdkColor col_array[CONG_DISPSPEC_GC_USAGE_NUM];
+	GdkGC* gc_array[CONG_DISPSPEC_GC_USAGE_NUM];
+#else
+	GdkColor col;
+	GdkGC* gc;
+#endif
+
+	CongDispspecElementHeaderInfo *header_info;
+
+	gchar *editor_service_id;
+	gchar *property_dialog_service_id;
+
+	gboolean should_spellcheck;
+
+	GHashTable *key_value_hash;
+};
+
 #if 0
 #define DS_DEBUG_MSG1(x)    g_message((x))
 #define DS_DEBUG_MSG2(x, a) g_message((x), (a))
@@ -38,29 +78,6 @@
 
 
 /* Internal data structure declarations: */
-static guint 
-g_str_or_null_hash (gconstpointer key)
-{
-	if (key) {
-		return g_str_hash (key);
-	} else {
-		return 0;
-	}
-}
-
-gboolean
-g_str_or_null_equal (gconstpointer a,
-		     gconstpointer b)
-{
-	if (a) {
-		if (b) {
-			return g_str_equal (a, b);
-		}
-	}
-
-	return (a==b);
-}
-
 /* Internal function prototypes: */
 static gpointer
 find_best_value_for_language (GHashTable *hash_of_language);
@@ -85,6 +102,11 @@ static const CongEnumMapping whitespace_numeration[] =
 	{"normalize", CONG_WHITESPACE_NORMALIZE}
 };
 
+/**
+ * gxx_callback_construct_dispspec_element:
+ *
+ * TODO: Write me
+ */
 CongDispspecElement*
 gxx_callback_construct_dispspec_element(void)
 {
@@ -92,12 +114,12 @@ gxx_callback_construct_dispspec_element(void)
 
 	element->whitespace = CONG_WHITESPACE_NORMALIZE;
 
-	element->hash_of_language_to_user_name = g_hash_table_new_full (g_str_or_null_hash,
-									g_str_or_null_equal,
+	element->hash_of_language_to_user_name = g_hash_table_new_full (cong_str_or_null_hash,
+									cong_str_or_null_equal,
 									g_free,
 									g_free);
-	element->hash_of_language_to_short_desc = g_hash_table_new_full (g_str_or_null_hash,
-									 g_str_or_null_equal,
+	element->hash_of_language_to_short_desc = g_hash_table_new_full (cong_str_or_null_hash,
+									 cong_str_or_null_equal,
 									 g_free,
 									 g_free);	
 	element->key_value_hash = g_hash_table_new_full (g_str_hash,
@@ -145,13 +167,20 @@ unsigned int hacked_cols[3][CONG_DISPSPEC_GC_USAGE_NUM] =
 	{ 0x632829, 0x632829, 0x632829, 0x632829 }
 };
 
-static void get_col(GdkColor *dst, const GdkColor *src, enum CongDispspecGCUsage usage)
+static void get_col(GdkColor *dst, const GdkColor *src, CongDispspecGCUsage usage)
 {
 	/* pick one of the test colour tables based on a dodgy hashing of the source colour: */
 	col_to_gcol(dst, hacked_cols[(src->red>>8)%2][usage]);
 }
 #endif /* #if NEW_LOOK */
 
+/**
+ * generate_gc_for_col:
+ * @col:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 GdkGC*
 generate_gc_for_col (const GdkColor *col)
 {
@@ -226,25 +255,35 @@ static void cong_dispspec_element_init_col(CongDispspecElement* element, unsigne
 
 /* Exported functions: */
 
-/* Construction  */
+/** 
+ * cong_dispspec_element_new:
+ * @ds: the #CongDispspec this will be added to
+ * @local_name: the local name for the new element; must be non-NULL
+ * @type:
+ * @autogenerate_username:  if TRUE, then generate a sane user-visible name for the element,
+ * using "header capitalisation"
+ *
+ * Constructs a new #CongDispspecElement, initialising fields to sane defaults.
+ *
+ * Returns: a freshly allocated #CongDispspecElement 
+ **/
 CongDispspecElement*
-cong_dispspec_element_new (const gchar* ns_uri, 
+cong_dispspec_element_new (CongDispspec *ds,
 			   const gchar* local_name, 
-			   enum CongElementType type,
+			   CongElementType type,
 			   gboolean autogenerate_username)
 {
 	CongDispspecElement* element;
 
 	g_return_val_if_fail (local_name, NULL);
 	
-	g_message("cong_dispspec_element_new (\"%s\",\"%s\",)", ns_uri, local_name);
+	g_message("cong_dispspec_element_new (\"%s\",)", local_name);
 
 	/* Use shared constructor code: */
 	element = gxx_callback_construct_dispspec_element();
 
-	if (ns_uri) {
-		element->ns_uri = g_strdup(ns_uri);
-	}
+	element->ds = ds;
+
 	element->local_name = g_strdup(local_name);
 
 	if (autogenerate_username) {
@@ -259,6 +298,8 @@ cong_dispspec_element_new (const gchar* ns_uri,
 
 	element->type = type;
 
+	element->should_spellcheck = TRUE;
+
 	/* Extract colour: */
 	{
 #if NEW_LOOK
@@ -274,15 +315,18 @@ cong_dispspec_element_new (const gchar* ns_uri,
 }
 
 /* Destruction  */
+/**
+ * cong_dispspec_element_destroy:
+ * @element:
+ *
+ * TODO: Write me
+ */
 void 
 cong_dispspec_element_destroy (CongDispspecElement *element)
 {
 	g_return_if_fail (element);
 
 	/* FIXME: could autogenerate this code: */
-	if (element->ns_uri) {
-		g_free (element->ns_uri);
-	}
 	if (element->local_name) {
 		g_free (element->local_name);
 	}
@@ -317,23 +361,60 @@ cong_dispspec_element_destroy (CongDispspecElement *element)
 	/* FIXME:  do we need to remove from the list? */
 }
 
-
-const gchar*
-cong_dispspec_element_get_ns_uri (CongDispspecElement *element)
+/** 
+ * cong_dispspec_element_get_dispspec:
+ * @element: the element in question
+ * 
+ * Returns: the #CongDispspec containing this element
+ */
+CongDispspec*
+cong_dispspec_element_get_dispspec (const CongDispspecElement *element)
 {
 	g_return_val_if_fail (element, NULL);
 
-	return element->ns_uri;
+	return element->ds;
 }
 
+
+/** 
+ * cong_dispspec_element_get_ns_uri:
+ * @element: the element in question
+ * 
+ * Returns: the namespace URI for this kind of element, or NULL if none
+ */
 const gchar*
-cong_dispspec_element_get_local_name(CongDispspecElement* element)
+cong_dispspec_element_get_ns_uri (const CongDispspecElement *element)
+{
+	g_return_val_if_fail (element, NULL);
+
+	if (element->ds) {
+		return cong_dispspec_get_ns_uri (element->ds);
+	} else {
+		return NULL;
+	}
+}
+
+/**
+ * cong_dispspec_element_get_local_name:
+ * @element: the element in question
+ * 
+ * Returns: the local name (relative to its namespace, if any) for this kind of element
+ */
+const gchar*
+cong_dispspec_element_get_local_name(const CongDispspecElement* element)
 {
 	g_return_val_if_fail (element, NULL);
 
 	return element->local_name;
 }
 
+/**
+ * cong_dispspec_element_username:
+ * @element:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 const gchar*
 cong_dispspec_element_username(CongDispspecElement* element)
 {
@@ -349,6 +430,13 @@ cong_dispspec_element_username(CongDispspecElement* element)
 	}
 }
 
+/**
+ * cong_dispspec_element_get_description:
+ * @element:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 const gchar*
 cong_dispspec_element_get_description(CongDispspecElement *element)
 {
@@ -357,6 +445,13 @@ cong_dispspec_element_get_description(CongDispspecElement *element)
 	return find_best_string_for_language (element->hash_of_language_to_short_desc);
 }
 
+/**
+ * cong_dispspec_element_get_icon:
+ * @element:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 GdkPixbuf*
 cong_dispspec_element_get_icon(CongDispspecElement *element)
 {
@@ -368,6 +463,16 @@ cong_dispspec_element_get_icon(CongDispspecElement *element)
 	return element->icon16;
 }
 
+/**
+ * cong_dispspec_element_get_value_for_key:
+ * @key: the key
+ * @element:  the dispspec element
+ * 
+ * Dispspec elements support a list of key/value string pairs; this is intended as a mechanism
+ * to allow plugins to have arbitrary data whilst having a DTD for xds files.
+ * 
+ * Returns:  the value, if found, or NULL if not present.
+ */
 const gchar*
 cong_dispspec_element_get_value_for_key (const gchar *key, 
 					 const CongDispspecElement *element)
@@ -379,15 +484,14 @@ cong_dispspec_element_get_value_for_key (const gchar *key,
 				    key);
 }
 
-CongDispspecElement*
-cong_dispspec_element_next(CongDispspecElement* element)
-{
-	g_return_val_if_fail(element, NULL);
-
-	return element->next;
-}
-
-enum CongElementType
+/**
+ * cong_dispspec_element_type:
+ * @element:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+CongElementType
 cong_dispspec_element_type(CongDispspecElement *element)
 {
 	g_return_val_if_fail(element, CONG_ELEMENT_TYPE_UNKNOWN);
@@ -395,7 +499,14 @@ cong_dispspec_element_type(CongDispspecElement *element)
 	return element->type;
 }
 
-enum CongWhitespaceHandling
+/**
+ * cong_dispspec_element_get_whitespace:
+ * @element:  The element of the display spec
+ *
+ * Get the #CongWhitespaceHandling behaviour for this element
+ * Returns:  
+ */
+CongWhitespaceHandling
 cong_dispspec_element_get_whitespace (CongDispspecElement *element)
 {
 	g_return_val_if_fail (element, CONG_WHITESPACE_NORMALIZE);
@@ -403,22 +514,42 @@ cong_dispspec_element_get_whitespace (CongDispspecElement *element)
 	return element->whitespace;
 }
 
+/**
+ * cong_dispspec_element_set_whitespace:
+ * @element:  The element of the display spec
+ * @whitespace: The new value for whitespace handling
+ *
+ * Set the #CongWhitespaceHandling behaviour for this element
+ */
 void
 cong_dispspec_element_set_whitespace (CongDispspecElement *element,
-				      enum CongWhitespaceHandling whitespace)
+				      CongWhitespaceHandling whitespace)
 {	
 	g_return_if_fail (element);
 
 	element->whitespace = whitespace;
 }
 
-
+/**
+ * cong_dispspec_element_collapseto:
+ * @element:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_dispspec_element_collapseto(CongDispspecElement *element)
 {
 	return element->collapseto;
 }
 
+/**
+ * cong_dispspec_element_is_structural:
+ * @element:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_dispspec_element_is_structural(CongDispspecElement *element)
 {
@@ -431,6 +562,13 @@ cong_dispspec_element_is_structural(CongDispspecElement *element)
 	}
 }
 
+/**
+ * cong_dispspec_element_is_span:
+ * @element:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_dispspec_element_is_span(CongDispspecElement *element)
 {
@@ -444,8 +582,16 @@ cong_dispspec_element_is_span(CongDispspecElement *element)
 }
 
 #if NEW_LOOK
+/**
+ * cong_dispspec_element_gc:
+ * @element:
+ * @usage:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 GdkGC*
-cong_dispspec_element_gc(CongDispspecElement *element, enum CongDispspecGCUsage usage)
+cong_dispspec_element_gc(CongDispspecElement *element, CongDispspecGCUsage usage)
 {
 	g_return_val_if_fail(element, NULL);
 	g_return_val_if_fail(usage<CONG_DISPSPEC_GC_USAGE_NUM, NULL);
@@ -453,8 +599,16 @@ cong_dispspec_element_gc(CongDispspecElement *element, enum CongDispspecGCUsage
 	return element->gc_array[usage];
 }
 
+/**
+ * cong_dispspec_element_col:
+ * @element:
+ * @usage:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 const GdkColor*
-cong_dispspec_element_col(CongDispspecElement *element, enum CongDispspecGCUsage usage)
+cong_dispspec_element_col(CongDispspecElement *element, CongDispspecGCUsage usage)
 {
 	g_return_val_if_fail(element, NULL);
 	g_return_val_if_fail(usage<CONG_DISPSPEC_GC_USAGE_NUM, NULL);
@@ -463,6 +617,7 @@ cong_dispspec_element_col(CongDispspecElement *element, enum CongDispspecGCUsage
 }
 
 #else
+
 GdkGC*
 cong_dispspec_element_gc(CongDispspecElement *element)
 {
@@ -480,6 +635,13 @@ cong_dispspec_element_col(CongDispspecElement *element)
 }
 #endif
 
+/**
+ * cong_dispspec_element_header_info:
+ * @element:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongDispspecElementHeaderInfo*
 cong_dispspec_element_header_info(CongDispspecElement *element)
 {
@@ -488,32 +650,40 @@ cong_dispspec_element_header_info(CongDispspecElement *element)
 	return element->header_info;
 }
 
-gchar*
+/**
+ * cong_dispspec_element_header_info_get_xpath_expression:
+ * @header_info:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+const gchar*
 cong_dispspec_element_header_info_get_xpath_expression (CongDispspecElementHeaderInfo *header_info)
 {
 	g_return_val_if_fail (header_info, NULL);
 
-	if (header_info->xpath) {
-		return g_strdup (header_info->xpath);
-	} else if (header_info->tagname) {
-		return g_strdup_printf ("normalize-space(child::%s)", header_info->tagname);
-	} else {
-		return NULL;
-	}
+	return header_info->xpath;
 }
 
-
+/**
+ * cong_dispspec_element_get_title:
+ * @element:
+ * @x:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gchar*
-cong_dispspec_element_get_title(CongDispspecElement *element, CongNodePtr x)
+cong_dispspec_element_get_title (CongDispspecElement *element, 
+				 CongNodePtr x)
 {
-#if 1
 	xmlXPathContextPtr ctxt;
 	xmlXPathObjectPtr xpath_obj;
-	gchar *xpath_string;
+	const gchar *xpath_string;
 
-	g_return_val_if_fail(element, NULL);
-	g_return_val_if_fail(element->header_info, NULL);
-	g_return_val_if_fail(x, NULL);
+	g_return_val_if_fail (element, NULL);
+	g_return_val_if_fail (element->header_info, NULL);
+	g_return_val_if_fail (x, NULL);
 
 	xpath_string = cong_dispspec_element_header_info_get_xpath_expression (element->header_info);
 
@@ -523,80 +693,34 @@ cong_dispspec_element_get_title(CongDispspecElement *element, CongNodePtr x)
 		
 		/* g_message("searching xpath \"%s\"",element->header_info->xpath); */
 		
-		ctxt = xmlXPathNewContext(x->doc);
+		ctxt = xmlXPathNewContext (x->doc);
 		
 		ctxt->node = x;
 		
-		xpath_obj = xmlXPathEval(xpath_string,
-					 ctxt);	
+		xpath_obj = xmlXPathEval (xpath_string,
+					  ctxt);	
 		if (xpath_obj) {
-			result = xmlXPathCastToString(xpath_obj);			
+			result = xmlXPathCastToString (xpath_obj);			
 		} else {
-			result = g_strdup(_("(xpath failed)"));
+			result = g_strdup (_("(xpath failed)"));
 		}	
 		
-		g_free (xpath_string);
-		
-		xmlXPathFreeContext(ctxt);
+		xmlXPathFreeContext (ctxt);
 		
 		return result;
 	} else {
 		return NULL;
 	}		
-#else
-	xmlXPathContextPtr ctxt;
-	xmlXPathObjectPtr xpath_obj;
-
-	g_return_val_if_fail(element, NULL);
-	g_return_val_if_fail(element->header_info, NULL);
-	g_return_val_if_fail(x, NULL);
-
-	/* g_message("cong_dispspec_element_get_title for <%s>", element->tagname); */
-
-	if (element->header_info->xpath) {
-		gchar *result = NULL;
-
-		/* g_message("searching xpath \"%s\"",element->header_info->xpath); */
-
-		ctxt = xmlXPathNewContext(x->doc);
-
-		ctxt->node = x;
-
-		xpath_obj = xmlXPathEval(element->header_info->xpath,
-					 ctxt);
-
-		if (xpath_obj) {
-			result = xmlXPathCastToString(xpath_obj);			
-		} else {
-			result = g_strdup(_("(xpath failed)"));
-		}	
-
-		xmlXPathFreeContext(ctxt);
-		
-		return result;
-	} else if (element->header_info->tagname) {
-		/* Search for a child node matching the tagname: */
-		CongNodePtr i;
-
-		/* g_message("searching for tag <%s>", element->header_info->tagname); */
-		
-		for (i = cong_node_first_child(x); i; i = cong_node_next(i) ) {
-			
-			/* printf("got node named \"%s\"\n", cong_node_name(i)); */			
-			
-			if (0==strcmp(cong_node_name(i), element->header_info->tagname)) {
-				return xml_fetch_clean_data(i);
-			}
-		}
-		
-		/* Not found: */
-		return NULL;
-	} else {
-		return NULL;
-	}
-#endif
 }
 
+/**
+ * cong_dispspec_element_get_section_header_text:
+ * @element:
+ * @x:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gchar*
 cong_dispspec_element_get_section_header_text(CongDispspecElement *element, CongNodePtr x)
 {
@@ -623,8 +747,16 @@ cong_dispspec_element_get_section_header_text(CongDispspecElement *element, Cong
 	}
 }
 
+/**
+ * cong_dispspec_element_get_font:
+ * @element:
+ * @role:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongFont*
-cong_dispspec_element_get_font(CongDispspecElement *element, enum CongFontRole role)
+cong_dispspec_element_get_font(CongDispspecElement *element, CongFontRole role)
 {
 	g_return_val_if_fail(element, NULL);
 	g_return_val_if_fail(role<CONG_FONT_ROLE_NUM, NULL);
@@ -634,6 +766,13 @@ cong_dispspec_element_get_font(CongDispspecElement *element, enum CongFontRole r
 				  role);
 }
 
+/**
+ * cong_dispspec_element_get_editor_service_id:
+ * @element:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 const gchar*
 cong_dispspec_element_get_editor_service_id(CongDispspecElement *element)
 {
@@ -642,6 +781,13 @@ cong_dispspec_element_get_editor_service_id(CongDispspecElement *element)
 	return element->editor_service_id;
 }
 
+/**
+ * cong_dispspec_element_get_property_dialog_service_id:
+ * @element:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 const gchar*
 cong_dispspec_element_get_property_dialog_service_id(CongDispspecElement *element)
 {
@@ -650,8 +796,18 @@ cong_dispspec_element_get_property_dialog_service_id(CongDispspecElement *elemen
 	return element->property_dialog_service_id;
 }
 
+/**
+ * cong_dispspec_element_from_xml:
+ * @ds: the #CongDispspec this will be added to
+ * @xml_element:  the element within the XML document
+ *
+ * Create an #CongDispspecElement from an xds XML representation
+ *
+ * Returns: a newly-created #CongDispspecElement
+ */
 CongDispspecElement*
-cong_dispspec_element_from_xml (xmlNodePtr xml_element)
+cong_dispspec_element_from_xml (CongDispspec *ds,
+				xmlNodePtr xml_element)
 {
 	CongDispspecElement* element;
 
@@ -661,6 +817,8 @@ cong_dispspec_element_from_xml (xmlNodePtr xml_element)
 
 	element = gxx_generated_object_from_xml_tree_fn_dispspec_element (xml_element);
 
+	element->ds = ds;
+
 	if (element->type == CONG_ELEMENT_TYPE_PLUGIN) {
 		xmlChar* id = xmlGetProp(xml_element,"service-id");
 		
@@ -691,7 +849,7 @@ cong_dispspec_element_from_xml (xmlNodePtr xml_element)
   			if (0==strcmp(child->name,"property-dialog")) {
   				DS_DEBUG_MSG1("got property-dialog\n");
 				
-				element->property_dialog_service_id = cong_node_get_attribute(child, "service-id");
+				element->property_dialog_service_id = cong_node_get_attribute(child, NULL, "service-id");
   			}
 
 			/* Handle "key-value-list": */
@@ -705,8 +863,8 @@ cong_dispspec_element_from_xml (xmlNodePtr xml_element)
 						DS_DEBUG_MSG1("got key-value-pair\n");
 						
 						g_hash_table_insert (element->key_value_hash,
-								     cong_node_get_attribute (key_value_iter, "key"),
-								     cong_node_get_attribute (key_value_iter, "value"));
+								     cong_node_get_attribute (key_value_iter, NULL, "key"),
+								     cong_node_get_attribute (key_value_iter, NULL, "value"));
 					}
 				}
 		            	
@@ -738,6 +896,15 @@ cong_dispspec_element_from_xml (xmlNodePtr xml_element)
 	return element;
 }
 
+/**
+ * cong_dispspec_element_to_xml:
+ * @element:  the dispspec element we are adding
+ * @xml_doc:  the XML document we are creating
+ *
+ * Create an element tag suitable for adding to an element-list tag within an xds XML document.
+ *
+ * Returns: a newly-created #xmlNodePtr
+ */
 xmlNodePtr
 cong_dispspec_element_to_xml (const CongDispspecElement *element,
 			      xmlDocPtr xml_doc)
@@ -793,3 +960,20 @@ find_best_string_for_language (GHashTable *hash_of_language)
 {
 	return (const gchar*)find_best_value_for_language (hash_of_language);
 }
+
+CongElementDescription*
+cong_dispspec_element_make_element_description (const CongDispspecElement *ds_element)
+{
+	g_return_val_if_fail (ds_element, NULL);
+
+	return cong_element_description_new (cong_dispspec_element_get_ns_uri (ds_element),
+					     ds_element->local_name);
+}
+
+gboolean
+cong_dispspec_element_should_spellcheck (const CongDispspecElement *ds_element)
+{
+	g_return_val_if_fail (ds_element, FALSE);
+
+	return ds_element->should_spellcheck;
+}
