@@ -24,6 +24,7 @@
 #include "cong-util.h"
 #include "cong-enum-mapping.h"
 #include "cong-vfs.h"
+#include "cong-language.h"
 
 #if 0
 #define DS_DEBUG_MSG1(x)    g_message((x))
@@ -51,13 +52,16 @@ struct CongSerialisationFormat
 
 struct CongExternalDocumentModel
 {
-	enum CongDocumentModelType model_type;
+	CongDocumentModelType model_type;
 	gchar *public_id;
 	gchar *system_id;
 };
 
 struct CongDispspec
 {
+	/* URI of namespace, or NULL: */
+	gchar *ns_uri;
+
 	/* The serialisation formats: */
 	guint num_serialisation_formats;
 	CongSerialisationFormat **serialisation_formats;
@@ -65,18 +69,18 @@ struct CongDispspec
 	/* The CongDocumentModels: */
 	CongExternalDocumentModel *document_models[NUM_CONG_DOCUMENT_MODEL_TYPES];
 
-	/* Implementation is an "intrusive list" of CongDispspecElement structs */ 
-	CongDispspecElement* first;
-	CongDispspecElement* last;
+	GList *list_of_elements;
 
 	/* We have a search tree, indexed by SearchTreeKeys */
 	GTree *search_tree; 
 
-	gchar *name;
-	gchar *desc;
+	CongPerLanguageData *per_lang_name;
+	CongPerLanguageData *per_lang_desc;
 	gchar *filename_extension;
 	GdkPixbuf *icon;
 
+	GList *list_of_typical_prefix; /* list of gchar* */
+
 	xmlNodePtr template;
 };
 
@@ -179,7 +183,7 @@ cong_serialisation_format_new (const gchar *extension)
 }
 
 static CongExternalDocumentModel*
-cong_external_document_model_new (enum CongDocumentModelType model_type,
+cong_external_document_model_new (CongDocumentModelType model_type,
 				  const gchar *public_id,
 				  const gchar *system_id)
 {
@@ -209,7 +213,14 @@ make_model_from_dtd (xmlDtdPtr dtd)
 /* Exported functions: */
 
 /* Barebones constructor, used to implement other ones: */
-CongDispspec* cong_dispspec_new(void)
+/**
+ * cong_dispspec_new:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+CongDispspec* 
+cong_dispspec_new(void)
 {
 	CongDispspec *dispspec;
 	
@@ -223,7 +234,16 @@ CongDispspec* cong_dispspec_new(void)
 }
 
 /* Constructors that use the standard format: */
-GnomeVFSResult cong_dispspec_new_from_xds_file(GnomeVFSURI *uri, CongDispspec** ds)
+/**
+ * cong_dispspec_new_from_xds_file:
+ * @uri:
+ * @ds:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+GnomeVFSResult 
+cong_dispspec_new_from_xds_file(GnomeVFSURI *uri, CongDispspec** ds)
 {
 	char* buffer;
 	GnomeVFSFileSize size;
@@ -245,7 +265,16 @@ GnomeVFSResult cong_dispspec_new_from_xds_file(GnomeVFSURI *uri, CongDispspec**
 	return vfs_result;	
 }
 
-CongDispspec* cong_dispspec_new_from_xds_buffer(const char *buffer, size_t size)
+/**
+ * cong_dispspec_new_from_xds_buffer:
+ * @buffer:
+ * @size:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+CongDispspec* 
+cong_dispspec_new_from_xds_buffer(const char *buffer, size_t size)
 {
 	CongDispspec* ds;
 
@@ -265,9 +294,19 @@ CongDispspec* cong_dispspec_new_from_xds_buffer(const char *buffer, size_t size)
 
 
 /* Constructors that try to generate from another format: */
-CongDispspec* cong_dispspec_new_generate_from_dtd (xmlDtdPtr dtd, 
-						   const gchar *name, 
-						   const gchar *description)
+/**
+ * cong_dispspec_new_generate_from_dtd:
+ * @dtd:
+ * @name:
+ * @description:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+CongDispspec* 
+cong_dispspec_new_generate_from_dtd (xmlDtdPtr dtd, 
+				     const gchar *name, 
+				     const gchar *description)
 {
 	CongDispspec *ds;
 
@@ -275,12 +314,19 @@ CongDispspec* cong_dispspec_new_generate_from_dtd (xmlDtdPtr dtd,
 
 	ds = cong_dispspec_new();
 
+	ds->per_lang_name = cong_per_language_data_new (g_free);
+	ds->per_lang_desc = cong_per_language_data_new (g_free);
+
 	if (name) {
-		ds->name = g_strdup(name);
+		cong_per_language_set_data_for_lang (ds->per_lang_name,
+						     NULL,
+						     g_strdup (name));
 	}
 
 	if (description) {
-		ds->desc = g_strdup(description);
+		cong_per_language_set_data_for_lang (ds->per_lang_desc,
+						     NULL,
+						     g_strdup (description));
 	}
 
 	ds->document_models[CONG_DOCUMENT_MODE_TYPE_DTD] = make_model_from_dtd (dtd);
@@ -291,6 +337,14 @@ CongDispspec* cong_dispspec_new_generate_from_dtd (xmlDtdPtr dtd,
 	return ds;
 }
 
+/**
+ * cong_dispspec_new_generate_from_xml_file:
+ * @doc:
+ * @extension:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongDispspec *
 cong_dispspec_new_generate_from_xml_file (xmlDocPtr doc,
 					  const gchar *extension)
@@ -310,7 +364,14 @@ cong_dispspec_new_generate_from_xml_file (xmlDocPtr doc,
 	return dispspec;
 }
 
-void cong_dispspec_delete (CongDispspec *dispspec)
+/**
+ * cong_dispspec_delete:
+ * @dispspec:
+ *
+ * TODO: Write me
+ */
+void 
+cong_dispspec_delete (CongDispspec *dispspec)
 {
 	/* FIXME: is this causing heap corruption? */
 #if 0
@@ -338,10 +399,21 @@ void cong_dispspec_delete (CongDispspec *dispspec)
 	if (dispspec->icon) {
 		g_object_unref (G_OBJECT(dispspec->icon));
 	}
+	if (dispspec->ns_uri) {
+		g_free (dispspec->ns_uri);
+	}
 #endif
 }
 
-xmlDocPtr cong_dispspec_make_xml(CongDispspec *dispspec)
+/**
+ * cong_dispspec_make_xml:
+ * @dispspec:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+xmlDocPtr 
+cong_dispspec_make_xml(CongDispspec *dispspec)
 {
 	xmlDocPtr xml_doc;
 	CongNodePtr root_node;
@@ -377,12 +449,33 @@ xmlDocPtr cong_dispspec_make_xml(CongDispspec *dispspec)
 					     "element-list",
 					     NULL);			
 		xmlAddChild(root_node, element_list);
+		
+		if (dispspec->ns_uri) {
+			xmlSetProp (element_list, "nsURI", dispspec->ns_uri);
+		}
+
+		/* Typical prefixes: */
+		{
+			GList *iter;
+			for (iter = dispspec->list_of_typical_prefix; iter; iter=iter->next) {
+				const gchar* prefix = (const gchar*)(iter->data);
+				CongNodePtr prefix_node = xmlNewDocNode (xml_doc,
+									 NULL,
+									 "typical-prefix",
+									 NULL);			
+				xmlAddChild (element_list, prefix_node);
+				if (prefix) {
+					xmlSetProp (prefix_node, "prefix", prefix);
+				}
+			}
+		}
 
 		/* The elements: */
 		{
-			CongDispspecElement* element;
+			GList *iter;
+			for (iter = dispspec->list_of_elements; iter; iter=iter->next) {
+				CongDispspecElement *element = (CongDispspecElement *)iter->data;
 			
-			for (element = dispspec->first; element; element = cong_dispspec_element_next (element)) {
 				xmlNodePtr element_as_xml = cong_dispspec_element_to_xml (element,
 											  xml_doc);
 				g_assert (element_as_xml);
@@ -398,31 +491,73 @@ xmlDocPtr cong_dispspec_make_xml(CongDispspec *dispspec)
 }
 
 /* Data for the dispspec: */
+/**
+ * cong_dispspec_get_name:
+ * @ds:
+ *
+ * Get a human-readable, translated name of the dispspec, in the most appropriate language
+ * Returns: the name
+ */
 const gchar*
 cong_dispspec_get_name (const CongDispspec *ds)
 {
-	g_return_val_if_fail(ds,NULL);
+	const gchar *result;
 
-	if (ds->name) {
-		return ds->name;
+	g_return_val_if_fail (ds,NULL);
+
+	result = cong_per_language_get_data (ds->per_lang_name);
+
+	if (result) {
+		return result;
 	} else {
 		return _("unnamed");
 	}
-
 }
 
+/**
+ * cong_dispspec_get_description:
+ * @ds:
+ *
+ * Get a human-readable, translated description of the dispspec, in the most appropriate language
+ * Returns: a string description (we may update this to give HTML or similar at some point)
+ */
 const gchar*
 cong_dispspec_get_description (const CongDispspec *ds)
 {
-	g_return_val_if_fail(ds,NULL);
+	const gchar *result;
 
-	if (ds->desc) {
-		return ds->desc;
+	g_return_val_if_fail (ds,NULL);
+
+	result = cong_per_language_get_data (ds->per_lang_desc);
+
+	if (result) {
+		return result;
 	} else {
 		return _("No description available.");
 	}
 }
 
+/** 
+ * cong_dispspec_get_ns_uri:
+ * @element: the dispspec in question
+ * 
+ * Returns: the namespace URI for this dispspec, or NULL if none
+ */
+const gchar*
+cong_dispspec_get_ns_uri (CongDispspec *dispspec)
+{
+	g_return_val_if_fail (dispspec, NULL);
+
+	return dispspec->ns_uri;
+}
+
+/**
+ * cong_dispspec_get_num_serialisation_formats:
+ * @ds:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 guint
 cong_dispspec_get_num_serialisation_formats (const CongDispspec *ds)
 {
@@ -431,6 +566,14 @@ cong_dispspec_get_num_serialisation_formats (const CongDispspec *ds)
 	return ds->num_serialisation_formats;
 }
 
+/**
+ * cong_dispspec_get_serialisation_format:
+ * @ds:
+ * @index:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 const CongSerialisationFormat*
 cong_dispspec_get_serialisation_format (const CongDispspec *ds,
 					guint index)
@@ -443,6 +586,14 @@ cong_dispspec_get_serialisation_format (const CongDispspec *ds,
 }
 
 /* Returns NULL if it can't find a serialisation format with that extension */
+/**
+ * cong_dispspec_lookup_filename_extension:
+ * @ds:
+ * @extension:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 const CongSerialisationFormat*
 cong_dispspec_lookup_filename_extension (const CongDispspec *ds,
 					 const gchar *extension)
@@ -463,6 +614,14 @@ cong_dispspec_lookup_filename_extension (const CongDispspec *ds,
 }
 
 /* Returns whether the dispspec uses that extension */
+/**
+ * cong_dispspec_matches_filename_extension:
+ * @ds:
+ * @extension:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_dispspec_matches_filename_extension (const CongDispspec *ds,
 					  const gchar *extension)
@@ -473,9 +632,17 @@ cong_dispspec_matches_filename_extension (const CongDispspec *ds,
 							       extension));
 }
 
+/**
+ * cong_dispspec_get_external_document_model:
+ * @ds:
+ * @model_type:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 const CongExternalDocumentModel*
 cong_dispspec_get_external_document_model (const CongDispspec *ds,
-					   enum CongDocumentModelType model_type)
+					   CongDocumentModelType model_type)
 {
 	g_return_val_if_fail(ds, NULL);
 	g_return_val_if_fail(model_type<NUM_CONG_DOCUMENT_MODEL_TYPES, NULL);
@@ -483,6 +650,13 @@ cong_dispspec_get_external_document_model (const CongDispspec *ds,
 	return ds->document_models[model_type];
 }
 
+/**
+ * cong_dispspec_get_icon:
+ * @ds:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 GdkPixbuf*
 cong_dispspec_get_icon(const CongDispspec *ds)
 {
@@ -494,7 +668,44 @@ cong_dispspec_get_icon(const CongDispspec *ds)
 	return ds->icon;
 }
 
+/**
+ * cong_dispspec_for_each_typical_prefix:
+ * @ds:
+ * @callback: function to be called for each typical prefix used by the namespace of this ds (if any)
+ *
+ * Iterates over all prefixes typically used for the namespace of this dispspec (if any).
+ * Note that NULL is a valid value for the prefix, meaning that using the default namespace is typical.
+ */
+void
+cong_dispspec_for_each_typical_prefix (const CongDispspec *ds,
+				       void
+				       (*callback) (const CongDispspec *ds,
+						    const gchar *prefix, /* can be NULL */
+						    gpointer user_data),
+				       gpointer user_data)
+{
+	GList *iter;
+
+	g_return_if_fail (ds);
+	g_return_if_fail (callback);
+	
+	for (iter = ds->list_of_typical_prefix; iter; iter=iter->next) {
+		(*callback) (ds, (const gchar*)(iter->data), user_data);
+	}
+
+}
+
+
 /* Getting at elements within a dispspec */
+/**
+ * cong_dispspec_lookup_element:
+ * @ds:
+ * @ns_uri:
+ * @local_name:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongDispspecElement*
 cong_dispspec_lookup_element (const CongDispspec *ds, 
 			      const gchar* ns_uri, 
@@ -509,6 +720,11 @@ cong_dispspec_lookup_element (const CongDispspec *ds,
 
 	g_assert(ds->search_tree);
 
+	if (!cong_util_ns_uri_equality (ns_uri, ds->ns_uri)) {
+		return FALSE;
+	}
+
+	/* FIXME: we don't need the URI checking per-element anymore: */
 	key.ns_uri = (gchar*)ns_uri;
 	key.local_name = (gchar*)local_name;
 
@@ -517,6 +733,14 @@ cong_dispspec_lookup_element (const CongDispspec *ds,
 	return element;
 }
 
+/**
+ * cong_dispspec_lookup_node:
+ * @ds:
+ * @node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongDispspecElement*
 cong_dispspec_lookup_node(const CongDispspec *ds, CongNodePtr node)
 {
@@ -528,7 +752,15 @@ cong_dispspec_lookup_node(const CongDispspec *ds, CongNodePtr node)
 					     cong_node_get_local_name(node));
 }
 
-enum CongElementType
+/**
+ * cong_dispspec_type:
+ * @ds:
+ * @ns_uri:
+ * @local_name:
+ *
+ * TODO: Write me
+ */
+CongElementType
 cong_dispspec_type (CongDispspec *ds, 
 		    const gchar* ns_uri, 
 		    const gchar* local_name)
@@ -544,76 +776,132 @@ cong_dispspec_type (CongDispspec *ds,
 	return cong_dispspec_element_type(element);
 }
 
+/**
+ * cong_dispspec_get_first_element:
+ * @ds:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongDispspecElement*
 cong_dispspec_get_first_element(CongDispspec *ds)
 {
 	g_return_val_if_fail(ds, NULL);
 
-	return ds->first;
+	if (ds->list_of_elements) {
+		return (CongDispspecElement*)(ds->list_of_elements->data);
+	} else {
+		return NULL;
+	}
 }
 
+/**
+ * cong_dispspec_for_each_element:
+ * @ds:
+ * @callback:
+ * @user_data:
+ *
+ * TODO: Write me
+ */
+void
+cong_dispspec_for_each_element (CongDispspec *ds, 
+				void
+				(*callback) (CongDispspec *ds,
+					     CongDispspecElement *ds_element,
+					     gpointer user_data),
+				gpointer user_data)
+{
+	GList *iter;
+	
+	g_return_if_fail (ds);
+	g_return_if_fail (callback);
+
+	for (iter=ds->list_of_elements; iter; iter=iter->next) {
+		(*callback) (ds,
+			     (CongDispspecElement*)iter->data,
+			     user_data);
+	}
+}
+
+
 /* Manipulating a dispspec: */
-void cong_dispspec_add_element (CongDispspec* ds, 
-				CongDispspecElement* element)
+/**
+ * cong_dispspec_add_element:
+ * @ds:
+ * @element:
+ *
+ * TODO: Write me
+ */
+void 
+cong_dispspec_add_element (CongDispspec* ds, 
+			   CongDispspecElement* element)
 {
 	struct SearchTreeKey *key;
 
 	g_return_if_fail(ds);
 	g_return_if_fail(element);
 
-	g_assert(element->next==NULL);
-	g_assert(element->local_name);
-
-	if (ds->first) {
-		g_assert(ds->last);
-		ds->last->next = element;
-	} else {
-		ds->first = element;
-	}
-
-	ds->last = element;
+	ds->list_of_elements = g_list_append (ds->list_of_elements,
+					      element);
 
 	key = g_new0(struct SearchTreeKey, 1);
-	if (element->ns_uri) {
-		key->ns_uri = g_strdup (element->ns_uri);
+	if (cong_dispspec_element_get_ns_uri (element)) {
+		key->ns_uri = g_strdup (cong_dispspec_element_get_ns_uri (element));
 	}
-	key->local_name = element->local_name;
+	g_assert (cong_dispspec_element_get_local_name (element));
+	key->local_name = g_strdup (cong_dispspec_element_get_local_name (element));
+	/* FIXME: does this leak? */
 
 	g_tree_insert(ds->search_tree, key, element);
 }
 
+/**
+ * cong_dispspec_get_num_elements:
+ * @ds:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 guint
 cong_dispspec_get_num_elements (CongDispspec *ds)
 {
 	guint count;
-	CongDispspecElement *ds_iter;
+	GList *iter;
 
 	g_return_val_if_fail (ds, 0);
 
 	/* o(n): */
 
 	count = 0;
-	for (ds_iter = ds->first; ds_iter; ds_iter=ds_iter->next) {
+	for (iter = ds->list_of_elements; iter; iter=iter->next) {
 		count++;
 	}
 	return count;	
 }
 
+/**
+ * cong_dispspec_get_element:
+ * @ds:
+ * @index:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongDispspecElement*
 cong_dispspec_get_element (CongDispspec *ds,
 			   guint index)
 {
 	guint count;
-	CongDispspecElement *ds_iter;
+	GList *iter;
 
 	g_return_val_if_fail (ds, NULL);
 
 	/* o(n): */
 
 	count = 0;
-	for (ds_iter = ds->first; ds_iter; ds_iter=ds_iter->next) {
+	for (iter = ds->list_of_elements; iter; iter=iter->next) {
 		if (count==index) {
-			return ds_iter;
+			return (CongDispspecElement*)iter->data;
 		}
 
 		count++;
@@ -624,7 +912,7 @@ cong_dispspec_get_element (CongDispspec *ds,
 /* Various functions that may get deprecated at some point: */
 #if NEW_LOOK
 #if 0
-GdkGC *cong_dispspec_gc_get(CongDispspec *ds, CongNodePtr x, enum CongDispspecGCUsage usage)
+GdkGC *cong_dispspec_gc_get(CongDispspec *ds, CongNodePtr x, CongDispspecGCUsage usage)
 {
 	CongDispspecElement* element = cong_dispspec_lookup_element(ds, );
 
@@ -636,7 +924,17 @@ GdkGC *cong_dispspec_gc_get(CongDispspec *ds, CongNodePtr x, enum CongDispspecGC
 }
 #endif
 #else
-GdkGC *cong_dispspec_name_gc_get(CongDispspec *ds, CongNodePtr t, int tog)
+/**
+ * cong_dispspec_name_gc_get:
+ * @ds:
+ * @t:
+ * @tog:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+GdkGC *
+cong_dispspec_name_gc_get(CongDispspec *ds, CongNodePtr t, int tog)
 {
 	CongDispspecElement* element = cong_dispspec_lookup_element(ds, cong_node_name(t));
 
@@ -647,8 +945,8 @@ GdkGC *cong_dispspec_name_gc_get(CongDispspec *ds, CongNodePtr t, int tog)
 	}
 }
 
-
-GdkGC *cong_dispspec_gc_get(CongDispspec *ds, CongNodePtr x, int tog)
+GdkGC *
+cong_dispspec_gc_get(CongDispspec *ds, CongNodePtr x, int tog)
 {
 	CongDispspecElement* element = cong_dispspec_lookup_element(ds, xml_frag_name_nice(x));
 
@@ -660,6 +958,14 @@ GdkGC *cong_dispspec_gc_get(CongDispspec *ds, CongNodePtr x, int tog)
 }
 #endif
 
+/**
+ * cong_dispspec_name_get:
+ * @ds:
+ * @node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 const gchar*
 cong_dispspec_name_get (CongDispspec *ds, 
 			CongNodePtr node)
@@ -691,12 +997,21 @@ char *cong_dispspec_name_name_get(CongDispspec *ds, TTREE *t)
 #endif
 
 #if 1
+/**
+ * cong_dispspec_element_structural:
+ * @ds:
+ * @ns_uri:
+ * @local_name:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean 
 cong_dispspec_element_structural (CongDispspec *ds, 
 				  const gchar *ns_uri, 
-				  const char *name)
+				  const char *local_name)
 {
-	CongDispspecElement* element = cong_dispspec_lookup_element (ds, ns_uri, name);
+	CongDispspecElement* element = cong_dispspec_lookup_element (ds, ns_uri, local_name);
 
 	if (NULL==element) {
 		return FALSE;
@@ -705,13 +1020,21 @@ cong_dispspec_element_structural (CongDispspec *ds,
 	return (CONG_ELEMENT_TYPE_STRUCTURAL == cong_dispspec_element_type(element));
 }
 
-
+/**
+ * cong_dispspec_element_collapse:
+ * @ds:
+ * @ns_uri:
+ * @local_name:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean 
 cong_dispspec_element_collapse (CongDispspec *ds, 
 				const gchar *ns_uri, 
-				const char *name)
+				const char *local_name)
 {
-	CongDispspecElement* element = cong_dispspec_lookup_element (ds, ns_uri, name);
+	CongDispspecElement* element = cong_dispspec_lookup_element (ds, ns_uri, local_name);
 
 	if (NULL==element) {
 		return FALSE;
@@ -720,13 +1043,21 @@ cong_dispspec_element_collapse (CongDispspec *ds,
 	return cong_dispspec_element_collapseto(element);
 }
 
-
+/**
+ * cong_dispspec_element_span:
+ * @ds:
+ * @ns_uri:
+ * @local_name:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_dispspec_element_span (CongDispspec *ds, 
 			    const gchar *ns_uri, 
-			    const char *name)
+			    const char *local_name)
 {
-	CongDispspecElement* element = cong_dispspec_lookup_element (ds, ns_uri, name);
+	CongDispspecElement* element = cong_dispspec_lookup_element (ds, ns_uri, local_name);
 
 	if (NULL==element) {
 		return FALSE;
@@ -735,13 +1066,21 @@ cong_dispspec_element_span (CongDispspec *ds,
 	return (CONG_ELEMENT_TYPE_SPAN == cong_dispspec_element_type(element));
 }
 
-
+/**
+ * cong_dispspec_element_insert:
+ * @ds:
+ * @ns_uri:
+ * @local_name:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean 
 cong_dispspec_element_insert (CongDispspec *ds, 
 			      const gchar *ns_uri, 
-			      const char *name)
+			      const char *local_name)
 {
-	CongDispspecElement* element = cong_dispspec_lookup_element(ds, ns_uri, name);
+	CongDispspecElement* element = cong_dispspec_lookup_element(ds, ns_uri, local_name);
 
 	if (NULL==element) {
 		return FALSE;
@@ -813,7 +1152,8 @@ element_callback_generate_dispspec_from_dtd (void *payload, void *ds, xmlChar *
 	g_assert (dispspec);
 	g_assert (element);
 
-	ds_element = cong_dispspec_element_new (element->prefix,
+	ds_element = cong_dispspec_element_new (dispspec,
+						/* element->prefix, */
 						name,
 						cong_dtd_element_guess_dispspec_type (element),
 						TRUE);
@@ -832,34 +1172,43 @@ static CongDispspec* parse_xmldoc(xmlDocPtr doc)
 		xmlNodePtr xml_dispspec;
 
 		for (xml_dispspec=doc->children; xml_dispspec; xml_dispspec = xml_dispspec->next) {
-			if (0==strcmp(xml_dispspec->name,"dispspec")) {
+			if (cong_node_is_element (xml_dispspec, NULL, "dispspec")) {
 
 				xmlNodePtr cur;
 				
 				DS_DEBUG_MSG1("got dispspec\n");
 
 				for (cur = xml_dispspec->children; cur; cur=cur->next) {
-					if (0==strcmp(cur->name,"element-list")) {
+					if (cong_node_is_element (cur, NULL, "element-list")) {
 						
 						xmlNodePtr xml_element;
 						DS_DEBUG_MSG1("got element-list\n");
 
+						ds->ns_uri = cong_node_get_attribute (cur, NULL, "nsURI");
+
 						for (xml_element = cur->children; xml_element; xml_element=xml_element->next) {
-							if(xml_element->type==XML_ELEMENT_NODE)
-							{
-								CongDispspecElement* element = cong_dispspec_element_from_xml (xml_element);
+							if (cong_node_is_element ( xml_element, NULL, "typical-prefix")) {
+								gchar *attr = cong_node_get_attribute (xml_element, NULL, "prefix");
+
+								ds->list_of_typical_prefix = g_list_append (ds->list_of_typical_prefix,
+													    attr);
+							}
+
+							if (cong_node_is_element ( xml_element, NULL, "element")) {
+								CongDispspecElement* element = cong_dispspec_element_from_xml (ds,
+															       xml_element);
 								
 								cong_dispspec_add_element(ds,element);
 							}
 						}
 						
-					} else if (0==strcmp(cur->name,"metadata")) {
+					} else if (cong_node_is_element (cur, NULL, "metadata")) {
 						parse_metadata(ds, doc, cur);
-					} else if  (0==strcmp(cur->name,"serialisation")) {
+					} else if  (cong_node_is_element (cur, NULL, "serialisation")) {
 						parse_serialisation (ds, doc, cur);
-					} else if (0==strcmp(cur->name,"document-models")) {
+					} else if (cong_node_is_element (cur, NULL, "document-models")) {
 						parse_document_models(ds, doc, cur);
-					} else if (0==strcmp(cur->name, "document-template")) {
+					} else if (cong_node_is_element (cur, NULL,  "document-template")) {
 						parse_template(ds, cur);
 					}
 				}
@@ -871,27 +1220,72 @@ static CongDispspec* parse_xmldoc(xmlDocPtr doc)
 }
 
 
+/* Create a generic "per-lang" container of gpointers with a GDestroyFunc, and an iterator so that you can easily write to/from XML functions... first step towards killing GXX as well... */
+#if 0
+/**
+ * cong_node_get_child_for_lang:
+ * @node: the parent node
+ *
+ * Search through the children of the given node, looking for the node with the most appropriate xml:lang 
+ * based upon the language.
+ *
+ * Returns: the most appropriate child node, or NULL if no child of the correct language exists
+ */
+CongNodePtr
+cong_node_get_child_for_lang (CongNodePtr node)
+{
+	CongNodePtr node_iter;
+
+	g_return_val_if_fail (node, NULL);
+
+	for (node_iter = node->children; node_iter; node_iter=node_iter->next) {
+
+		if (cong_node_type (node_iter)==CONG_NODE_ELEMENT) {
+		}
+	}
+	return NULL;
+}
+#endif
+
+static gpointer
+get_content_string_cb (xmlDocPtr xml_doc, CongNodePtr node)
+{
+#if 0
+#error
+	xmlChar* str = xmlNodeListGetString (xml_doc, node, 1);
+	if (str) {
+		return g_strdup (str);
+	} else {
+		return NULL;
+	}
+#else
+	/* FIXME: assumes only a single TEXT child, so will break in the presence of comments etc */
+	if (node->children) {
+		if (cong_node_type (node->children)==CONG_NODE_TYPE_TEXT) {
+			return g_strdup (node->children->content);
+		}
+	}
+	return NULL;
+#endif
+}
+
 static void
 parse_metadata(CongDispspec *ds, xmlDocPtr doc, xmlNodePtr node)
 {
-	xmlNodePtr xml_element;
 	DS_DEBUG_MSG1("got metadata\n");
-	
-	for (xml_element = node->children; xml_element; xml_element=xml_element->next) {
-		if (0==strcmp(xml_element->name,"name")) {
-			xmlChar* str = xmlNodeListGetString(doc, xml_element->xmlChildrenNode, 1);
-			if (str) {
-				ds->name = g_strdup(str);
-			}
-		}
 
-		if (0==strcmp(xml_element->name,"description")) {
-			xmlChar* str = xmlNodeListGetString(doc, xml_element->xmlChildrenNode, 1);
-			if (str) {
-				ds->desc = g_strdup(str);
-			}
-		}
-	}
+	ds->per_lang_name = cong_per_language_data_new_from_xml (doc,
+								 node,
+								 NULL,
+								 "name",
+								 get_content_string_cb,
+								 g_free);
+	ds->per_lang_desc = cong_per_language_data_new_from_xml (doc,
+								 node,
+								 NULL,
+								 "description",
+								 get_content_string_cb,
+								 g_free);
 }
 
 
@@ -964,10 +1358,10 @@ parse_external_document_model (CongDispspec *ds,
 	gchar *type;
 
 	DS_DEBUG_MSG1("got external-document-model\n");
-	type = cong_node_get_attribute (node, "type");
+	type = cong_node_get_attribute (node, NULL, "type");
 
 	if (type) {
-		enum CongDocumentModelType model_type = cong_enum_mapping_lookup (document_model_enum_mapping,
+		CongDocumentModelType model_type = cong_enum_mapping_lookup (document_model_enum_mapping,
 										  sizeof(document_model_enum_mapping)/sizeof(CongEnumMapping),
 										  "type",
 										  CONG_DOCUMENT_MODE_TYPE_DTD);
@@ -1001,13 +1395,28 @@ parse_template(CongDispspec *ds, xmlNodePtr node)
 	}
 }
 	
-void col_to_gcol(GdkColor *gcol, unsigned int col)
+/**
+ * col_to_gcol:
+ * @gcol:
+ * @col:
+ *
+ * TODO: Write me
+ */
+void 
+col_to_gcol(GdkColor *gcol, unsigned int col)
 {
   gcol->blue = (col << 8) & 0xff00;
   gcol->green = (col & 0xff00);
   gcol->red = (col >> 8) & 0xff00;
 }
 
+/**
+ * cong_external_document_model_get_public_id:
+ * @model:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 const gchar*
 cong_external_document_model_get_public_id (const CongExternalDocumentModel* model)
 {
@@ -1016,6 +1425,13 @@ cong_external_document_model_get_public_id (const CongExternalDocumentModel* mod
 	return model->public_id;
 }
 
+/**
+ * cong_external_document_model_get_system_id:
+ * @model:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 const gchar*
 cong_external_document_model_get_system_id (const CongExternalDocumentModel* model)
 {
@@ -1024,6 +1440,26 @@ cong_external_document_model_get_system_id (const CongExternalDocumentModel* mod
 	return model->system_id;
 }
 
+static CongNodePtr 
+make_name_node_callback (xmlDocPtr xml_doc,
+			 gpointer data)
+{
+	return cong_node_new_element_full_with_content (xml_doc, 
+							NULL,
+							"name",
+							data);
+}
+
+static CongNodePtr 
+make_desc_node_callback (xmlDocPtr xml_doc,
+			 gpointer data)
+{
+	return cong_node_new_element_full_with_content (xml_doc, 
+							NULL,
+							"description",
+							data);
+}
+
 /* Subroutines for converting a CongDispspec to XML XDS: */
 static void add_xml_for_metadata (xmlDocPtr xml_doc, 
 				  CongNodePtr root, 
@@ -1036,24 +1472,13 @@ static void add_xml_for_metadata (xmlDocPtr xml_doc,
 				 "metadata",
 				 NULL);			
 	xmlAddChild(root, metadata);
-	
-	if (dispspec->name) {
-		
-		xmlAddChild (metadata, 
-			     xmlNewDocRawNode (xml_doc,
-					       NULL,
-					       "name",
-					       dispspec->name)
-			     ); 
-	}
-	if (dispspec->desc) {
-		xmlAddChild (metadata, 
-			     xmlNewDocRawNode (xml_doc,
-					       NULL,
-					       "description",
-					       dispspec->desc)
-			     );
-	}
+
+	cong_per_language_data_to_xml (dispspec->per_lang_name,
+				       metadata,
+				       make_name_node_callback);
+	cong_per_language_data_to_xml (dispspec->per_lang_desc,
+				       metadata,
+				       make_desc_node_callback);
 	
 	/* FIXME: we can't yet save the icon name */
 
@@ -1101,7 +1526,7 @@ add_xml_for_document_models (xmlDocPtr xml_doc,
 		     node_document_models);
 	
 	for (i=0; i<NUM_CONG_DOCUMENT_MODEL_TYPES; i++) {
-		enum CongDocumentModelType model_type = (enum CongDocumentModelType)i;
+		CongDocumentModelType model_type = (CongDocumentModelType)i;
 
 		const CongExternalDocumentModel* model = cong_dispspec_get_external_document_model (dispspec,
 												    model_type);
@@ -1178,7 +1603,7 @@ promote_element (CongDispspec * dispspec,
 		{
 			if (contains_carriage_return(xmlNodeGetContent (node)))
 			{
-				CongDispspecElement *ds_element = cong_dispspec_element_new (cong_node_get_ns_uri (node->parent),
+				CongDispspecElement *ds_element = cong_dispspec_element_new (dispspec,
 											     node->parent->name,
 											     CONG_ELEMENT_TYPE_STRUCTURAL,
 											     TRUE);
@@ -1191,7 +1616,7 @@ promote_element (CongDispspec * dispspec,
 		{
 			if (contains_text (xmlNodeGetContent (node)))
 			{
-				CongDispspecElement *ds_element = cong_dispspec_element_new (cong_node_get_ns_uri (node->parent),
+				CongDispspecElement *ds_element = cong_dispspec_element_new (dispspec,
 											     node->parent->name,
 											     CONG_ELEMENT_TYPE_STRUCTURAL,
 											     TRUE);
@@ -1219,14 +1644,14 @@ handle_elements_from_xml (CongDispspec * dispspec, xmlNodePtr cur)
 				}
 				else if (contains_text (xmlNodeGetContent (cur))) {
 					if (contains_carriage_return(xmlNodeGetContent (cur))) {
-						CongDispspecElement *ds_element = cong_dispspec_element_new (cong_node_get_ns_uri (cur->parent),
+						CongDispspecElement *ds_element = cong_dispspec_element_new (dispspec,
 													     cur->parent->name,
 													     CONG_ELEMENT_TYPE_STRUCTURAL,
 													     TRUE);
 						g_assert (ds_element);
 						cong_dispspec_add_element (dispspec, ds_element);
 					} else {
-						CongDispspecElement *ds_element = cong_dispspec_element_new (cong_node_get_ns_uri (cur->parent),
+						CongDispspecElement *ds_element = cong_dispspec_element_new (dispspec,
 													     cur->parent->name,
 													     CONG_ELEMENT_TYPE_SPAN,
 													     TRUE);
@@ -1234,7 +1659,7 @@ handle_elements_from_xml (CongDispspec * dispspec, xmlNodePtr cur)
 						cong_dispspec_add_element (dispspec, ds_element);
 					}
 				} else {
-					CongDispspecElement *ds_element = cong_dispspec_element_new (cong_node_get_ns_uri (cur->parent),
+					CongDispspecElement *ds_element = cong_dispspec_element_new (dispspec,
 												     cur->parent->name,
 												     CONG_ELEMENT_TYPE_STRUCTURAL,
 												     TRUE);
@@ -1268,7 +1693,7 @@ ensure_all_elements_covered (CongDispspec * dispspec, xmlNodePtr cur)
 		element = cong_dispspec_lookup_element (dispspec, cong_node_get_ns_uri (cur), cong_node_get_local_name (cur));
 		if (NULL==element) {
 			/* Then we've found an element that doesn't have any handler in the dispspec; better create a structural tag for it... */
-			CongDispspecElement *ds_element = cong_dispspec_element_new (cong_node_get_ns_uri (cur),
+			CongDispspecElement *ds_element = cong_dispspec_element_new (dispspec,
 										     cong_node_get_local_name (cur),
 										     CONG_ELEMENT_TYPE_STRUCTURAL,
 										     TRUE);
@@ -1307,7 +1732,7 @@ xml_to_dispspec (CongDispspec *dispspec,
 	{
 		xmlNodePtr root_element = xmlDocGetRootElement(doc);
 
-		CongDispspecElement *ds_element = cong_dispspec_element_new (cong_node_get_ns_uri (root_element),
+		CongDispspecElement *ds_element = cong_dispspec_element_new (dispspec,
 									     root_element->name,
 									     CONG_ELEMENT_TYPE_STRUCTURAL,
 									     TRUE);
@@ -1324,7 +1749,9 @@ xml_to_dispspec (CongDispspec *dispspec,
 
 	if (doc)
 	{
-		dispspec->name = g_strdup(doc->URL);
+		/* FIXME: reinstate this code? */
+		/* dispspec->name = g_strdup(doc->URL); */
+
 		handle_elements_from_xml (dispspec, xmlDocGetRootElement (doc));
 		ensure_all_elements_covered(dispspec, xmlDocGetRootElement (doc));
 	}
@@ -1348,7 +1775,13 @@ load_dtd (xmlDocPtr doc)
 	return NULL;
 }
 
-
+/**
+ * cong_dispspec_get_template:
+ * @ds:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 xmlNodePtr
 cong_dispspec_get_template(const CongDispspec *ds)
 {
@@ -1371,21 +1804,39 @@ calc_coverage_recursive (const CongDispspec *ds,
 	g_assert (node);
 
 	if (node->type == XML_ELEMENT_NODE) {
-		coverage_data->total_elements++;
-
-		if (NULL!=cong_dispspec_lookup_node (ds, node)) {
-			coverage_data->covered_elements++;
+		/* Only deal with elements outside a namespace: */
+		if (NULL==node->ns) {
+			coverage_data->total_elements++;
+			
+			if (NULL!=cong_dispspec_lookup_node (ds, node)) {
+				coverage_data->covered_elements++;
+			}
 		}
 	}
 
 	/* FIXME:  this gives slightly skewed results for entities */
-	for (iter = node->children; iter; iter=iter->next) {
-		calc_coverage_recursive (ds,
-					 iter,
+	if (node->type==XML_ENTITY_REF_NODE) {
+		calc_coverage_recursive (ds, 
+					 node->children,
 					 coverage_data);
+	} else {						  
+		for (iter = node->children; iter; iter=iter->next) {
+			calc_coverage_recursive (ds,
+						 iter,
+						 coverage_data);
+		}
 	}
 }
 
+/**
+ * cong_dispspec_calculate_coverage:
+ * @ds: a dispspec
+ * @xml_doc: a document
+ *
+ * Calculate the proportion of elements without namespaces in the document that
+ * are covered by this dispspec.
+ * Returns: a fraction between 0.0 and 1.0.  If all elements have namespaces, 0.0 is returned
+ */
 gdouble
 cong_dispspec_calculate_coverage (const CongDispspec *ds,
 				  xmlDocPtr xml_doc)
