@@ -207,6 +207,14 @@ cong_editor_area_instance_init (CongEditorArea *area)
 	area->private = g_new0(CongEditorAreaDetails,1);
 }
 
+/**
+ * cong_editor_area_construct:
+ * @area:
+ * @editor_widget:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorArea*
 cong_editor_area_construct (CongEditorArea *area,
 			    CongEditorWidget3* editor_widget)
@@ -230,7 +238,13 @@ cong_editor_area_construct (CongEditorArea *area,
 	return area;
 }
 
-
+/**
+ * cong_editor_area_get_widget:
+ * @area:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorWidget3*
 cong_editor_area_get_widget (CongEditorArea *area)
 {
@@ -239,6 +253,13 @@ cong_editor_area_get_widget (CongEditorArea *area)
 	return PRIVATE(area)->editor_widget;
 }
 
+/**
+ * cong_editor_area_get_document:
+ * @area:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongDocument*
 cong_editor_area_get_document (CongEditorArea *area)
 {
@@ -247,14 +268,25 @@ cong_editor_area_get_document (CongEditorArea *area)
 	return cong_editor_widget3_get_document (cong_editor_area_get_widget (area));
 }
 
-
+/**
+ * cong_editor_area_is_hidden:
+ * @area:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean 
 cong_editor_area_is_hidden (CongEditorArea *area)
 {
 	return PRIVATE(area)->is_hidden;
 }
 
-
+/**
+ * cong_editor_area_show:
+ * @area:
+ *
+ * TODO: Write me
+ */
 void
 cong_editor_area_show (CongEditorArea *area)
 {
@@ -262,6 +294,12 @@ cong_editor_area_show (CongEditorArea *area)
 	/* FIXME: do we need to emit any events? */
 }
 
+/**
+ * cong_editor_area_hide:
+ * @area:
+ *
+ * TODO: Write me
+ */
 void
 cong_editor_area_hide (CongEditorArea *area)
 {
@@ -269,6 +307,13 @@ cong_editor_area_hide (CongEditorArea *area)
 	/* FIXME: do we need to emit any events? */
 }
 
+/**
+ * cong_editor_area_get_state:
+ * @editor_area:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 GtkStateType
 cong_editor_area_get_state (CongEditorArea *area)
 {
@@ -277,6 +322,13 @@ cong_editor_area_get_state (CongEditorArea *area)
 	return PRIVATE(area)->state;
 }
 
+/**
+ * cong_editor_area_set_state:
+ * @editor_area:
+ * @state:
+ *
+ * TODO: Write me
+ */
 void
 cong_editor_area_set_state (CongEditorArea *area,
 			    GtkStateType state)
@@ -291,6 +343,13 @@ cong_editor_area_set_state (CongEditorArea *area,
 	}
 }
 
+/**
+ * cong_editor_area_get_cursor:
+ * @area:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 GdkCursor*
 cong_editor_area_get_cursor (CongEditorArea *area)
 {
@@ -302,6 +361,13 @@ cong_editor_area_get_cursor (CongEditorArea *area)
 	}
 }
 
+/**
+ * cong_editor_area_set_cursor:
+ * @area:
+ * @cursor:
+ *
+ * TODO: Write me
+ */
 void
 cong_editor_area_set_cursor (CongEditorArea *area,
 			     GdkCursor *cursor)
@@ -319,8 +385,14 @@ cong_editor_area_set_cursor (CongEditorArea *area,
 	}
 }
 
-
 #if 1
+/**
+ * cong_editor_area_get_window_coords:
+ * @area:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 const GdkRectangle*
 cong_editor_area_get_window_coords (CongEditorArea *area)
 {
@@ -330,7 +402,15 @@ cong_editor_area_get_window_coords (CongEditorArea *area)
 }
 #endif
 
-
+/**
+ * cong_editor_area_get_requisition:
+ * @area:
+ * @orientation:
+ * @width_hint:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 guint
 cong_editor_area_get_requisition (CongEditorArea *area,
 				  GtkOrientation orientation,
@@ -376,6 +456,14 @@ cong_editor_area_get_requisition (CongEditorArea *area,
 	return cache->last_calculated_requisition;
 }
 
+/**
+ * cong_editor_area_get_requisition_width:
+ * @area:
+ * @width_hint:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gint
 cong_editor_area_get_requisition_width (CongEditorArea *area,
 					int width_hint)
@@ -386,6 +474,14 @@ cong_editor_area_get_requisition_width (CongEditorArea *area,
 
 }
 
+/**
+ * cong_editor_area_get_requisition_height:
+ * @area:
+ * @width_hint:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gint
 cong_editor_area_get_requisition_height (CongEditorArea *area,
 					 int width_hint)
@@ -395,6 +491,14 @@ cong_editor_area_get_requisition_height (CongEditorArea *area,
 						 width_hint);
 }
 
+/**
+ * cong_editor_area_get_cached_requisition:
+ * @area:
+ * @orientation:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gint
 cong_editor_area_get_cached_requisition (CongEditorArea *area,
 					 GtkOrientation orientation)
@@ -408,7 +512,6 @@ cong_editor_area_get_cached_requisition (CongEditorArea *area,
 	return cache->last_calculated_requisition;
 }
 
-
 #if 0
 void 
 cong_editor_area_set_requisition (CongEditorArea *area,
@@ -422,6 +525,13 @@ cong_editor_area_set_requisition (CongEditorArea *area,
 }
 #endif
 
+/**
+ * cong_editor_area_debug_render_area:
+ * @area:
+ * @gc:
+ *
+ * TODO: Write me
+ */
 void 
 cong_editor_area_debug_render_area (CongEditorArea *area,
 				    GdkGC *gc)
@@ -438,6 +548,12 @@ cong_editor_area_debug_render_area (CongEditorArea *area,
 			    PRIVATE(area)->window_area.height-1);
 }
 
+/**
+ * cong_editor_area_debug_render_state:
+ * @area:
+ *
+ * TODO: Write me
+ */
 void
 cong_editor_area_debug_render_state (CongEditorArea *area)
 {
@@ -472,6 +588,13 @@ cong_editor_area_debug_render_state (CongEditorArea *area)
 
 
 /* CongEditorArea methods: */
+/**
+ * cong_editor_area_get_parent:
+ * @area:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorArea*
 cong_editor_area_get_parent (CongEditorArea *area)
 {
@@ -490,6 +613,13 @@ do_recursive_render (CongEditorArea *area,
 	return FALSE;
 }
 
+/**
+ * cong_editor_area_recursive_render:
+ * @area:
+ * @widget_rect:
+ *
+ * TODO: Write me
+ */
 void
 cong_editor_area_recursive_render (CongEditorArea *area,
 				   const GdkRectangle *widget_rect)
@@ -539,6 +669,14 @@ cong_editor_area_recursive_render (CongEditorArea *area,
 	}
 }
 
+/**
+ * cong_editor_area_on_button_press:
+ * @editor_area:
+ * @event:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_editor_area_on_button_press (CongEditorArea *editor_area, 
 				  GdkEventButton *event)
@@ -556,6 +694,14 @@ cong_editor_area_on_button_press (CongEditorArea *editor_area,
 	return result;
 }
 
+/**
+ * cong_editor_area_on_motion_notify:
+ * @editor_area:
+ * @event:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_editor_area_on_motion_notify (CongEditorArea *editor_area, 
 				   GdkEventMotion *event)
@@ -573,6 +719,14 @@ cong_editor_area_on_motion_notify (CongEditorArea *editor_area,
 	return result;
 }
 
+/**
+ * cong_editor_area_on_key_press:
+ * @editor_area:
+ * @event:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_editor_area_on_key_press (CongEditorArea *editor_area, 
 			       GdkEventKey *event)
@@ -590,19 +744,42 @@ cong_editor_area_on_key_press (CongEditorArea *editor_area,
 	return result;
 }
 
+/**
+ * cong_editor_area_calc_requisition:
+ * @editor_area:
+ * @orientation:
+ * @width_hint:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 guint
 cong_editor_area_calc_requisition (CongEditorArea *editor_area, 
 				   GtkOrientation orientation,
 				   int width_hint)
 {
 	g_return_val_if_fail (editor_area, 0);
-
-	return CONG_EEL_CALL_METHOD_WITH_RETURN_VALUE (CONG_EDITOR_AREA_CLASS,
+	
+	if (PRIVATE(editor_area)->is_hidden) {
+	        return 0;
+        } else {
+		return CONG_EEL_CALL_METHOD_WITH_RETURN_VALUE (CONG_EDITOR_AREA_CLASS,
 						       editor_area,
 						       calc_requisition, 
 						       (editor_area, orientation, width_hint));
+        }						       
 }
 
+/**
+ * cong_editor_area_set_allocation:
+ * @editor_area:
+ * @x:
+ * @y:
+ * @width:
+ * @height:
+ *
+ * TODO: Write me
+ */
 void 
 cong_editor_area_set_allocation (CongEditorArea *editor_area,
 				 gint x,
@@ -654,7 +831,12 @@ cong_editor_area_set_allocation (CongEditorArea *editor_area,
 	}
 }
 
-
+/**
+ * cong_editor_area_queue_redraw:
+ * @editor_area:
+ *
+ * TODO: Write me
+ */
 void
 cong_editor_area_queue_redraw (CongEditorArea *editor_area)
 {
@@ -671,6 +853,13 @@ cong_editor_area_queue_redraw (CongEditorArea *editor_area)
 				    rect->height);
 }
 
+/**
+ * cong_editor_area_flush_requisition_cache:
+ * @editor_area:
+ * @orientation:
+ *
+ * TODO: Write me
+ */
 void
 cong_editor_area_flush_requisition_cache (CongEditorArea *editor_area,
 					  GtkOrientation orientation)
@@ -698,6 +887,15 @@ cong_editor_area_flush_requisition_cache (CongEditorArea *editor_area,
 	}
 }
 
+/**
+ * cong_editor_area_for_all:
+ * @editor_area:
+ * @func:
+ * @user_data:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorArea*
 cong_editor_area_for_all (CongEditorArea *editor_area, 
 			  CongEditorAreaCallbackFunc func, 
@@ -777,6 +975,15 @@ cong_editor_area_recurse (CongEditorArea *editor_area,
 }
 #endif
 
+/**
+ * cong_editor_area_covers_xy:
+ * @editor_area:
+ * @x:
+ * @y:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_editor_area_covers_xy (CongEditorArea *editor_area, 
 			    gint x,
@@ -804,6 +1011,15 @@ static gboolean is_area_at_xy (CongEditorArea *editor_area,
 }
 
 /* Function gets immediate child (either "internal" or "non-internal") at the coords, if any: */
+/**
+ * cong_editor_area_get_immediate_child_at:
+ * @area:
+ * @x:
+ * @y:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorArea*
 cong_editor_area_get_immediate_child_at (CongEditorArea *area,
 					 gint x,
@@ -818,6 +1034,16 @@ cong_editor_area_get_immediate_child_at (CongEditorArea *area,
 					 is_area_at_xy,
 					 &search);
 }
+
+/**
+ * cong_editor_area_get_deepest_child_at:
+ * @area:
+ * @x:
+ * @y:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorArea*
 cong_editor_area_get_deepest_child_at (CongEditorArea *area,
 				       gint x,
@@ -849,6 +1075,13 @@ cong_editor_area_get_deepest_child_at (CongEditorArea *area,
 	return NULL;
 }
 
+/**
+ * cong_editor_area_get_gdk_window:
+ * @editor_area:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 GdkWindow*
 cong_editor_area_get_gdk_window(CongEditorArea *editor_area)
 {
@@ -858,6 +1091,13 @@ cong_editor_area_get_gdk_window(CongEditorArea *editor_area)
 }
 
 /* Associate the editor area with a particular editor node: */
+/**
+ * cong_editor_area_connect_node_signals:
+ * @area:
+ * @editor_node:
+ *
+ * TODO: Write me
+ */
 void
 cong_editor_area_connect_node_signals (CongEditorArea *area,
 				       CongEditorNode *editor_node)
@@ -890,6 +1130,13 @@ cong_editor_area_connect_node_signals (CongEditorArea *area,
 }
 
 /* Protected stuff: */
+/**
+ * cong_editor_area_protected_postprocess_add_internal_child:
+ * @area:
+ * @internal_child:
+ *
+ * TODO: Write me
+ */
 void
 cong_editor_area_protected_postprocess_add_internal_child (CongEditorArea *area,
 							   CongEditorArea *internal_child)
@@ -903,6 +1150,13 @@ cong_editor_area_protected_postprocess_add_internal_child (CongEditorArea *area,
 			  area);
 }
 
+/**
+ * cong_editor_area_protected_set_parent:
+ * @area:
+ * @parent:
+ *
+ * TODO: Write me
+ */
 void
 cong_editor_area_protected_set_parent (CongEditorArea *area,
 				       CongEditorArea *parent)
@@ -965,17 +1219,21 @@ on_signal_button_press_for_area_with_node (CongEditorArea *editor_area,
 
 	case 3: /* Normally the right mouse button: */
 		{
-			GtkWidget* menu;
+			cong_document_begin_edit (doc);
 
-			menu = cong_ui_popup_init(doc, 
-						  cong_editor_node_get_node (editor_node),
-						  GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(editor_widget))));
-			gtk_menu_popup (GTK_MENU(menu), 
-					NULL, 
-					NULL, 
-					NULL, 
-					NULL, event->button,
-					event->time);		      
+			/*#error*/
+			/* FIXME: this adds a selection command: */
+			cong_document_select_node (doc,
+						   cong_editor_node_get_node (editor_node));
+			cong_ui_popup_init(doc, 
+					   cong_editor_node_get_node (editor_node),
+					   GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(editor_widget))),
+					   cong_editor_widget_get_primary_window(editor_widget) );
+			cong_ui_show_context_menu (cong_editor_widget_get_primary_window(editor_widget),
+						   event->button,
+						   event->time);
+
+			cong_document_end_edit (doc);
 		}
 		return TRUE;
 	}		
