@@ -39,6 +39,7 @@
 #include "cong-command.h"
 #include "cong-traversal-node.h"
 #include "cong-ui-hooks.h"
+#include "cong-util.h"
 
 enum FragmentState {
 	FRAG_NORMAL,
@@ -61,7 +62,10 @@ add_attrs_for_state (PangoAttrList *attr_list,
 		     guint start_index,
 		     guint end_index,
 		     enum FragmentState state);
-
+static void
+add_attrs_for_error (PangoAttrList *attr_list,
+		     guint start_index,
+		     guint end_index);
 
 #define PRIVATE(x) ((x)->private)
 
@@ -108,7 +112,7 @@ generate_line_areas_recursive (CongEditorNode *editor_node,
 			       gint line_width,
 			       gint initial_indent);
 
-static enum CongFlowType
+static CongFlowType
 get_flow_type (CongEditorNode *editor_node);
 
 /* FIXME:  We probably shouldn't have every text node in the doc listening to every text node change... probably should allow for a dispatch mechanism within the widget */
@@ -199,6 +203,15 @@ finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+/**
+ * cong_editor_node_text_construct:
+ * @editor_node_text:
+ * @widget:
+ * @traversal_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorNodeText*
 cong_editor_node_text_construct (CongEditorNodeText *editor_node_text,
 				 CongEditorWidget3* editor_widget,
@@ -207,7 +220,7 @@ cong_editor_node_text_construct (CongEditorNodeText *editor_node_text,
 #if 0
 	gchar *markup;	
 #endif
-	enum CongWhitespaceHandling whitespace = cong_node_get_whitespace_handling (cong_editor_widget3_get_document (editor_widget),
+	CongWhitespaceHandling whitespace = cong_node_get_whitespace_handling (cong_editor_widget3_get_document (editor_widget),
 										    cong_traversal_node_get_node (traversal_node));
 
 	cong_editor_node_construct (CONG_EDITOR_NODE (editor_node_text),
@@ -267,7 +280,14 @@ dispose (GObject *object)
 	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (object));
 }
 
-
+/**
+ * cong_editor_node_text_new:
+ * @widget:
+ * @traversal_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorNode*
 cong_editor_node_text_new (CongEditorWidget3 *widget,
 			   CongTraversalNode *traversal_node)
@@ -282,6 +302,15 @@ cong_editor_node_text_new (CongEditorWidget3 *widget,
 				 );
 }
 
+/**
+ * cong_editor_node_text_convert_original_byte_offset_to_stripped:
+ * @editor_node_text:
+ * @original_byte_offset:
+ * @stripped_byte_offset:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_editor_node_text_convert_original_byte_offset_to_stripped (CongEditorNodeText *editor_node_text,
 								int original_byte_offset,
@@ -292,6 +321,15 @@ cong_editor_node_text_convert_original_byte_offset_to_stripped (CongEditorNodeTe
 									 stripped_byte_offset);
 }
 
+/**
+ * cong_eel_pango_layout_calc_up:
+ * @layout:
+ * @input_index:
+ * @output_index:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_eel_pango_layout_calc_up (PangoLayout *layout,
 			       int input_index,
@@ -333,6 +371,15 @@ cong_eel_pango_layout_calc_up (PangoLayout *layout,
 	return FALSE;
 }
 
+/**
+ * cong_eel_pango_layout_calc_down:
+ * @layout:
+ * @input_index:
+ * @output_index:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_eel_pango_layout_calc_down (PangoLayout *layout,
 				 int input_index,
@@ -371,6 +418,15 @@ cong_eel_pango_layout_calc_down (PangoLayout *layout,
 	return FALSE;
 }
 
+/**
+ * cong_editor_node_text_calc_up:
+ * @editor_node_text:
+ * @input_byte_offset:
+ * @output_byte_offset:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_editor_node_text_calc_up (CongEditorNodeText *editor_node_text,
 			       int input_byte_offset,
@@ -401,6 +457,15 @@ cong_editor_node_text_calc_up (CongEditorNodeText *editor_node_text,
 
 }
 
+/**
+ * cong_editor_node_text_calc_down:
+ * @editor_node_text:
+ * @input_byte_offset:
+ * @output_byte_offset:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_editor_node_text_calc_down (CongEditorNodeText *editor_node_text,
 				 int input_byte_offset,
@@ -481,6 +546,15 @@ gboolean
 cong_selection_get_end_byte_offset (CongSelection *selection, 
 				    CongNodePtr node,
 				    gint *output);
+/**
+ * cong_selection_get_start_byte_offset:
+ * @selection:
+ * @node:
+ * @output:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_selection_get_start_byte_offset (CongSelection *selection, 
 				      CongNodePtr node,
@@ -532,6 +606,15 @@ cong_selection_get_start_byte_offset (CongSelection *selection,
 	return FALSE;
 }
 
+/**
+ * cong_selection_get_end_byte_offset:
+ * @selection:
+ * @node:
+ * @output:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_selection_get_end_byte_offset (CongSelection *selection, 
 				    CongNodePtr node,
@@ -585,6 +668,15 @@ cong_selection_get_end_byte_offset (CongSelection *selection,
 }
 #endif
 
+/**
+ * generate_line_areas_recursive:
+ * @editor_node:
+ * @line_width:
+ * @initial_indent:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 static CongEditorLineFragments*
 generate_line_areas_recursive (CongEditorNode *editor_node,
 			       gint line_width,
@@ -593,12 +685,21 @@ generate_line_areas_recursive (CongEditorNode *editor_node,
 	CongEditorLineFragments *result;
 	CongEditorNodeText *node_text = CONG_EDITOR_NODE_TEXT(editor_node);
 
+	/*
+	 * DJB 2004/08/20
+	 * We base the "whitespace handling" of this set of line fragments on the
+	 * value of the whitespace attribute of the dispspec element for this
+	 * node. Is this the best way to get this information?
+	 */
+	CongWhitespaceHandling whitespace = cong_node_get_whitespace_handling (cong_editor_node_get_document (editor_node),
+										    cong_editor_node_get_node (editor_node));
+
 #if 0
 	g_message("CongEditorNodeText::generate_line_areas_recursive, cached text =\"%s\"", 
 		  cong_text_cache_get_text (PRIVATE(node_text)->text_cache));
 #endif
 
-	result = cong_editor_line_fragments_new();
+	result = cong_editor_line_fragments_new (whitespace);
 
 #if 1
 	/* Set the "geometry" of the PangoLayout: */
@@ -945,9 +1046,11 @@ on_signal_button_press (CongEditorArea *editor_area,
 
 	case 3:
 		/* Right mouse button for most users:  Context menu: */
-		editor_popup_build(editor_widget, parent_window);
-		editor_popup_show(cong_app_singleton()->popup, event);
-		
+		editor_popup_build (editor_widget, 
+				    parent_window);
+		cong_ui_show_context_menu (cong_editor_widget_get_primary_window(editor_widget),
+					   event->button,
+					   event->time);
 		return TRUE;
 	}
 
@@ -1005,7 +1108,7 @@ on_signal_motion_notify (CongEditorArea *editor_area,
 	return FALSE;
 }
 
-static enum CongFlowType
+static CongFlowType
 get_flow_type(CongEditorNode *editor_node)
 {
 	return CONG_FLOW_TYPE_INLINE;
@@ -1087,6 +1190,34 @@ get_text_cache_input_attributes (CongEditorNodeText *editor_node_text)
 				add_attrs_for_state (attr_list, 0, strlen(text), FRAG_NORMAL);
 			}
 		}		
+
+		/* Spellcheck hack: */
+		{
+			CongDocument *doc = cong_editor_node_get_document (CONG_EDITOR_NODE (editor_node_text));
+
+			if (cong_document_should_spellcheck_node (doc, this_node)) {
+				PangoLanguage *language;
+				GList *list_of_words;
+				GList *iter;
+
+				language = cong_document_get_language_for_node (doc, 
+										this_node);
+
+				list_of_words = cong_util_get_words (language,
+								     this_node->content);
+				for (iter = list_of_words; iter; iter=iter->next) {
+					CongWord *word = (CongWord*)iter->data;
+
+					if (cong_util_spellcheck_word (language,
+								       this_node->content,
+								       word)) {
+						add_attrs_for_error (attr_list,
+								     word->start_byte_offset,
+								     word->start_byte_offset+word->length_in_bytes);
+					}									   
+				}
+			}
+		}
 	}
 #else
 	/* Big hack: */
@@ -1097,9 +1228,14 @@ get_text_cache_input_attributes (CongEditorNodeText *editor_node_text)
 	}
 #endif
 
+
 	return attr_list;
 }
 
+
+
+	   
+
 #if 0
 gboolean
 cong_selection_is_valid (CongSelection *selection)
@@ -1530,6 +1666,22 @@ add_attrs_for_state (PangoAttrList *attr_list,
 	}
 }
 
+static void
+add_attrs_for_error (PangoAttrList *attr_list,
+		     guint start_index,
+		     guint end_index)
+{
+	if (start_index!=end_index) {
+		PangoAttribute *attr;
+		
+		attr = pango_attr_underline_new (PANGO_UNDERLINE_ERROR);
+		attr->start_index = start_index;
+		attr->end_index = end_index;
+		
+		pango_attr_list_insert (attr_list, attr);
+	}	
+}
+
 static void
 refresh_pango_layout (CongEditorNodeText *editor_node_text)
 {
@@ -1551,4 +1703,3 @@ refresh_pango_layout (CongEditorNodeText *editor_node_text)
 	pango_attr_list_unref (attr_list);
 #endif
 }
-
