@@ -68,7 +68,7 @@ struct CongEditorNodeDetails
 	gboolean is_selected;
 };
 
-static enum CongFlowType
+static CongFlowType
 get_flow_type(CongEditorNode *editor_node);
 
 #if 0
@@ -122,6 +122,15 @@ cong_editor_node_instance_init (CongEditorNode *node)
 	node->priv = g_new0(CongEditorNodeDetails,1);
 }
 
+/**
+ * cong_editor_node_construct:
+ * @editor_node:
+ * @widget:
+ * @traversal_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorNode*
 cong_editor_node_construct (CongEditorNode *editor_node,
 			    CongEditorWidget3* editor_widget,
@@ -133,13 +142,21 @@ cong_editor_node_construct (CongEditorNode *editor_node,
 	return editor_node;
 }
 
+/**
+ * cong_editor_node_manufacture:
+ * @widget:
+ * @traversal_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorNode*
 cong_editor_node_manufacture (CongEditorWidget3* widget,
 			      CongTraversalNode *traversal_node)
 {
 	CongDocument *doc;
 	CongNodePtr xml_node;
-	enum CongNodeType type;
+	CongNodeType type;
 
 	g_return_val_if_fail (IS_CONG_EDITOR_WIDGET3 (widget), NULL);
 	g_return_val_if_fail (IS_CONG_TRAVERSAL_NODE (traversal_node), NULL);
@@ -207,17 +224,11 @@ cong_editor_node_manufacture (CongEditorWidget3* widget,
 		}
 		
 	case CONG_NODE_TYPE_TEXT:
+	case CONG_NODE_TYPE_CDATA_SECTION:
 		{
 			return cong_editor_node_text_new (widget, 
 							  traversal_node);
 		}
-		
-	case CONG_NODE_TYPE_CDATA_SECTION:
-		{
-			return cong_editor_node_unimplemented_new (widget, 
-								   traversal_node,
-								   cong_node_type_description (type));
-		}
 
 	case CONG_NODE_TYPE_ENTITY_REF:
 		{
@@ -286,8 +297,13 @@ cong_editor_node_manufacture (CongEditorWidget3* widget,
 	}
 }
 
-
-
+/**
+ * cong_editor_node_get_widget:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorWidget3*
 cong_editor_node_get_widget (CongEditorNode *editor_node)
 {
@@ -296,6 +312,13 @@ cong_editor_node_get_widget (CongEditorNode *editor_node)
 	return PRIVATE(editor_node)->widget;
 }
 
+/**
+ * cong_editor_node_get_document:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongDocument*
 cong_editor_node_get_document (CongEditorNode *editor_node)
 {
@@ -304,6 +327,13 @@ cong_editor_node_get_document (CongEditorNode *editor_node)
 	return cong_editor_widget3_get_document( cong_editor_node_get_widget(editor_node));
 }
 
+/**
+ * cong_editor_node_get_node:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongNodePtr
 cong_editor_node_get_node (CongEditorNode *editor_node)
 {
@@ -312,6 +342,13 @@ cong_editor_node_get_node (CongEditorNode *editor_node)
 	return cong_traversal_node_get_node (PRIVATE(editor_node)->traversal_node);
 }
 
+/**
+ * cong_editor_node_get_traversal_node:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongTraversalNode*
 cong_editor_node_get_traversal_node (CongEditorNode *editor_node)
 {
@@ -320,6 +357,13 @@ cong_editor_node_get_traversal_node (CongEditorNode *editor_node)
 	return PRIVATE(editor_node)->traversal_node;	
 }
 
+/**
+ * cong_editor_node_get_traversal_parent:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorNode*
 cong_editor_node_get_traversal_parent (CongEditorNode *editor_node)
 {
@@ -333,6 +377,13 @@ cong_editor_node_get_traversal_parent (CongEditorNode *editor_node)
 								       traversal_node_parent);
 }
 
+/**
+ * cong_editor_node_is_selected:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_editor_node_is_selected (CongEditorNode *editor_node)
 {
@@ -341,6 +392,13 @@ cong_editor_node_is_selected (CongEditorNode *editor_node)
 	return PRIVATE(editor_node)->is_selected;	
 }
 
+/**
+ * cong_editor_node_private_set_selected:
+ * @editor_node:
+ * @is_selected:
+ *
+ * TODO: Write me
+ */
 void
 cong_editor_node_private_set_selected (CongEditorNode *editor_node,
 				       gboolean is_selected)
@@ -355,7 +413,13 @@ cong_editor_node_private_set_selected (CongEditorNode *editor_node,
 	}	
 }
 
-
+/**
+ * cong_editor_node_generate_block_area:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorArea*
 cong_editor_node_generate_block_area (CongEditorNode *editor_node)
 {
@@ -369,6 +433,15 @@ cong_editor_node_generate_block_area (CongEditorNode *editor_node)
 						       (editor_node));
 }
 
+/**
+ * cong_editor_node_generate_line_areas_recursive:
+ * @editor_node:
+ * @line_width:
+ * @initial_indent:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorLineFragments*
 cong_editor_node_generate_line_areas_recursive (CongEditorNode *editor_node,
 						gint line_width,
@@ -389,6 +462,12 @@ cong_editor_node_generate_line_areas_recursive (CongEditorNode *editor_node,
 						       (editor_node, line_width, initial_indent));
 }
 
+/**
+ * cong_editor_node_line_regeneration_required:
+ * @editor_node:
+ *
+ * TODO: Write me
+ */
 void
 cong_editor_node_line_regeneration_required (CongEditorNode *editor_node)
 {
@@ -398,10 +477,17 @@ cong_editor_node_line_regeneration_required (CongEditorNode *editor_node)
 		       signals[LINE_REGENERATION_REQUIRED], 0);
 }
 
-enum CongFlowType
+/**
+ * cong_editor_node_get_flow_type:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+CongFlowType
 cong_editor_node_get_flow_type (CongEditorNode *editor_node)
 {
-	enum CongFlowType flow_type;
+	CongFlowType flow_type;
 		
 	g_return_val_if_fail (editor_node, CONG_FLOW_TYPE_BLOCK);
 
@@ -425,6 +511,13 @@ cong_editor_node_get_flow_type (CongEditorNode *editor_node)
 }
 
 #if 1
+/**
+ * cong_editor_node_is_referenced_entity_decl:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_editor_node_is_referenced_entity_decl (CongEditorNode *editor_node)
 {
@@ -437,6 +530,13 @@ cong_editor_node_is_referenced_entity_decl (CongEditorNode *editor_node)
 	return cong_traversal_node_is_referenced_entity_decl (traversal_node);
 }
 
+/**
+ * cong_editor_node_get_prev:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorNode*
 cong_editor_node_get_prev (CongEditorNode *editor_node)
 {
@@ -470,6 +570,13 @@ cong_editor_node_get_prev (CongEditorNode *editor_node)
 	}
 }
 
+/**
+ * cong_editor_node_get_next:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorNode*
 cong_editor_node_get_next (CongEditorNode *editor_node)
 {
@@ -502,6 +609,13 @@ cong_editor_node_get_next (CongEditorNode *editor_node)
 	}
 }
 #else
+/**
+ * cong_editor_node_is_referenced_entity_decl:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_editor_node_is_referenced_entity_decl (CongEditorNode *editor_node)
 {
@@ -517,6 +631,13 @@ cong_editor_node_is_referenced_entity_decl (CongEditorNode *editor_node)
 	return FALSE;
 }
 
+/**
+ * cong_editor_node_get_prev:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorNode*
 cong_editor_node_get_prev (CongEditorNode *editor_node)
 {
@@ -541,6 +662,13 @@ cong_editor_node_get_prev (CongEditorNode *editor_node)
 	}
 }
 
+/**
+ * cong_editor_node_get_next:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorNode*
 cong_editor_node_get_next (CongEditorNode *editor_node)
 {
@@ -565,6 +693,13 @@ cong_editor_node_get_next (CongEditorNode *editor_node)
 }
 #endif
 
+/**
+ * cong_editor_node_get_child_policy:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorChildPolicy*
 cong_editor_node_get_child_policy (CongEditorNode *editor_node)
 {
@@ -573,6 +708,13 @@ cong_editor_node_get_child_policy (CongEditorNode *editor_node)
 	return PRIVATE(editor_node)->child_policy;
 }
 
+/**
+ * cong_editor_node_set_child_policy:
+ * @editor_node:
+ * @child_policy:
+ *
+ * TODO: Write me
+ */
 void
 cong_editor_node_set_child_policy (CongEditorNode *editor_node,
 				   CongEditorChildPolicy *child_policy)
@@ -582,6 +724,13 @@ cong_editor_node_set_child_policy (CongEditorNode *editor_node,
 	PRIVATE(editor_node)->child_policy = child_policy;
 }
 
+/**
+ * cong_editor_node_get_parents_child_policy:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorChildPolicy*
 cong_editor_node_get_parents_child_policy (CongEditorNode *editor_node)
 {
@@ -590,6 +739,13 @@ cong_editor_node_get_parents_child_policy (CongEditorNode *editor_node)
 	return PRIVATE(editor_node)->parents_child_policy;
 }
 
+/**
+ * cong_editor_node_set_parents_child_policy:
+ * @editor_node:
+ * @child_policy:
+ *
+ * TODO: Write me
+ */
 void
 cong_editor_node_set_parents_child_policy (CongEditorNode *editor_node,
 					   CongEditorChildPolicy *child_policy)
@@ -599,7 +755,7 @@ cong_editor_node_set_parents_child_policy (CongEditorNode *editor_node,
 	PRIVATE(editor_node)->parents_child_policy = child_policy;
 }
 
-static enum CongFlowType
+static CongFlowType
 get_flow_type(CongEditorNode *editor_node)
 {
 	return CONG_FLOW_TYPE_BLOCK;
