@@ -65,6 +65,8 @@ struct CongEditorWidget3Details
 {
 	CongDocument *doc;
 
+	CongPrimaryWindow *primary_window;
+
 	GHashTable *hash_of_traversal_node_to_editor_node;
 
 	CongEditorArea *root_area;
@@ -269,7 +271,13 @@ cong_editor_widget3_instance_init (CongEditorWidget3 *widget)
 	widget->private = g_new0(CongEditorWidget3Details,1);
 }
 
-/* Disconnects all signals from instance that have the supplied user_data as the closure data; returns the number disconnected: */
+/**
+ * cong_eel_disconnect_all_with_data:
+ * @instance: The instance owning the signal handler to be found
+ * @user_data: The closure data of the handler's closure
+ *
+ * Returns: The number of signals disconnected
+ */
 guint
 cong_eel_disconnect_all_with_data (gpointer instance,
 				   gpointer user_data)
@@ -297,14 +305,25 @@ cong_eel_disconnect_all_with_data (gpointer instance,
 	}
 }
 
+/**
+ * cong_editor_widget3_construct:
+ * @editor_widget:
+ * @doc:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorWidget3*
 cong_editor_widget3_construct (CongEditorWidget3 *editor_widget,
-			       CongDocument *doc)
+			       CongDocument *doc,
+			       CongPrimaryWindow *primary_window)
 {
 	PRIVATE(editor_widget)->doc = doc;
 
 	g_object_ref(G_OBJECT(doc));
 
+	PRIVATE(editor_widget)->primary_window = primary_window;
+
 #if 1
 	/* FIXME: need to clean up properly: */
 	PRIVATE(editor_widget)->hash_of_traversal_node_to_editor_node = g_hash_table_new_full (NULL,
@@ -414,27 +433,53 @@ cong_editor_widget3_construct (CongEditorWidget3 *editor_widget,
 	return editor_widget;
 }
 
-GtkWidget* cong_editor_widget3_new(CongDocument *doc)
+/**
+ * cong_editor_widget3_new:
+ * @doc:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+GtkWidget* 
+cong_editor_widget3_new(CongDocument *doc,
+			CongPrimaryWindow *primary_window)
 {
 	g_return_val_if_fail(doc, NULL);
 
 	return GTK_WIDGET( cong_editor_widget3_construct (g_object_new (CONG_EDITOR_WIDGET3_TYPE, NULL),
-							  doc)
+							  doc,
+							  primary_window)
 			   );
 }
 
-CongDocument *cong_editor_widget3_get_document(CongEditorWidget3 *editor_widget)
+/**
+ * cong_editor_widget3_get_document:
+ * @editor_widget:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+CongDocument *
+cong_editor_widget3_get_document(CongEditorWidget3 *editor_widget)
 {
 	g_return_val_if_fail(editor_widget, NULL);
 
 	return PRIVATE(editor_widget)->doc;
 }
 
-CongDispspec *cong_editor_widget_get_dispspec(CongEditorWidget3 *editor_widget)
+/**
+ * cong_editor_widget_get_default_dispspec:
+ * @editor_widget:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+CongDispspec *
+cong_editor_widget_get_default_dispspec(CongEditorWidget3 *editor_widget)
 {
 	g_return_val_if_fail(editor_widget, NULL);
 
-	return cong_document_get_dispspec(PRIVATE(editor_widget)->doc);
+	return cong_document_get_default_dispspec (PRIVATE(editor_widget)->doc);
 }
 
 #if 0
@@ -463,6 +508,14 @@ void cong_editor_widget_force_layout_update(CongEditorWidget3 *editor_widget)
 }
 #endif
 
+/**
+ * cong_editor_widget3_get_editor_node_for_traversal_node:
+ * @editor_widget:
+ * @traversal_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorNode*
 cong_editor_widget3_get_editor_node_for_traversal_node (CongEditorWidget3 *editor_widget,
 							CongTraversalNode *traversal_node)
@@ -502,6 +555,15 @@ for_each_editor_node_cb (CongDocumentTraversal *doc_traversal,
 	
 }
 
+/**
+ * cong_editor_widget3_for_each_editor_node:
+ * @widget:
+ * @xml_node:
+ * @editor_node_callback:
+ * @user_data:
+ *
+ * TODO: Write me
+ */
 void
 cong_editor_widget3_for_each_editor_node (CongEditorWidget3 *editor_widget,
 					  CongNodePtr xml_node,
@@ -524,6 +586,14 @@ cong_editor_widget3_for_each_editor_node (CongEditorWidget3 *editor_widget,
 							 &tmp_struct);
 }
 
+/**
+ * cong_editor_widget3_get_an_editor_node:
+ * @editor_widget:
+ * @node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorNode*
 cong_editor_widget3_get_an_editor_node (CongEditorWidget3 *editor_widget,
 					CongNodePtr xml_node)
@@ -544,6 +614,13 @@ cong_editor_widget3_get_an_editor_node (CongEditorWidget3 *editor_widget,
 	}
 }
 
+/**
+ * cong_editor_widget3_get_prehighlight_editor_area:
+ * @editor_widget:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorArea*
 cong_editor_widget3_get_prehighlight_editor_area (CongEditorWidget3 *editor_widget)
 {
@@ -552,6 +629,13 @@ cong_editor_widget3_get_prehighlight_editor_area (CongEditorWidget3 *editor_widg
 	return PRIVATE (editor_widget)->prehighlight_area;
 }
 
+/**
+ * cong_editor_widget3_set_prehighlight_editor_area:
+ * @editor_widget:
+ * @editor_area:
+ *
+ * TODO: Write me
+ */
 void
 cong_editor_widget3_set_prehighlight_editor_area (CongEditorWidget3 *editor_widget,
 						  CongEditorArea* editor_area)
@@ -593,7 +677,13 @@ cong_editor_widget3_set_prehighlight_editor_area (CongEditorWidget3 *editor_widg
 	}
 }
 
-
+/**
+ * cong_editor_widget3_get_test_gc:
+ * @editor_widget:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 GdkGC*
 cong_editor_widget3_get_test_gc (CongEditorWidget3 *editor_widget)
 {
@@ -602,7 +692,15 @@ cong_editor_widget3_get_test_gc (CongEditorWidget3 *editor_widget)
 	return PRIVATE(editor_widget)->test_gc;
 }
 
-
+/**
+ * cong_editor_widget3_get_area_at:
+ * @editor_widget:
+ * @x:
+ * @y:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorArea*
 cong_editor_widget3_get_area_at (CongEditorWidget3 *editor_widget,
 				 gint x,
@@ -613,8 +711,15 @@ cong_editor_widget3_get_area_at (CongEditorWidget3 *editor_widget,
 						      y);
 }
 
+/**
+ * cong_flow_type_get_debug_string:
+ * @flow_type:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 const gchar*
-cong_flow_type_get_debug_string(enum CongFlowType flow_type)
+cong_flow_type_get_debug_string(CongFlowType flow_type)
 {
 	switch (flow_type) {
 	default: g_assert_not_reached();
@@ -623,6 +728,13 @@ cong_flow_type_get_debug_string(enum CongFlowType flow_type)
 	}
 }
 
+/**
+ * cong_editor_widget3_add_popup_items:
+ * @editor_widget:
+ * @menu:
+ *
+ * TODO: Write me
+ */
 void
 cong_editor_widget3_add_popup_items (CongEditorWidget3 *editor_widget,
 				     GtkMenu *menu)
@@ -644,6 +756,15 @@ cong_editor_widget3_add_popup_items (CongEditorWidget3 *editor_widget,
 	gtk_widget_show (submenu_menu);
 }
 
+/**
+ * cong_editor_widget3_get_preedit_data:
+ * @editor_widget:
+ * @output_string:
+ * @output_pango_attr_list:
+ * @output_cursor_pos:
+ *
+ * TODO: Write me
+ */
 void
 cong_editor_widget3_get_preedit_data (CongEditorWidget3 *editor_widget,
 				      gchar **output_string, 
@@ -1068,7 +1189,7 @@ cong_editor_widget3_get_destination_location_for_keypress (CongEditorWidget3 *ed
 
 	doc = cong_editor_widget3_get_document (editor_widget);
 	cursor = cong_document_get_cursor (doc);
-	dispspec = cong_document_get_dispspec (doc);
+	dispspec = cong_document_get_default_dispspec (doc);
 
 	switch (keyval) {
 	default: 
@@ -1088,31 +1209,31 @@ cong_editor_widget3_get_destination_location_for_keypress (CongEditorWidget3 *ed
 		if (state & GDK_CONTROL_MASK) {
 			return cong_location_calc_prev_word(&cursor->location, doc, output_loc);
 		} else {
-			return cong_location_calc_prev_char(&cursor->location, dispspec, output_loc);
+			return cong_location_calc_prev_char(&cursor->location, doc, output_loc);
 		}
 	
 	case GDK_Right:
 		if (state & GDK_CONTROL_MASK) {
 			return cong_location_calc_next_word(&cursor->location, doc, output_loc);
 		} else {
-			return cong_location_calc_next_char(&cursor->location, dispspec, output_loc);
+			return cong_location_calc_next_char(&cursor->location, doc, output_loc);
 		}
 	case GDK_Home:
 		if (state & GDK_CONTROL_MASK) {
-			return cong_location_calc_document_start(&cursor->location, dispspec, output_loc);
+			return cong_location_calc_document_start(&cursor->location, doc, output_loc);
 		} else {
-			return cong_location_calc_line_start(&cursor->location, dispspec, output_loc);
+			return cong_location_calc_line_start(&cursor->location, doc, output_loc);
 		}
 	case GDK_End:
 		if (state & GDK_CONTROL_MASK) {
-			return cong_location_calc_document_end(&cursor->location, dispspec, output_loc);
+			return cong_location_calc_document_end(&cursor->location, doc, output_loc);
 		} else {
-			return cong_location_calc_line_end(&cursor->location, dispspec, output_loc);
+			return cong_location_calc_line_end(&cursor->location, doc, output_loc);
 		}
 	case GDK_Page_Up:
-		return cong_location_calc_prev_page(&cursor->location, dispspec, output_loc);
+		return cong_location_calc_prev_page(&cursor->location, doc, output_loc);
 	case GDK_Page_Down:
-		return cong_location_calc_next_page(&cursor->location, dispspec, output_loc);
+		return cong_location_calc_next_page(&cursor->location, doc, output_loc);
 	}
 }
 
@@ -1232,7 +1353,7 @@ key_press_event_handler (GtkWidget *w,
 	case GDK_Return:
 		{
 			if  (cong_location_exists(&cursor->location)) {
-				if (cong_location_node_type(&cursor->location) == CONG_NODE_TYPE_TEXT) {
+				if (cong_node_type_is_textual_content (cong_location_node_type(&cursor->location))) {
 					CongDispspecElement *ds_element = cong_document_get_dispspec_element_for_node(doc, cursor->location.node->parent);
 
 					if (ds_element) {
@@ -1540,6 +1661,12 @@ populate_widget3(CongEditorWidget3 *widget)
        	/* FIXME: Also need to do the opposite when widget is destroyed */
 }
 
+/**
+ * cong_editor_widget3_node_should_have_editor_node:
+ * @node:
+ *
+ * TODO: Write me
+ */
 gboolean
 cong_editor_widget3_node_should_have_editor_node (CongNodePtr node)
 {
@@ -1560,7 +1687,7 @@ create_areas(CongEditorWidget3 *widget,
 {
 	CongEditorChildPolicy *parents_child_policy = NULL;
 	CongEditorChildPolicy *this_child_policy = NULL;
-	enum CongFlowType flow_type;
+	CongFlowType flow_type;
 	CongNodePtr node = cong_editor_node_get_node (editor_node);
 
 #if LOG_EDITOR_AREAS
@@ -1694,6 +1821,15 @@ cong_editor_widget3_get_child_policy_for_editor_node (CongEditorWidget3 *widget,
 #endif
 	
 }
+
+/**
+ * cong_editor_widget3_get_parents_child_policy_for_editor_node:
+ * @widget:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorChildPolicy*
 cong_editor_widget3_get_parents_child_policy_for_editor_node (CongEditorWidget3 *widget,
 							      CongEditorNode *editor_node)
@@ -1710,6 +1846,15 @@ cong_editor_widget3_get_parents_child_policy_for_editor_node (CongEditorWidget3
 	
 }
 #else
+
+/**
+ * cong_editor_widget3_get_primary_area_for_editor_node:
+ * @widget:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorArea*
 cong_editor_widget3_get_primary_area_for_editor_node (CongEditorWidget3 *widget,
 						      CongEditorNode *editor_node)
@@ -1721,6 +1866,14 @@ cong_editor_widget3_get_primary_area_for_editor_node (CongEditorWidget3 *widget,
 						     editor_node));
 }
 
+/**
+ * cong_editor_widget3_get_parent_flow_holder_for_editor_node:
+ * @widget:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorAreaFlowHolder*
 cong_editor_widget3_get_parent_flow_holder_for_editor_node (CongEditorWidget3 *widget,
 							    CongEditorNode *editor_node)
@@ -1732,6 +1885,14 @@ cong_editor_widget3_get_parent_flow_holder_for_editor_node (CongEditorWidget3 *w
 								 editor_node));
 }
 
+/**
+ * cong_editor_widget3_get_child_flow_holder_for_editor_node:
+ * @widget:
+ * @editor_node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongEditorAreaFlowHolder*
 cong_editor_widget3_get_child_flow_holder_for_editor_node (CongEditorWidget3 *widget,
 							   CongEditorNode *editor_node)
@@ -1743,3 +1904,9 @@ cong_editor_widget3_get_child_flow_holder_for_editor_node (CongEditorWidget3 *wi
 								editor_node));
 }
 #endif
+
+CongPrimaryWindow*
+cong_editor_widget_get_primary_window(CongEditorWidget3 *editor_widget)
+{
+	return PRIVATE(editor_widget)->primary_window;
+}
