@@ -35,6 +35,7 @@
 #include "cong-eel.h"
 #include "cong-attribute-editor.h"
 #include "cong-command.h"
+#include "cong-util.h"
 
 #define CONG_ADVANCED_NODE_PROPERTIES_VIEW(x) ((CongAdvancedNodePropertiesView*)(x))
 typedef struct CongAdvancedNodePropertiesView CongAdvancedNodePropertiesView;
@@ -50,10 +51,10 @@ static void raw_attr_list_refresh(CongAdvancedNodePropertiesView *view,
 static void on_document_node_make_orphan(CongView *view, gboolean before_event, CongNodePtr node, CongNodePtr former_parent);
 static void on_document_node_add_after(CongView *view, gboolean before_event, CongNodePtr node, CongNodePtr older_sibling);
 static void on_document_node_add_before(CongView *view, gboolean before_event, CongNodePtr node, CongNodePtr younger_sibling);
-static void on_document_node_set_parent(CongView *view, gboolean before_event, CongNodePtr node, CongNodePtr adoptive_parent); /* added to end of child list */
+static void on_document_node_set_parent(CongView *view, gboolean before_event, CongNodePtr node, CongNodePtr adoptive_parent, gboolean add_to_end);
 static void on_document_node_set_text(CongView *view, gboolean before_event, CongNodePtr node, const xmlChar *new_content);
-static void on_document_node_set_attribute(CongView *view, gboolean before_event, CongNodePtr node, const xmlChar *name, const xmlChar *value);
-static void on_document_node_remove_attribute(CongView *view, gboolean before_event, CongNodePtr node, const xmlChar *name);
+static void on_document_node_set_attribute(CongView *view, gboolean before_event, CongNodePtr node, xmlNs *ns_ptr, const xmlChar *name, const xmlChar *value);
+static void on_document_node_remove_attribute(CongView *view, gboolean before_event, CongNodePtr node, xmlNs *ns_ptr, const xmlChar *name);
 static void on_selection_change(CongView *view);
 static void on_cursor_change(CongView *view);
 
@@ -147,7 +148,7 @@ static void on_document_node_add_before(CongView *view, gboolean before_event, C
 	/* UNWRITTEN */
 }
 
-static void on_document_node_set_parent(CongView *view, gboolean before_event, CongNodePtr node, CongNodePtr adoptive_parent)
+static void on_document_node_set_parent(CongView *view, gboolean before_event, CongNodePtr node, CongNodePtr adoptive_parent, gboolean add_to_end)
 {
 	CongAdvancedNodePropertiesView *properties_view;
 
@@ -181,7 +182,7 @@ static void on_document_node_set_text(CongView *view, gboolean before_event, Con
 	/* UNWRITTEN */
 }
 
-static void on_document_node_set_attribute(CongView *view, gboolean before_event, CongNodePtr node, const xmlChar *name, const xmlChar *value)
+static void on_document_node_set_attribute(CongView *view, gboolean before_event, CongNodePtr node, xmlNs *ns_ptr, const xmlChar *name, const xmlChar *value)
 {
 	CongAdvancedNodePropertiesView *properties_view;
 
@@ -200,7 +201,7 @@ static void on_document_node_set_attribute(CongView *view, gboolean before_event
 			      &properties_view->raw_attr);
 }
 
-static void on_document_node_remove_attribute(CongView *view, gboolean before_event, CongNodePtr node, const xmlChar *name)
+static void on_document_node_remove_attribute(CongView *view, gboolean before_event, CongNodePtr node, xmlNs *ns_ptr, const xmlChar *name)
 {
 	CongAdvancedNodePropertiesView *properties_view;
 
@@ -226,9 +227,16 @@ static void on_cursor_change(CongView *view)
 {
 }
 
-
-void init_view_xpath_view(CongAdvancedNodePropertiesView *view,
-			  struct XPathView* xpath_view)
+/**
+ * init_view_xpath_view:
+ * @view:
+ * @xpath_view:
+ *
+ * TODO: Write me
+ */
+void 
+init_view_xpath_view(CongAdvancedNodePropertiesView *view,
+		     struct XPathView* xpath_view)
 {
 	gchar *xpath;
 
@@ -256,14 +264,24 @@ static void raw_attr_list_refresh(CongAdvancedNodePropertiesView *view,
 
 	for (attr_iter = view->node->properties; attr_iter; attr_iter=attr_iter->next) {
 		GtkTreeIter iter;
+
+		char *qualified_name;
+		
+		qualified_name = 
+			cong_util_get_qualified_attribute_name(attr_iter->ns,
+							       attr_iter->name);
+		
+
 		gtk_list_store_append(raw_attr->list_store,
 				      &iter);
 		
 		gtk_list_store_set(raw_attr->list_store,
 				   &iter,
-				   RAW_ATTR_MODEL_COLUMN_NAME, attr_iter->name,
+				   RAW_ATTR_MODEL_COLUMN_NAME, qualified_name,
 				   RAW_ATTR_MODEL_COLUMN_VALUE, attr_iter->children->content,
 				   -1);
+
+		g_free(qualified_name);
 	}
 }
 
@@ -279,7 +297,7 @@ static gchar* get_attr_name_for_tree_iter(CongAdvancedNodePropertiesView *view,
 	
 	result = g_value_dup_string(value);
 	g_value_unset(value);
-	
+
 	g_free(value);
 
 	return result;
@@ -308,25 +326,59 @@ static gchar* get_attr_name_for_tree_path(CongAdvancedNodePropertiesView *view,
 }
 
 static void on_name_edited(GtkCellRendererText *cellrenderertext,
-			   gchar *arg1,
-			   gchar *arg2,
+			   gchar *path,
+			   gchar *new_text,
 			   gpointer user_data)
 {
-	/* FIXME: arg1 appears to be path; arg2 is value; is this really the case? */
+	/* FIXME: path appears to be path; new_text is value; 
+	 * is this really the case? At least in "gtkcellrednderertext.h" */
 
 	CongAdvancedNodePropertiesView *view = user_data;
-	gchar* attr_name = get_attr_name_for_tree_path(view, arg1);
-	gchar *attr_value = cong_node_get_attribute(view->node, attr_name);
+	gchar *qualified_attr_name = get_attr_name_for_tree_path(view, path);
+
+	const char *local_name;
+	xmlNs *ns_ptr = cong_node_get_attr_ns(view->node, 
+							qualified_attr_name,
+							&local_name);
+	const char *new_local_name;
+	xmlNs *new_ns_ptr = cong_node_get_attr_ns(view->node, 
+							    new_text,
+							    &new_local_name);
+
+	gchar *attr_value = cong_node_get_attribute(view->node, 
+						    ns_ptr,
+						    local_name);
 
 	g_message("on_name_edited");
 
+	/* ##FIXME: Add name check for qualified attribute name  */
+
+	if(new_ns_ptr == NULL && new_local_name != new_text) {
+		/* new unknown namespace */
+		gchar *msg = g_strdup_printf ("Creation of new namespace for attribute \"%s\"", 
+					      new_text);
+
+		/* ##FIXME */
+		CONG_DO_UNIMPLEMENTED_DIALOG_WITH_BUGZILLA_ID (NULL, 
+							       msg,
+							       135858);
+
+		g_free (msg);
+		g_free(qualified_attr_name);
+		g_free(attr_value);
+		
+		return ;
+	}
+	
+
+
 	/* Ignore if the node already has an attribute of that name: */
-	if (!xmlHasProp(view->node, arg2)) {
+	if (!cong_node_has_attribute(view->node, new_ns_ptr, new_local_name)) {
 		CongDocument *doc = cong_view_get_document(CONG_VIEW(view));
 
 		gchar *desc = g_strdup_printf(_("Rename attribute \"%s\" as \"%s\""),
-					      attr_name,
-					      arg2);
+					      qualified_attr_name,
+					      new_text);
 
 		CongCommand *cmd = cong_document_begin_command (doc,
 								desc,
@@ -334,36 +386,44 @@ static void on_name_edited(GtkCellRendererText *cellrenderertext,
 
 		cong_command_add_node_remove_attribute (cmd,
 							view->node,
-							attr_name);
+							ns_ptr,
+							local_name);
 		cong_command_add_node_set_attribute (cmd,
 						     view->node,
-						     arg2,
+						     new_ns_ptr,
+						     new_local_name,
 						     attr_value);
 		cong_document_end_command (doc,
 					   cmd);
 	}
 	
-	g_free(attr_name);
+	g_free(qualified_attr_name);
 	g_free(attr_value);
 }
 
 static void on_value_edited(GtkCellRendererText *cellrenderertext,
-			    gchar *arg1,
-			    gchar *arg2,
+			    gchar *path,
+			    gchar *new_text,
 			    gpointer user_data)
 {
-	/* FIXME: arg1 appears to be path; arg2 is value; is this really the case? */
+	/* FIXME: path appears to be path; new_text is value; 
+	 * is this really the case? At least in "gtkcellrednderertext.h" */
 
 	CongAdvancedNodePropertiesView *view = user_data;
-	gchar* attr_name = get_attr_name_for_tree_path(view, arg1);
+	gchar* qualified_name = get_attr_name_for_tree_path(view, path);
+	const char *local_name;
+	xmlNs *ns_ptr = cong_node_get_attr_ns(view->node, 
+							qualified_name,
+							&local_name);
+
 	CongDocument *doc = cong_view_get_document(CONG_VIEW(view));
 
-	g_message("on_value_edited %s = %s", attr_name, arg2);
+	g_message("on_value_edited %s = %s", qualified_name, new_text);
 
 	{
 		gchar *desc = g_strdup_printf(_("Set attribute \"%s\" to \"%s\""),
-					      attr_name,
-					      arg2);
+					      qualified_name,
+					      new_text);
 
 		CongCommand *cmd = cong_document_begin_command (doc,
 								desc,
@@ -371,13 +431,14 @@ static void on_value_edited(GtkCellRendererText *cellrenderertext,
 
 		cong_command_add_node_set_attribute (cmd,
 						     view->node,
-						     attr_name,
-						     arg2);
+						     ns_ptr,
+						     local_name,
+						     new_text);
 		cong_document_end_command (doc,
 					   cmd);
 	}
 
-	g_free(attr_name);
+	g_free(qualified_name);
 }
 
 static void on_add_attribute(GtkButton *button,
@@ -394,7 +455,7 @@ static void on_add_attribute(GtkButton *button,
 	while (1) {
 		attr_name = g_strdup_printf("attribute%i", num);
 
-		if (xmlHasProp(view->node, attr_name)) {
+		if (cong_node_has_attribute(view->node, NULL, attr_name)) {
 			g_free(attr_name);
 			num++;
 		} else {
@@ -402,7 +463,8 @@ static void on_add_attribute(GtkButton *button,
 		}
 	}
 
-	g_assert(!xmlHasProp(view->node, attr_name));
+	/* is this _not_ always true ? */
+	g_assert(!cong_node_has_attribute(view->node, NULL, attr_name));
 
 	{
 		gchar *desc = g_strdup_printf(_("Add attribute \"%s\""),
@@ -414,12 +476,14 @@ static void on_add_attribute(GtkButton *button,
 
 		cong_command_add_node_set_attribute (cmd,
 						     view->node,
+						     NULL,
 						     attr_name,
 						     "");
 		cong_document_end_command (doc,
 					   cmd);
 	}
 
+	g_free(attr_name);
 }
 
 static void on_delete_attribute(GtkButton *button,
@@ -433,10 +497,14 @@ static void on_delete_attribute(GtkButton *button,
 	if (gtk_tree_selection_get_selected (selection,
                                              NULL,
                                              &iter)) {
-		gchar* attr_name = get_attr_name_for_tree_iter(view, &iter);
+		char *qualified_name = get_attr_name_for_tree_iter(view, &iter);
+		const char *local_name;
+		xmlNs *ns_ptr = cong_node_get_attr_ns(view->node, 
+								qualified_name,
+								&local_name);
 
 		gchar *desc = g_strdup_printf(_("Delete attribute \"%s\""),
-					      attr_name);
+					      qualified_name);
 
 		CongCommand *cmd = cong_document_begin_command (doc,
 								desc,
@@ -444,11 +512,12 @@ static void on_delete_attribute(GtkButton *button,
 
 		cong_command_add_node_remove_attribute (cmd,
 							view->node,
-							attr_name);
+							ns_ptr,
+							local_name);
 		cong_document_end_command (doc,
 					   cmd);
 
-		g_free(attr_name);
+		g_free(qualified_name);
 	}
 }
 
@@ -463,6 +532,12 @@ static void on_tree_view_selection_change(GtkTreeSelection *treeselection,
 	
 }
 
+/**
+ * init_view_namespace:
+ * @view:
+ *
+ * TODO: Write me
+ */
 void 
 init_view_namespace (CongAdvancedNodePropertiesView *view)
 {
@@ -490,9 +565,16 @@ init_view_namespace (CongAdvancedNodePropertiesView *view)
 						     TRUE);
 }
 
-
-void init_view_raw_attr(CongAdvancedNodePropertiesView *view,
-			struct RawAttr* raw_attr)
+/**
+ * init_view_raw_attr:
+ * @view:
+ * @raw_attr:
+ *
+ * TODO: Write me
+ */
+void 
+init_view_raw_attr(CongAdvancedNodePropertiesView *view,
+		   struct RawAttr* raw_attr)
 {
 	/* FIXME: use libglade for this... */
 	GtkWidget *vbox = gtk_vbox_new(FALSE, 6);
@@ -574,6 +656,12 @@ void init_view_raw_attr(CongAdvancedNodePropertiesView *view,
 	cong_dialog_category_add_selflabelled_field(raw_attr->category, vbox, TRUE);
 }
 
+/**
+ * cong_util_get_tag_string_for_node:
+ * @node:
+ *
+ * TODO: Write me
+ */
 gchar*
 cong_util_get_tag_string_for_node (CongNodePtr node)
 {
@@ -586,6 +674,12 @@ cong_util_get_tag_string_for_node (CongNodePtr node)
 #endif
 }
 
+/**
+ * cong_util_get_tag_string_for_node_escaped:
+ * @node:
+ *
+ * TODO: Write me
+ */
 gchar*
 cong_util_get_tag_string_for_node_escaped (CongNodePtr node)
 {
@@ -860,7 +954,12 @@ populate_recursive (GtkTreeStore *store,
 
 }
 
-
+/**
+ * make_tree_model_for_element_model:
+ * @xml_element:
+ *
+ * TODO: Write me
+ */
 GtkTreeModel*
 make_tree_model_for_element_model (xmlElementPtr xml_element)
 {  
@@ -887,6 +986,12 @@ make_tree_model_for_element_model (xmlElementPtr xml_element)
 	return GTK_TREE_MODEL (store);
 }
 
+/**
+ * make_widget_for_element_model:
+ * @xml_element:
+ *
+ * TODO: Write me
+ */
 GtkWidget*
 make_widget_for_element_model (xmlElementPtr xml_element)
 {
@@ -957,6 +1062,14 @@ make_dtd_content_model (CongDocument *doc,
 						     TRUE);
 }
 
+/**
+ * cong_node_properties_dtd_new:
+ * @doc:
+ * @node:
+ * @within_notebook:
+ *
+ * TODO: Write me
+ */
 GtkWidget*
 cong_node_properties_dtd_new (CongDocument *doc, 
 			      CongNodePtr node,
@@ -990,6 +1103,14 @@ cong_node_properties_dtd_new (CongDocument *doc,
 	return NULL;
 }
 
+/**
+ * cong_node_properties_dialog_advanced_new:
+ * @doc:
+ * @node:
+ * @within_notebook:
+ *
+ * TODO: Write me
+ */
 GtkWidget*
 cong_node_properties_dialog_advanced_new(CongDocument *doc, 
 					 CongNodePtr node,
@@ -1035,9 +1156,18 @@ cong_node_properties_dialog_advanced_new(CongDocument *doc,
 	return widget;
 }
 
-void cong_ui_append_advanced_node_properties_page(GtkNotebook *notebook,
-						  CongDocument *doc, 
-						  CongNodePtr node)
+/**
+ * cong_ui_append_advanced_node_properties_page:
+ * @notebook:
+ * @doc:
+ * @node:
+ *
+ * TODO: Write me
+ */
+void 
+cong_ui_append_advanced_node_properties_page(GtkNotebook *notebook,
+					     CongDocument *doc, 
+					     CongNodePtr node)
 {
 	GtkWidget* dtd_page;
 
@@ -1064,6 +1194,39 @@ void cong_ui_append_advanced_node_properties_page(GtkNotebook *notebook,
 				 );
 }
 
+/**
+ * cong_node_properties_dialog_new:
+ * @doc: The document being displayed.
+ * @node: The node to display.
+ * @parent_window: The parent window of the document.
+ *
+ * Create a dialog window which displays, and allows the user to edit,
+ * the properties of the supplied node. The format of the dialog
+ * depends on the node type. 
+ *
+ * The main interest will be for nodes of type CONG_NODE_TYPE_ELEMENT.
+ * If there is a plugin registered for this node then the plugin will be
+ * used, otherwise the attributes of the node will be displayed. In the
+ * later case, the attributes will be displayed in a "raw" form (as
+ * a set of name and value pairs) and, if a DTD associated with this file,
+ * using an interface that follows the DTD (e.g. will limit enumerated
+ * values to the allowable choices only).
+ *
+ * For CONG_NODE_TYPE_TEXT nodes the properties of the parent node will
+ * be displayed. This is primarily so that users can access the attributes
+ * of span nodes, but it will work for any text node. Should there be
+ * a way to indicate that this has happened (i.e. some indication
+ * in the dialog that we are actually displaying the parent's
+ * properties)?
+ *
+ * Other node types result in a dialog which only lists the name
+ * and the XPath location of the node.
+ *
+ * At present the returned dialog window should be displayed using
+ * gtk_widget_show() rather than gtk_dialog_run().
+ *
+ * Returns: a #GtkWidget displaying the properties of the node.
+ */
 GtkWidget*
 cong_node_properties_dialog_new (CongDocument *doc, 
 				 CongNodePtr node, 
@@ -1072,6 +1235,18 @@ cong_node_properties_dialog_new (CongDocument *doc,
 	g_return_val_if_fail (doc, NULL);
 	g_return_val_if_fail (node, NULL);
 
+	/*
+	 * If this is a text node then display the properties of its
+	 * parent node instead. I do not think this can recurse "wildly"
+	 * (presumably a text node can not have a text node as its
+	 * parent).
+	 */
+	if (cong_node_type(node)==CONG_NODE_TYPE_TEXT) {
+		return cong_node_properties_dialog_new (doc,
+							cong_node_parent(node),
+							parent_window);
+	}
+
 	/* Should we use a plugin for this node?: */
 	if (cong_node_type(node)==CONG_NODE_TYPE_ELEMENT) {
 		CongDispspecElement *element;
@@ -1108,7 +1283,7 @@ cong_node_properties_dialog_new (CongDocument *doc,
 		dialog = gtk_dialog_new_with_buttons(_("Properties"),
 						     parent_window,
 						     GTK_DIALOG_MODAL,
-						     GTK_STOCK_OK, GTK_RESPONSE_OK,
+						     GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
 						     NULL);		
 
 		gtk_container_set_border_width(GTK_CONTAINER(dialog), 6);
@@ -1154,12 +1329,10 @@ cong_node_properties_dialog_new (CongDocument *doc,
 		}
 
 		g_signal_connect_swapped (G_OBJECT (dialog), 
-					  "response", 
-					  G_CALLBACK (gtk_widget_destroy),
+					  "response", G_CALLBACK (gtk_widget_destroy),
 					  GTK_OBJECT (dialog));
 
 
 		return GTK_WIDGET(dialog);
 	}
 }
-
