@@ -33,6 +33,7 @@
 #include "cong-util.h"
 #include "cong-vfs.h"
 #include "cong-traversal-node.h"
+#include "cong-source-layout.h"
 
 #include <libxslt/xsltInternals.h>
 #include <libxslt/transform.h>
@@ -87,6 +88,16 @@ void cong_plugin_unregister_exporter(CongPlugin *plugin,
 
 
 /* Implementation of CongPlugin: */
+/**
+ * cong_plugin_construct:
+ * @plugin:
+ * @plugin_id:
+ * @register_callback:
+ * @configure_callback:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongPlugin*
 cong_plugin_construct (CongPlugin *plugin,
 		       const gchar *plugin_id,
@@ -106,6 +117,13 @@ cong_plugin_construct (CongPlugin *plugin,
 	return plugin;
 }
 
+/**
+ * cong_plugin_add_service:
+ * @plugin:
+ * @service:
+ *
+ * TODO: Write me
+ */
 void
 cong_plugin_add_service (CongPlugin *plugin,
 			 CongService *service)
@@ -118,7 +136,14 @@ cong_plugin_add_service (CongPlugin *plugin,
 							   service);
 }
 
-
+/**
+ * cong_plugin_for_each_service:
+ * @plugin:
+ * @callback:
+ * @user_data:
+ *
+ * TODO: Write me
+ */
 void
 cong_plugin_for_each_service (CongPlugin *plugin, 
 			      void 
@@ -132,6 +157,15 @@ cong_plugin_for_each_service (CongPlugin *plugin,
 	g_list_foreach(PRIVATE(plugin)->list_of_service, (GFunc)callback, user_data);	
 }
 
+/**
+ * cong_plugin_for_each_service_of_type:
+ * @plugin:
+ * @type:
+ * @callback:
+ * @user_data:
+ *
+ * TODO: Write me
+ */
 void
 cong_plugin_for_each_service_of_type (CongPlugin *plugin, 
 				      GType type,
@@ -154,6 +188,15 @@ cong_plugin_for_each_service_of_type (CongPlugin *plugin,
 	}
 }
 
+/**
+ * cong_plugin_locate_service_by_id:
+ * @plugin:
+ * @type:
+ * @service_id:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongService*
 cong_plugin_locate_service_by_id (CongPlugin *plugin, 
 				  GType type,
@@ -168,7 +211,7 @@ cong_plugin_locate_service_by_id (CongPlugin *plugin,
 		CongService *service = CONG_SERVICE (iter->data);
 
 		if (g_type_check_instance_is_a ((GTypeInstance*)service, type)) {
-			g_message ("searching for \"%s\" found \"%s\"", service_id, cong_service_get_id (service));
+			/* g_message ("searching for \"%s\" found \"%s\"", service_id, cong_service_get_id (service)); */
 			if (0==strcmp(service_id, cong_service_get_id (service))) {
 				return service;
 			}
@@ -183,7 +226,16 @@ typedef void
 (CongServiceCallback) (CongService *service,
 		       gpointer user_data);
 
-void cong_plugin_for_each_document_factory(CongPlugin *plugin, void (*callback)(CongServiceDocumentFactory *factory, gpointer user_data), gpointer user_data)
+/**
+ * cong_plugin_for_each_document_factory:
+ * @plugin:
+ * @callback:
+ * @user_data:
+ *
+ * TODO: Write me
+ */
+void 
+cong_plugin_for_each_document_factory(CongPlugin *plugin, void (*callback)(CongServiceDocumentFactory *factory, gpointer user_data), gpointer user_data)
 {
 	g_return_if_fail (IS_CONG_PLUGIN (plugin));
 	g_return_if_fail (callback);
@@ -198,7 +250,16 @@ void cong_plugin_for_each_document_factory(CongPlugin *plugin, void (*callback)(
 #endif
 }
 
-void cong_plugin_for_each_importer(CongPlugin *plugin, void (*callback)(CongServiceImporter *importer, gpointer user_data), gpointer user_data)
+/**
+ * cong_plugin_for_each_importer:
+ * @plugin:
+ * @callback:
+ * @user_data:
+ *
+ * TODO: Write me
+ */
+void 
+cong_plugin_for_each_importer(CongPlugin *plugin, void (*callback)(CongServiceImporter *importer, gpointer user_data), gpointer user_data)
 {
 	g_return_if_fail (IS_CONG_PLUGIN (plugin));
 	g_return_if_fail (callback);
@@ -213,7 +274,16 @@ void cong_plugin_for_each_importer(CongPlugin *plugin, void (*callback)(CongServ
 #endif
 }
 
-void cong_plugin_for_each_exporter(CongPlugin *plugin, void (*callback)(CongServiceExporter *exporter, gpointer user_data), gpointer user_data)
+/**
+ * cong_plugin_for_each_exporter:
+ * @plugin:
+ * @callback:
+ * @user_data:
+ *
+ * TODO: Write me
+ */
+void 
+cong_plugin_for_each_exporter(CongPlugin *plugin, void (*callback)(CongServiceExporter *exporter, gpointer user_data), gpointer user_data)
 {
 	g_return_if_fail (IS_CONG_PLUGIN (plugin));
 	g_return_if_fail (callback);
@@ -229,7 +299,16 @@ void cong_plugin_for_each_exporter(CongPlugin *plugin, void (*callback)(CongServ
 }
 
 #if ENABLE_PRINTING
-void cong_plugin_for_each_print_method(CongPlugin *plugin, void (*callback)(CongServicePrintMethod *print_method, gpointer user_data), gpointer user_data)
+/**
+ * cong_plugin_for_each_print_method:
+ * @plugin:
+ * @callback:
+ * @user_data:
+ *
+ * TODO: Write me
+ */
+void 
+cong_plugin_for_each_print_method(CongPlugin *plugin, void (*callback)(CongServicePrintMethod *print_method, gpointer user_data), gpointer user_data)
 {
 	g_return_if_fail (IS_CONG_PLUGIN (plugin));
 	g_return_if_fail (callback);
@@ -245,7 +324,16 @@ void cong_plugin_for_each_print_method(CongPlugin *plugin, void (*callback)(Cong
 }
 #endif
 
-void cong_plugin_for_each_doc_tool(CongPlugin *plugin, void (*callback)(CongServiceDocTool *doc_tool, gpointer user_data), gpointer user_data)
+/**
+ * cong_plugin_for_each_doc_tool:
+ * @plugin:
+ * @callback:
+ * @user_data:
+ *
+ * TODO: Write me
+ */
+void 
+cong_plugin_for_each_doc_tool(CongPlugin *plugin, void (*callback)(CongServiceDocTool *doc_tool, gpointer user_data), gpointer user_data)
 {
 	g_return_if_fail (IS_CONG_PLUGIN (plugin));
 	g_return_if_fail (callback);
@@ -260,7 +348,16 @@ void cong_plugin_for_each_doc_tool(CongPlugin *plugin, void (*callback)(CongServ
 #endif
 }
 
-void cong_plugin_for_each_node_tool(CongPlugin *plugin, void (*callback)(CongServiceNodeTool *node_tool, gpointer user_data), gpointer user_data)
+/**
+ * cong_plugin_for_each_node_tool:
+ * @plugin:
+ * @callback:
+ * @user_data:
+ *
+ * TODO: Write me
+ */
+void 
+cong_plugin_for_each_node_tool(CongPlugin *plugin, void (*callback)(CongServiceNodeTool *node_tool, gpointer user_data), gpointer user_data)
 {
 	g_return_if_fail (IS_CONG_PLUGIN (plugin));
 	g_return_if_fail (callback);
@@ -277,6 +374,14 @@ void cong_plugin_for_each_node_tool(CongPlugin *plugin, void (*callback)(CongSer
 
 
 #if 1
+/**
+ * cong_plugin_locate_custom_property_dialog_by_id:
+ * @plugin:
+ * @service_id:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongServiceNodePropertyDialog*
 cong_plugin_locate_custom_property_dialog_by_id (CongPlugin *plugin, 
 						 const gchar *service_id)
@@ -297,6 +402,14 @@ cong_plugin_locate_custom_property_dialog_by_id (CongPlugin *plugin,
 	}
 }
 
+/**
+ * cong_plugin_locate_editor_node_factory_by_id:
+ * @plugin:
+ * @service_id:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongServiceEditorNodeFactory*
 cong_plugin_locate_editor_node_factory_by_id (CongPlugin *plugin,
 					      const gchar *service_id)
@@ -318,6 +431,14 @@ cong_plugin_locate_editor_node_factory_by_id (CongPlugin *plugin,
 
 }
 #else
+/**
+ * cong_plugin_locate_custom_property_dialog_by_id:
+ * @plugin:
+ * @service_id:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongServiceNodePropertyDialog*
 cong_plugin_locate_custom_property_dialog_by_id (CongPlugin *plugin, 
 						 const gchar *service_id)
@@ -338,6 +459,14 @@ cong_plugin_locate_custom_property_dialog_by_id (CongPlugin *plugin,
 	return NULL;
 }
 
+/**
+ * cong_plugin_locate_editor_node_factory_by_id:
+ * @plugin:
+ * @service_id:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongServiceEditorNodeFactory*
 cong_plugin_locate_editor_node_factory_by_id (CongPlugin *plugin,
 					      const gchar *service_id)
@@ -359,7 +488,30 @@ cong_plugin_locate_editor_node_factory_by_id (CongPlugin *plugin,
 }
 #endif
 
-gchar* cong_plugin_get_gconf_namespace(CongPlugin *plugin)
+/**
+ * cong_plugin_get_id:
+ * @plugin:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+const gchar* 
+cong_plugin_get_id (CongPlugin *plugin)
+{
+	g_return_val_if_fail (IS_CONG_PLUGIN (plugin), NULL);
+
+	return PRIVATE (plugin)->plugin_id;
+}
+
+/**
+ * cong_plugin_get_gconf_namespace:
+ * @plugin:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+gchar* 
+cong_plugin_get_gconf_namespace(CongPlugin *plugin)
 {
 	g_return_val_if_fail (IS_CONG_PLUGIN (plugin), NULL);
 
@@ -368,7 +520,8 @@ gchar* cong_plugin_get_gconf_namespace(CongPlugin *plugin)
 	return g_strdup_printf( (CONG_GCONF_PATH "plugins/%s"), PRIVATE (plugin)->plugin_id);
 }
 
-gchar* cong_plugin_get_gconf_key(CongPlugin *plugin, const gchar *local_part)
+gchar* 
+cong_plugin_get_gconf_key(CongPlugin *plugin, const gchar *local_part)
 {
 	g_return_val_if_fail (IS_CONG_PLUGIN (plugin), NULL);
 	g_return_val_if_fail (local_part, NULL);
@@ -378,11 +531,17 @@ gchar* cong_plugin_get_gconf_key(CongPlugin *plugin, const gchar *local_part)
 	return g_strdup_printf( (CONG_GCONF_PATH "plugins/%s/%s"), PRIVATE (plugin)->plugin_id, local_part);
 }
 
-
-
+/**
+ * cong_ui_new_document_from_manufactured_xml:
+ * @xml_doc:
+ * @parent_window:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongDocument*
-cong_ui_new_document_from_manufactured_xml(xmlDocPtr xml_doc,
-					   GtkWindow *parent_window)
+cong_ui_new_document_from_manufactured_xml (xmlDocPtr xml_doc,
+					    GtkWindow *parent_window)
 {
 	CongDocument *cong_doc;
 	CongDispspec *ds;
@@ -393,6 +552,7 @@ cong_ui_new_document_from_manufactured_xml(xmlDocPtr xml_doc,
 							      xml_doc,
 							      NULL);
 
+	#if 0
 	if (ds == NULL) {
 		ds = query_for_forced_dispspec (_("Conglomerate could not find display information for the new file"), 
 						xml_doc, 
@@ -404,18 +564,47 @@ cong_ui_new_document_from_manufactured_xml(xmlDocPtr xml_doc,
 			return NULL;
 		}
 	}
+	#endif
 	
 	g_assert(xml_doc);
-	g_assert(ds);
 
 	cong_doc = cong_document_new_from_xmldoc(xml_doc, ds, NULL);
 
+	/* Clean up the xml_doc; eventually there might be a user preference for this */
+	{
+		CongCommand *command;
+		CongSourceCleanupOptions options;
+
+		/* FIXME: get user preferences about preferred XML layout */
+		options.use_tabs = TRUE;
+		options.wrap_text = TRUE;
+		options.num_text_columns = 80;
+		
+		command = cong_document_begin_command (cong_doc,
+						       "",
+						       NULL);
+		
+		cong_command_add_cleanup_source (command,
+						 &options);
+		
+		cong_document_end_preprocessor_command (cong_doc,
+							command);
+	}
+
 	cong_primary_window_new(cong_doc);
 	g_object_unref(G_OBJECT(cong_doc));
 
 	return cong_doc;
 }
 
+/**
+ * cong_ui_new_document_from_imported_xml:
+ * @xml_doc:
+ * @parent_window:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongDocument*
 cong_ui_new_document_from_imported_xml(xmlDocPtr xml_doc,
 				       GtkWindow *parent_window)
@@ -452,13 +641,83 @@ cong_ui_new_document_from_imported_xml(xmlDocPtr xml_doc,
 	return cong_doc;
 }
 
-xmlDocPtr cong_ui_transform_doc(CongDocument *doc,
-				const gchar *stylesheet_filename,
-				GtkWindow *toplevel_window)
+
+static char**
+make_libxslt_params (GList *list_of_parameters)
+{
+	int num_params = g_list_length (list_of_parameters);
+	char **result = g_malloc0 (sizeof(char*) * ((num_params*2)+1));
+	int i;
+	GList *iter;
+
+	for (i=0, iter=list_of_parameters; iter; iter=iter->next, i++) {
+		CongStylesheetParameter *param = (CongStylesheetParameter*)iter->data;
+
+		/* shallow copy */
+		result[(2*i)] = param->name;
+		result[(2*i)+1] = param->value;
+	}
+	result[(2*i)] = NULL;
+
+	return result;
+}
+
+static void
+free_libxslt_params (char **libxslt_params)
+{
+	/* it was only a shallow copy */
+	g_free (libxslt_params);
+}
+
+static void
+set_window_cursor_helper (gpointer primary_window, gpointer cursor)
+{
+	gdk_window_set_cursor (((CongPrimaryWindow *)primary_window)->window->window,
+			       (GdkCursor *)cursor );
+}
+
+static void
+set_window_cursor (GdkCursor *cursor)
+{
+	/*
+	 * It seems that we need the gdk_flush() to ensure the
+	 * change happens before calling something processor-intensive
+	 */
+	g_list_foreach ( cong_app_singleton()->primary_windows,
+			 set_window_cursor_helper,
+			 cursor );
+	gdk_flush();
+}
+
+/**
+ * cong_ui_transform_doc:
+ * @doc:
+ * @stylesheet_filename:
+ * @list_of_parameters: a #GList of #CongStylesheetParameter
+ * @toplevel_window:
+ *
+ * TODO: Write me
+ *
+ * The routine sets all primary windows of Conglomerate to have a busy cursor whilst
+ * the processing occurs. It will not pick up any dialog, or other, windows
+ * that may have been created by Conglomerate. At the moment this is not a problem,
+ * but it may be in the future. It may be possible to use GTK (or GDK) functions
+ * to fid all the windows that are currently controlled by Conglomerate, and
+ * so avoid this issue.
+ *
+ * Returns:
+ */
+xmlDocPtr 
+cong_ui_transform_doc(CongDocument *doc,
+		      const gchar *stylesheet_filename,
+		      GList *list_of_parameters,
+		      GtkWindow *toplevel_window)
 {
 	xsltStylesheetPtr xsl;
 	xmlDocPtr input_clone;
 	xmlDocPtr result;
+	char **libxslt_params;
+	GdkCursor *busy_cursor;
 
 	g_return_val_if_fail (doc, NULL);
 	g_return_val_if_fail (stylesheet_filename, NULL);
@@ -466,15 +725,20 @@ xmlDocPtr cong_ui_transform_doc(CongDocument *doc,
 	xsl = xsltParseStylesheetFile(stylesheet_filename);
 
 	if (NULL==xsl) {
-		gchar *why_failed = g_strdup_printf(_("There was a problem reading the stylesheet file \"%s\""),stylesheet_filename);
+		/*
+		 * Since we can not assume that these are the DocBook stylesheets we can not
+		 * be too precise with the error message. The message below assumes that whatever stylesheet
+		 * was used was accessed via the XML catalog mechanism, hence the note about the catalog
+		 * file. It would be nice to make the URL an actual link but that is a lot more work.
+		 */
+		gchar *why_failed = g_strdup_printf (_("There was a problem reading the stylesheet file \"%s\""),stylesheet_filename);
+		GtkDialog* dialog = cong_error_dialog_new (toplevel_window,
+							   _("Conglomerate could not transform the document"),
+							   why_failed,
+							   _("Since Conglomerate could not read the stylesheet it is possible that your XML catalog file is set up incorrectly. See http://www.xmlsoft.org/catalog.html for more information on catalogs."));
 
-		GtkDialog* dialog = cong_error_dialog_new(toplevel_window,
-							  _("Conglomerate could not transform the document"),
-							  why_failed,
-							  "FIXME");
-	
-		cong_error_dialog_run(GTK_DIALOG(dialog));
-		gtk_widget_destroy(GTK_WIDGET(dialog));
+		cong_error_dialog_run (GTK_DIALOG(dialog));
+		gtk_widget_destroy (GTK_WIDGET(dialog));
 		return NULL;
 	}
 
@@ -484,77 +748,152 @@ xmlDocPtr cong_ui_transform_doc(CongDocument *doc,
 	input_clone = xmlCopyDoc(cong_document_get_xml(doc), TRUE);
 	g_assert(input_clone);
 
-	result = xsltApplyStylesheet(xsl, input_clone, NULL);
-	g_assert(result);
+	cong_stylesheet_parameter_list_debug (list_of_parameters);
+	libxslt_params = make_libxslt_params (list_of_parameters);
 
-	xmlFreeDoc(input_clone);
+	busy_cursor = gdk_cursor_new(GDK_WATCH);
+	set_window_cursor (busy_cursor);
 
-	if (result->children==NULL) {
-		gchar *why_failed = g_strdup_printf(_("There was a problem applying the stylesheet file"));
+	result = xsltApplyStylesheet(xsl, input_clone, (const char**)libxslt_params);
 
-		GtkDialog* dialog = cong_error_dialog_new(toplevel_window,
-							  _("Conglomerate could not transform the document"),
-							  why_failed,
-							  "FIXME");
+	set_window_cursor (NULL);
+	gdk_cursor_unref (busy_cursor);
+
+	/* g_assert(result); */
+	if (result!=NULL && result->children==NULL) {
+		/* is this necessary? */
+		xmlFreeDoc(result);
+		result = NULL;
+	}
+	if (result==NULL) {
+		gchar *why_failed = g_strdup_printf (_("There was a problem applying the stylesheet file \"%s\" to \"%s\""),
+						     stylesheet_filename,
+						     cong_document_get_filename(doc));
+#ifdef SILLY_IDEA
+		if (list_of_parameters!=NULL) {
+			/*
+			 * add the stylesheet parameters to the dialog. This is probably a bad idea if the list 
+			 * contains more than a few items. Two problems when there are a large number of 
+			 * parameters are:
+			 *   1 - the loop below is inefficient in this situation
+			 *   2 - the dialog will be too large to read effectively
+			 *
+			 * could use the libxslt_params array rather than the list_of_parameters since this
+			 * can be used by g_strjoinv() and other related functions.
+			 */
+			GList *iter;
+			CongStylesheetParameter *param;
+			gchar *temp = g_strconcat (why_failed, _("\n\nThe following stylesheet parameters were used:\n"));
+			g_free (why_failed);
+			why_failed = temp;
+
+			for (iter=list_of_parameters; iter; iter=iter->next) {
+				param = (CongStylesheetParameter *)iter->data;
+				temp = g_strdup_printf ("%s  %s\t%s\n", why_failed, param->name, param->value);
+				g_free (why_failed);
+				why_failed = temp;
+			}
+		}
+#endif
+		GtkDialog* dialog = cong_error_dialog_new (toplevel_window,
+							   _("Conglomerate could not transform the document"),
+							   why_failed,
+							   "");
 	
-		cong_error_dialog_run(GTK_DIALOG(dialog));
-		gtk_widget_destroy(GTK_WIDGET(dialog));
-		return NULL;
+		cong_error_dialog_run (GTK_DIALOG(dialog));
+		gtk_widget_destroy (GTK_WIDGET(dialog));
 	}
 
+	xmlFreeDoc(input_clone);
+	free_libxslt_params (libxslt_params);
 	xsltFreeStylesheet(xsl);
 
 	return result;
 }
 
-void cong_ui_transform_doc_to_uri(CongDocument *doc,
-				  const gchar *stylesheet_filename,
-				  const gchar *string_uri,
-				  GtkWindow *toplevel_window)
+/**
+ * cong_ui_transform_doc_to_uri:
+ * @doc:
+ * @stylesheet_filename:
+ * @list_of_parameters: a #GList of #CongStylesheetParameter
+ * @string_uri:
+ * @toplevel_window:
+ *
+ * Applies the stylesheet (@stylesheet_filename) to the
+ * document (@doc) and saves the output to @string_uri.
+ * 
+ * At present, it is assumed that a dialog window will be
+ * created if there is an error in the processing (e.g.
+ * unable to find the stylesheet or write to the given URI).
+ * No dialog is created on success (so that the calling routine
+ * can customise such a dialog, or not have one).
+ *
+ * It also relies on the #cong_ui_transform_doc() routines for
+ * providing feed-back to the user that something is happening.
+ *
+ * Returns: true on success, false on failure.
+ */
+gboolean
+cong_ui_transform_doc_to_uri(CongDocument *doc,
+			     const gchar *stylesheet_filename,
+			     GList *list_of_parameters,
+			     const gchar *string_uri,
+			     GtkWindow *toplevel_window)
 {
 	xmlDocPtr doc_ptr;
 	GnomeVFSURI *vfs_uri;
 	GnomeVFSResult vfs_result;
 	GnomeVFSFileSize file_size;
 
-	g_return_if_fail (doc);
-	g_return_if_fail (stylesheet_filename);
-	g_return_if_fail (string_uri);
-
-	/* FIXME:  need some kind of feedback e.g. a busy cursor */
+	g_return_val_if_fail (doc, 0);
+	g_return_val_if_fail (stylesheet_filename, 0);
+	g_return_val_if_fail (string_uri, 0);
 
 	doc_ptr = cong_ui_transform_doc(doc,
 					stylesheet_filename,
+					list_of_parameters,
 					toplevel_window);
-
-	if (doc_ptr) {
-		vfs_uri = gnome_vfs_uri_new(string_uri);
-	
-		vfs_result = cong_vfs_save_xml_to_uri (doc_ptr, 
-						       vfs_uri,	
-						       &file_size);
-		
-		if (vfs_result != GNOME_VFS_OK) {
-			GtkDialog* dialog = cong_error_dialog_new_from_file_save_failure(toplevel_window,
-											 string_uri, 
-											 vfs_result, 
-											 &file_size);
-			
-			cong_error_dialog_run(GTK_DIALOG(dialog));
-			gtk_widget_destroy(GTK_WIDGET(dialog));
-		}
-		
-		gnome_vfs_uri_unref(vfs_uri);
-
-		xmlFreeDoc(doc_ptr);
+	if (doc_ptr == NULL) {
+		return 0;
 	}
+
+	vfs_uri = gnome_vfs_uri_new(string_uri);
+	vfs_result = cong_vfs_save_xml_to_uri (doc_ptr, 
+					       vfs_uri,	
+					       &file_size);
+		
+	if (vfs_result != GNOME_VFS_OK) {
+		GtkDialog* dialog = cong_error_dialog_new_from_file_save_failure(toplevel_window,
+										 string_uri, 
+										 vfs_result, 
+										 &file_size);
+			
+		cong_error_dialog_run(GTK_DIALOG(dialog));
+		gtk_widget_destroy(GTK_WIDGET(dialog));
+		return 0;
+	}
+		
+	gnome_vfs_uri_unref(vfs_uri);
+	xmlFreeDoc(doc_ptr);
+	return 1;
 }
 
 /* Handy methods for "Import" methods; doing the necessary UI hooks: */
-gboolean cong_ui_load_imported_file_content(const gchar *string_uri,
-					    char** buffer,
-					    GnomeVFSFileSize* size,
-					    GtkWindow *parent_window)
+/**
+ * cong_ui_load_imported_file_content:
+ * @uri:
+ * @buffer:
+ * @size:
+ * @parent_window:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+gboolean 
+cong_ui_load_imported_file_content(const gchar *string_uri,
+				   char** buffer,
+				   GnomeVFSFileSize* size,
+				   GtkWindow *parent_window)
 {
 	GnomeVFSResult vfs_result;
 
@@ -580,8 +919,21 @@ gboolean cong_ui_load_imported_file_content(const gchar *string_uri,
 	return TRUE;
 }
 
-
 /* Registration methods for various services: */
+/**
+ * cong_plugin_register_document_factory:
+ * @plugin:
+ * @name:
+ * @description:
+ * @id:
+ * @page_creation_callback:
+ * @action_callback:
+ * @icon:
+ * @user_data:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongServiceDocumentFactory*
 cong_plugin_register_document_factory (CongPlugin *plugin, 
 				       const gchar *name, 
@@ -617,6 +969,22 @@ cong_plugin_register_document_factory (CongPlugin *plugin,
 	return factory;
 }
 
+/**
+ * cong_plugin_register_doc_tool:
+ * @plugin:
+ * @name:
+ * @description:
+ * @service_id:
+ * @menu_text:
+ * @tooltip_text:
+ * @tooltip_further_text:
+ * @doc_filter:
+ * @action_callback:
+ * @user_data:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongServiceDocTool*
 cong_plugin_register_doc_tool (CongPlugin *plugin,
 			       const gchar *name, 
@@ -654,6 +1022,22 @@ cong_plugin_register_doc_tool (CongPlugin *plugin,
 
 }
 
+/**
+ * cong_plugin_register_node_tool:
+ * @plugin:
+ * @name:
+ * @description:
+ * @service_id:
+ * @menu_text:
+ * @tooltip_text:
+ * @tooltip_further_text:
+ * @node_filter:
+ * @action_callback:
+ * @user_data:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongServiceNodeTool*
 cong_plugin_register_node_tool (CongPlugin *plugin,
 				const gchar *name, 
@@ -691,12 +1075,25 @@ cong_plugin_register_node_tool (CongPlugin *plugin,
 
 }
 
-CongServiceEditorNodeFactory *cong_plugin_register_editor_node_factory(CongPlugin *plugin, 
-								      const gchar *name, 
-								      const gchar *description,
-								      const gchar *service_id,
-								      CongEditorNodeFactoryMethod factory_method,
-								      gpointer user_data)
+/**
+ * cong_plugin_register_editor_node_factory:
+ * @plugin:
+ * @name:
+ * @description:
+ * @plugin_id:
+ * @factory_method:
+ * @user_data:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+CongServiceEditorNodeFactory *
+cong_plugin_register_editor_node_factory(CongPlugin *plugin, 
+					 const gchar *name, 
+					 const gchar *description,
+					 const gchar *service_id,
+					 CongEditorNodeFactoryMethod factory_method,
+					 gpointer user_data)
 {
 	CongServiceEditorNodeFactory *editor_node_factory;
 
@@ -719,13 +1116,27 @@ CongServiceEditorNodeFactory *cong_plugin_register_editor_node_factory(CongPlugi
 	return editor_node_factory;
 }
 
-
+/**
+ * cong_plugin_register_exporter:
+ * @plugin:
+ * @name:
+ * @description:
+ * @id:
+ * @doc_filter:
+ * @options_widget_callback:
+ * @action_callback:
+ * @user_data:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongServiceExporter*
 cong_plugin_register_exporter (CongPlugin *plugin, 
 			       const gchar *name, 
 			       const gchar *description,
 			       const gchar *service_id,
 			       CongServiceExporterDocumentFilter doc_filter,
+			       CongServiceExporterOptionsWidgetCallback options_widget_callback,
 			       CongServiceExporterActionCallback action_callback,
 			       gpointer user_data)
 {
@@ -743,6 +1154,7 @@ cong_plugin_register_exporter (CongPlugin *plugin,
 						    description,
 						    service_id,
 						    doc_filter,
+						    options_widget_callback,
 						    action_callback,
 						    user_data);
 
@@ -752,12 +1164,27 @@ cong_plugin_register_exporter (CongPlugin *plugin,
 	return exporter;
 }
 
+/**
+ * cong_plugin_register_importer:
+ * @plugin:
+ * @name:
+ * @description:
+ * @id:
+ * @filter_factory_callback:
+ * @options_widget_callback:
+ * @action_callback:
+ * @user_data:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongServiceImporter*
 cong_plugin_register_importer (CongPlugin *plugin, 
 			       const gchar *name, 
 			       const gchar *description,
-			       const gchar *service_id,
-			       CongServiceImporterMimeFilter mime_filter,
+			       const gchar *service_id,	
+			       CongServiceImporterMakeFilterCallback filter_factory_callback,
+			       CongServiceImporterOptionsWidgetCallback options_widget_callback,
 			       CongServiceImporterActionCallback action_callback,
 			       gpointer user_data)
 {
@@ -767,14 +1194,15 @@ cong_plugin_register_importer (CongPlugin *plugin,
 	g_return_val_if_fail (name, NULL);
 	g_return_val_if_fail (description, NULL);
 	g_return_val_if_fail (service_id, NULL);
-	g_return_val_if_fail (mime_filter, NULL);
+	g_return_val_if_fail (filter_factory_callback, NULL);
 	g_return_val_if_fail (action_callback, NULL);
 
         importer = cong_service_importer_construct (g_object_new (CONG_SERVICE_IMPORTER_TYPE, NULL),
 						    name,
 						    description,
 						    service_id,
-						    mime_filter,
+						    filter_factory_callback,
+						    options_widget_callback,
 						    action_callback,
 						    user_data);
 
@@ -784,6 +1212,18 @@ cong_plugin_register_importer (CongPlugin *plugin,
 	return importer;
 }
 
+/**
+ * cong_plugin_register_custom_property_dialog:
+ * @plugin:
+ * @name:
+ * @description:
+ * @service_id:
+ * @factory_method:
+ * @user_data:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongServiceNodePropertyDialog*
 cong_plugin_register_custom_property_dialog (CongPlugin *plugin,
 					     const gchar *name, 
@@ -813,6 +1253,17 @@ cong_plugin_register_custom_property_dialog (CongPlugin *plugin,
 
 }
 
+/**
+ * cong_plugin_register_custom_property_dialog_for_element:
+ * @plugin:
+ * @element_name:
+ * @service_id:
+ * @factory_method:
+ * @user_data:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongServiceNodePropertyDialog*
 cong_plugin_register_custom_property_dialog_for_element (CongPlugin *plugin,
 							 const gchar *element_name,
@@ -847,6 +1298,19 @@ cong_plugin_register_custom_property_dialog_for_element (CongPlugin *plugin,
 }
 
 #if ENABLE_PRINTING
+/**
+ * cong_plugin_register_print_method:
+ * @plugin:
+ * @name:
+ * @description:
+ * @id:
+ * @doc_filter:
+ * @action_callback:
+ * @user_data:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 CongServicePrintMethod*
 cong_plugin_register_print_method (CongPlugin *plugin, 
 				   const gchar *name, 
