@@ -85,36 +85,15 @@
  * 
  */
 
-struct CongPrimaryWindow
-{
-	CongDocument *doc;
-
-/*  	GtkWidget *w; */
-
-	GtkWidget *cong_editor_widget3;
-	GtkWidget *scroller3;
-
-	GtkWidget *window, *menus;
-	GtkToolbar *toolbar;
 #if 1
-	GtkWidget *app_bar;
-#else
-	GtkWidget *status;
-#endif
-	GtkWidget *auth, *butt_submit, *butt_find;
-	
-#if 0
-	guint status_main_ctx;
-#endif
-
-	GtkAccelGroup *accel;
-
-
-
-};
-
-#if 1
-GtkWidget* cong_gui_get_a_window(void)
+/**
+ * cong_gui_get_a_window:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+GtkWidget* 
+cong_gui_get_a_window(void)
 {
 	CongPrimaryWindow *primary_window;
 
@@ -132,49 +111,104 @@ GtkWidget* cong_gui_get_a_window(void)
 	}
 }
 #else
-GtkWidget* cong_gui_get_window(struct cong_gui* gui)
+/**
+ * cong_gui_get_window:
+ * @gui:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+GtkWidget* 
+cong_gui_get_window(struct cong_gui* gui)
 {
 	g_assert(gui!=NULL);
 
 	return GTK_WIDGET(gui->window);
 }
 
-GtkWidget* cong_gui_get_popup(struct cong_gui* gui)
+/**
+ * cong_gui_get_popup:
+ * @gui:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+GtkWidget* 
+cong_gui_get_popup(struct cong_gui* gui)
 {
 	g_assert(gui!=NULL);
 
 	return gui->popup;
 }
 
-void cong_gui_set_popup(struct cong_gui* gui, GtkWidget* popup)
+/**
+ * cong_gui_set_popup:
+ * @gui:
+ * @popup:
+ *
+ * TODO: Write me
+ */
+void 
+cong_gui_set_popup(struct cong_gui* gui, GtkWidget* popup)
 {
 	g_assert(gui!=NULL);
 
 	gui->popup=popup;
 }
 
-GtkWidget* cong_gui_get_button_submit(struct cong_gui* gui)
+/**
+ * cong_gui_get_button_submit:
+ * @gui:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+GtkWidget* 
+cong_gui_get_button_submit(struct cong_gui* gui)
 {
 	g_assert(gui!=NULL);
 
 	return gui->butt_submit;
 }
 
-GtkTreeStore* cong_gui_get_tree_store(struct cong_gui* gui)
+/**
+ * cong_gui_get_tree_store:
+ * @gui:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+GtkTreeStore* 
+cong_gui_get_tree_store(struct cong_gui* gui)
 {
 	g_assert(gui!=NULL);
 
 	return gui->global_tree_store;
 }
 
-GtkWidget* cong_gui_get_root(struct cong_gui* gui)
+/**
+ * cong_gui_get_root:
+ * @gui:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+GtkWidget* 
+cong_gui_get_root(struct cong_gui* gui)
 {
 	g_assert(gui!=NULL);
 
 	return gui->root;
 }
 
-void cong_gui_destroy_tree_store(struct cong_gui* gui)
+/**
+ * cong_gui_destroy_tree_store:
+ * @gui:
+ *
+ * TODO: Write me
+ */
+void 
+cong_gui_destroy_tree_store(struct cong_gui* gui)
 {
 	gui->global_tree_store = gtk_tree_store_new (TREEVIEW_N_COLUMNS, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_POINTER, G_TYPE_STRING, G_TYPE_STRING);
 	gtk_tree_view_set_model(gui->global_tree_view, GTK_TREE_MODEL(gui->global_tree_store));
@@ -189,15 +223,16 @@ void cong_gui_destroy_tree_store(struct cong_gui* gui)
  
 struct CongApp the_app;
 
-extern char *ilogo_xpm[];
-extern char *auth_off_xpm[];
-extern char *icon_connect[];
-extern char *icon_new[];
-extern char *icon_assemble[];
-extern char *icon_openfile[];
-extern char *icon_submit[];
-
-GtkPixmap* cong_primary_window_create_pixmap(CongPrimaryWindow *primary_window, char** xpm)
+/**
+ * cong_primary_window_create_pixmap:
+ * @primary_window:
+ * @xpm:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+GtkPixmap* 
+cong_primary_window_create_pixmap(CongPrimaryWindow *primary_window, char** xpm)
 {
 	GdkPixmap *p;
 	GdkBitmap *mask;
@@ -213,202 +248,15 @@ GtkPixmap* cong_primary_window_create_pixmap(CongPrimaryWindow *primary_window,
 	return GTK_PIXMAP(gtk_pixmap_new(p, mask));
 }
 
-gint toolbar_callback_undo(GtkWidget *w, gpointer data)
-{
-	CongPrimaryWindow *primary_window = data;
-	CongDocument *doc = cong_primary_window_get_document(primary_window);
-
-	cong_document_undo(doc);
-
-	return TRUE;
-}
-
-gint toolbar_callback_redo(GtkWidget *w, gpointer data)
-{
-	CongPrimaryWindow *primary_window = data;
-	CongDocument *doc = cong_primary_window_get_document(primary_window);
-
-	cong_document_redo(doc);
-
-	return TRUE;
-}
-
-gint toolbar_callback_cut(GtkWidget *w, gpointer data)
-{
-	CongPrimaryWindow *primary_window = data;
-	CongDocument *doc = cong_primary_window_get_document(primary_window);
-
-	cong_document_cut_selection(doc);
-
-	return TRUE;
-}
-
-gint toolbar_callback_copy(GtkWidget *w, gpointer data)
-{
-	CongPrimaryWindow *primary_window = data;
-	CongDocument *doc = cong_primary_window_get_document(primary_window);
-
-	cong_document_copy_selection(doc);
-
-	return TRUE;
-}
-
-gint toolbar_callback_paste(GtkWidget *w, gpointer data)
-{
-	CongPrimaryWindow *primary_window = data;
-	CongDocument *doc = cong_primary_window_get_document(primary_window);
-
-	cong_document_paste_clipboard_or_selection(doc, w);
-
-	return TRUE;
-}
-
-
-static void 
-on_history_changed_undo (CongCommandHistory *history,
-			 gpointer user_data)
-{
-	GtkWidget *undo = GTK_WIDGET (user_data);
-	
-	gtk_widget_set_sensitive (undo,
-				  cong_command_history_can_undo (history));
-}
-
-static void 
-on_history_changed_redo (CongCommandHistory *history,
-			 gpointer user_data)
-{
-	GtkWidget *redo = GTK_WIDGET (user_data);
-	
-	gtk_widget_set_sensitive (redo,
-				  cong_command_history_can_redo (history));	
-}
-
-static void 
-on_selection_changed_copy (CongDocument *document,
-			 gpointer user_data)
-{
-	GtkWidget *copy = GTK_WIDGET (user_data);
-	
-        CongSelection *selection = cong_document_get_selection(document);
-        CongRange *range = cong_selection_get_ordered_range(selection);
-        gtk_widget_set_sensitive (copy,
-                                  cong_range_can_be_copied (range));
-}
-
-static void 
-on_selection_changed_cut (CongDocument *document,
-			 gpointer user_data)
-{
-	GtkWidget *cut = GTK_WIDGET (user_data);
-	
-        CongSelection *selection = cong_document_get_selection(document);
-        CongRange *range = cong_selection_get_ordered_range(selection);
-        gtk_widget_set_sensitive (cut,
-                                  cong_range_can_be_cut (range));
-}
-
-void cong_primary_window_toolbar_populate(CongPrimaryWindow *primary_window)
-{
-	if ( !(primary_window->doc) ) {
-		gtk_toolbar_insert_stock(primary_window->toolbar, 
-				 GTK_STOCK_OPEN,
-				 _("Open document"),
-				 _("Open document"),
-				 GTK_SIGNAL_FUNC(toolbar_callback_open),
-				 primary_window,
-				 -1);
-		
-		gtk_toolbar_insert_stock(primary_window->toolbar,
-				GTK_STOCK_NEW,
-				_("New document"),
-				_("New document"),
-				GTK_SIGNAL_FUNC(toolbar_callback_new),
-				primary_window,
-				-1);
-
-	}
-
-	if (primary_window->doc) {
-		CongDocument *doc = cong_primary_window_get_document (primary_window);
-		CongCommandHistory *history = cong_document_get_command_history (doc);
-		GtkWidget *undo;
-		GtkWidget *redo;
-		GtkWidget *copy;
-		GtkWidget *cut;
-		GtkWidget *paste;
-
-		gtk_toolbar_insert_stock(primary_window->toolbar, 
-					 GTK_STOCK_SAVE,
-					 _("Save document"),
-					 _("Save document"), 
-					 GTK_SIGNAL_FUNC(toolbar_callback_save),
-					 primary_window,
-					 -1);
-		gtk_toolbar_append_space(primary_window->toolbar);
-		undo = gtk_toolbar_insert_stock(primary_window->toolbar, 
-						GTK_STOCK_UNDO,
-						_("Undo"),
-						_("Undo"), 
-						GTK_SIGNAL_FUNC(toolbar_callback_undo),
-						primary_window,
-						-1);
-		redo = gtk_toolbar_insert_stock(primary_window->toolbar, 
-						GTK_STOCK_REDO,
-						_("Redo"),
-						_("Redo"), 
-						GTK_SIGNAL_FUNC(toolbar_callback_redo),
-						primary_window,
-						-1);
-		g_signal_connect (G_OBJECT(history),
-				  "changed",
-				  G_CALLBACK(on_history_changed_undo),
-				  undo);
-		g_signal_connect (G_OBJECT(history),
-				  "changed",
-				  G_CALLBACK(on_history_changed_redo),
-				  redo);		
-		gtk_widget_set_sensitive (undo, FALSE);
-		gtk_widget_set_sensitive (redo, FALSE);
-
-		gtk_toolbar_append_space(primary_window->toolbar);
-		cut = gtk_toolbar_insert_stock(primary_window->toolbar, 
-					 GTK_STOCK_CUT,
-					 _("Cut"),
-					 _("Cut"), 
-					 GTK_SIGNAL_FUNC(toolbar_callback_cut),
-					 primary_window,
-					 -1);
-		copy = gtk_toolbar_insert_stock(primary_window->toolbar, 
-					 GTK_STOCK_COPY,
-					 _("Copy"),
-					 _("Copy"), 
-					 GTK_SIGNAL_FUNC(toolbar_callback_copy),
-					 primary_window,
-					 -1);
-		paste = gtk_toolbar_insert_stock(primary_window->toolbar, 
-					 GTK_STOCK_PASTE,
-					 _("Paste"),
-					 _("Paste"), 
-					 GTK_SIGNAL_FUNC(toolbar_callback_paste),
-					 primary_window,
-					 -1);
-		g_signal_connect (G_OBJECT(doc),
-				  "selection_change",
-				  G_CALLBACK(on_selection_changed_copy),
-				  copy);
-		g_signal_connect (G_OBJECT(doc),
-				  "selection_change",
-				  G_CALLBACK(on_selection_changed_cut),
-				  cut);		
-		/* FIXME: What signal to connect paste to in order to update sensitivity? */
-		gtk_widget_set_sensitive (copy, FALSE);
-		gtk_widget_set_sensitive (cut, FALSE);
-		gtk_widget_set_sensitive (paste, cong_document_can_paste(doc));
-	}
-}
-
-gboolean cong_primary_window_can_close(CongPrimaryWindow *primary_window)
+/**
+ * cong_primary_window_can_close:
+ * @primary_window:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+gboolean 
+cong_primary_window_can_close(CongPrimaryWindow *primary_window)
 {
 	g_assert(primary_window);
 
@@ -457,19 +305,35 @@ gboolean cong_primary_window_can_close(CongPrimaryWindow *primary_window)
 	return TRUE;
 }
 
-
-gint delete_event( GtkWidget *widget,
-		   GdkEvent  *event,
-		   gpointer   data )
+/**
+ * delete_event:
+ * @widget:
+ * @event:
+ * @data:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+gint 
+delete_event( GtkWidget *widget,
+	      GdkEvent  *event,
+	      gpointer   data )
 {
 	/* If the window can't close, return TRUE so the callback chain
 	   ends here */
 	return ! cong_primary_window_can_close((CongPrimaryWindow *)data);
 }
 
-
-void destroy( GtkWidget *widget,
-	      gpointer   data )
+/**
+ * destroy:
+ * @widget:
+ * @data:
+ *
+ * TODO: Write me
+ */
+void 
+destroy( GtkWidget *widget,
+	 gpointer   data )
 {
 	cong_primary_window_free((CongPrimaryWindow *)data);
 	if (g_list_length(cong_app_singleton()->primary_windows) == 0) {		
@@ -477,91 +341,157 @@ void destroy( GtkWidget *widget,
 	}
 }
 
+static void
+add_standard_layout_for_doc (CongPrimaryWindow *primary_window, 
+			     CongDocument *doc)
+{
+	GtkWidget *w1 = NULL, *w2 = NULL;
+	GtkWidget *sidebar_notebook = NULL;
+
+	/* --- Main window -> hpane --- */
+	w1 = gtk_hpaned_new();
+	gnome_app_set_contents(GNOME_APP(primary_window->window),w1);
+	gtk_widget_show(w1);
+	
+	/* --- Notebook to appear in the sidebar: --- */
+	sidebar_notebook = gtk_notebook_new();
+	gtk_widget_show(sidebar_notebook);
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(sidebar_notebook), GTK_POS_BOTTOM);
+	
+	gtk_paned_add1(GTK_PANED(w1), sidebar_notebook);
+	
+	/* --- Tree view --- */
+	LOG_PRIMARY_WINDOW_CREATION1 ("Creating overview");
+	
+	w2 = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(w2), GTK_POLICY_AUTOMATIC,
+				       GTK_POLICY_AUTOMATIC);
+	gtk_widget_set_size_request(GTK_WIDGET(w2), 100, 0);
+	gtk_widget_show(w2);
+	
+	gtk_notebook_append_page(GTK_NOTEBOOK(sidebar_notebook),
+				 w2,
+				 gtk_label_new(_("Overview"))
+				 );
+	
+	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(w2), 
+					      cong_tree_view_get_widget (cong_overview_view_new (primary_window->doc,
+												 primary_window)));
+	
+	/* --- Bookmark view --- */
+#if 0
+	gtk_notebook_append_page(GTK_NOTEBOOK(sidebar_notebook),
+				 cong_bookmark_view_new(primary_window->doc),
+				 gtk_label_new(_("Bookmarks"))
+				 );
+	
+#endif
+	
+	/* --- Raw XML view --- */
+	LOG_PRIMARY_WINDOW_CREATION1 ("Creating raw XML view");
+	gtk_notebook_append_page (GTK_NOTEBOOK(sidebar_notebook),
+				  cong_dom_view_new(primary_window->doc,
+						    primary_window),
+				  gtk_label_new(_("Raw XML"))
+				  );
+	
+		/* Set up the editor_widget v3: */
+#if ENABLE_MAIN_WIDGET
+	{
+		LOG_PRIMARY_WINDOW_CREATION1 ("Creating v3 widget");
+		primary_window->cong_editor_widget3 = cong_editor_widget3_new(doc,
+									      primary_window);
+		
+		/* --- Scrolling area for editor widget 3--- */
+		primary_window->scroller3 = gtk_scrolled_window_new(NULL, NULL);
+		
+		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(primary_window->scroller3), 
+						GTK_POLICY_AUTOMATIC,
+						GTK_POLICY_ALWAYS);
+		
+		gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(primary_window->scroller3), 
+						       GTK_WIDGET(primary_window->cong_editor_widget3));
+		gtk_widget_show (primary_window->cong_editor_widget3);
+		gtk_widget_show(primary_window->scroller3);
+		
+		gtk_paned_add2 (GTK_PANED(w1), 
+				primary_window->scroller3);
+	}
+#endif
+	
+}
+
+static void
+add_tree_layout_for_doc (CongPrimaryWindow *primary_window, 
+			 CongDocument *doc)
+{
+	GtkWidget *widget = cong_dom_view_new (primary_window->doc,
+					       primary_window);
+	gnome_app_set_contents(GNOME_APP(primary_window->window),widget);
+}
+
+static void
+refresh_statusbar (CongPrimaryWindow *primary_window)
+{
+	gchar *status_text = NULL;
+
+	g_assert (primary_window);
+	
+	/* Set status bar to XPath of selection (if any) */
+	if (primary_window->doc) {
+		CongNodePtr selected_node = cong_document_get_selected_node (primary_window->doc);
+
+		if (selected_node) {
+			status_text = cong_node_get_path (selected_node);
+		}
+	}
+
+	if (status_text) {
+		gnome_appbar_set_status (GNOME_APPBAR(primary_window->app_bar), 
+					 status_text);
+		g_free(status_text);
+	} else {
+		gnome_appbar_set_status (GNOME_APPBAR(primary_window->app_bar), 
+					 "");
+	}
+}
+
+static void
+end_edit_cb (CongDocument *doc,
+	     CongPrimaryWindow *primary_window)
+{
+	refresh_statusbar (primary_window);
+}
+
+/**
+ * cong_primary_window_add_doc:
+ * @primary_window:
+ * @doc:
+ * 
+ * Adds the #CongDocument @doc to a given window. This function is called from 
+ * cong_primary_window_new().
+ *
+ */
 void
 cong_primary_window_add_doc (CongPrimaryWindow *primary_window, CongDocument *doc)
 {
-/* Not sure if these doc != NULL tests area still needed here */
-	GtkWidget *w1 = NULL, *w2 = NULL;
-	GtkWidget *sidebar_notebook = NULL;
 	GdkColor gcol;
 	GtkStyle *style;
 	int i;
-	GtkItemFactory *item_factory = NULL;
 	
 	g_assert (primary_window);
 
 	if (doc) {
-		gchar *status_text;
 		primary_window->doc = doc;
 		g_object_ref(G_OBJECT(doc));
 
-		/* --- Main window -> hpane --- */
-		w1 = gtk_hpaned_new();
-		gnome_app_set_contents(GNOME_APP(primary_window->window),w1);
-		gtk_widget_show(w1);
-		
-		/* --- Notebook to appear in the sidebar: --- */
-		sidebar_notebook = gtk_notebook_new();
-		gtk_widget_show(sidebar_notebook);
-		gtk_notebook_set_tab_pos(GTK_NOTEBOOK(sidebar_notebook), GTK_POS_BOTTOM);
-
-		gtk_paned_add1(GTK_PANED(w1), sidebar_notebook);
-
-		/* --- Tree view --- */
-		LOG_PRIMARY_WINDOW_CREATION1 ("Creating overview");
-		
-		w2 = gtk_scrolled_window_new(NULL, NULL);
-		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(w2), GTK_POLICY_AUTOMATIC,
-					       GTK_POLICY_AUTOMATIC);
-		gtk_widget_set_size_request(GTK_WIDGET(w2), 100, 0);
-		gtk_widget_show(w2);
-
-		gtk_notebook_append_page(GTK_NOTEBOOK(sidebar_notebook),
-					 w2,
-					 gtk_label_new(_("Overview"))
-					 );
-
-		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(w2), 
-						      cong_tree_view_get_widget (cong_overview_view_new (primary_window->doc)));
-		
-		/* --- Bookmark view --- */
-#if 0
-		gtk_notebook_append_page(GTK_NOTEBOOK(sidebar_notebook),
-					 cong_bookmark_view_new(primary_window->doc),
-					 gtk_label_new(_("Bookmarks"))
-					 );
-
-#endif
-	
-		/* --- Raw XML view --- */
-		LOG_PRIMARY_WINDOW_CREATION1 ("Creating raw XML view");
-		gtk_notebook_append_page(GTK_NOTEBOOK(sidebar_notebook),
-					 cong_dom_view_new(primary_window->doc),
-					 gtk_label_new(_("Raw XML"))
-					 );
-		
-		/* Set up the editor_widget v3: */
-#if ENABLE_MAIN_WIDGET
-		{
-			LOG_PRIMARY_WINDOW_CREATION1 ("Creating v3 widget");
-			primary_window->cong_editor_widget3 = cong_editor_widget3_new(doc);
-
-			/* --- Scrolling area for editor widget 3--- */
-			primary_window->scroller3 = gtk_scrolled_window_new(NULL, NULL);
-			
-			gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(primary_window->scroller3), 
-							GTK_POLICY_AUTOMATIC,
-							GTK_POLICY_ALWAYS);
-			
-			gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(primary_window->scroller3), 
-							       GTK_WIDGET(primary_window->cong_editor_widget3));
-			gtk_widget_show (primary_window->cong_editor_widget3);
-			gtk_widget_show(primary_window->scroller3);
-
-			gtk_paned_add2 (GTK_PANED(w1), 
-					primary_window->scroller3);
+		if (1 /* cong_document_get_default_dispspec (doc) */) {
+			add_standard_layout_for_doc (primary_window, 
+						     doc);
+		} else {
+			add_tree_layout_for_doc (primary_window,
+						 doc);
 		}
-#endif
 
 		/* TEMPORARY: Set white background */
 		
@@ -578,34 +508,36 @@ cong_primary_window_add_doc (CongPrimaryWindow *primary_window, CongDocument *do
 			style->bg[i] = gcol;
 		}
 		
-		/* update the statusbar */
-		
-		status_text = g_strdup(cong_dispspec_get_name( cong_document_get_dispspec(primary_window->doc) ));
-#if 1
-		gnome_appbar_set_status (GNOME_APPBAR(primary_window->app_bar), 
-			 status_text);
-#else
-		gtk_statusbar_push(GTK_STATUSBAR(primary_window->status), 
-				   primary_window->status_main_ctx,
-				   status_text);
-#endif
-		g_free(status_text);
+		/* Set up status bar: */
+		refresh_statusbar (primary_window);
+		g_signal_connect_after (G_OBJECT (doc),
+					"end_edit",
+					G_CALLBACK (end_edit_cb),
+					primary_window);
+
 		cong_document_set_primary_window(doc, primary_window);
 	} /* if (doc) */
 	
 	/* toolbar update */
+#if 0
 	cong_primary_window_toolbar_populate (primary_window);
+#endif
 	/* title update */
 	cong_primary_window_update_title (primary_window);
+
 	/* update the menus */
-	item_factory = gtk_item_factory_from_widget (primary_window->menus);
-	cong_menus_create_items(item_factory, primary_window);
+	cong_menus_setup_document_action_group (primary_window);
 }
 
-
-void cong_primary_window_make_gui(CongPrimaryWindow *primary_window)
+/**
+ * cong_primary_window_make_gui:
+ * @primary_window:
+ *
+ * TODO: Write me
+ */
+void 
+cong_primary_window_make_gui(CongPrimaryWindow *primary_window)
 {
-	GtkItemFactory *item_factory;
 	gchar *title;
 
 	g_assert(primary_window);
@@ -621,7 +553,7 @@ void cong_primary_window_make_gui(CongPrimaryWindow *primary_window)
 	gtk_window_set_default_icon_from_file(
 		gnome_program_locate_file (cong_app_get_gnome_program (cong_app_singleton()),
 					   GNOME_FILE_DOMAIN_APP_PIXMAP,
-					   "conglomerate-icon-32.png", FALSE, 
+					   "conglomerate/conglomerate-icon-32.png", FALSE, 
 					   NULL), 
 		NULL);
 
@@ -637,27 +569,23 @@ void cong_primary_window_make_gui(CongPrimaryWindow *primary_window)
 
 	/* --- Menus --- */
 
-	primary_window->accel = gtk_accel_group_new();
-	item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", primary_window->accel);
-	gtk_item_factory_set_translate_func(item_factory, (GtkTranslateFunc) gettext, NULL, NULL);
-
-	cong_menus_create_items(item_factory, primary_window);
-
-	gtk_window_add_accel_group(GTK_WINDOW(primary_window->window), primary_window->accel);
-
-	primary_window->menus = gtk_item_factory_get_widget(item_factory, "<main>");
+	primary_window->merge_id = cong_menus_setup_ui_layout (primary_window);
+	primary_window->menus = gtk_ui_manager_get_widget (cong_primary_window_get_ui_manager (primary_window), "/MainMenuBar");
 	gnome_app_set_menus(GNOME_APP(primary_window->window), GTK_MENU_BAR(primary_window->menus));
 	gtk_widget_show(primary_window->menus);
+
+	{
+		GtkAccelGroup *accel_group;
+		
+		accel_group = gtk_ui_manager_get_accel_group (primary_window->ui_manager);
+		gtk_window_add_accel_group (GTK_WINDOW (primary_window->window), accel_group);	
+	}
 	
 	/* --- Toolbar --- */
-	primary_window->toolbar = GTK_TOOLBAR(gtk_toolbar_new());
+	primary_window->toolbar = GTK_TOOLBAR (gtk_ui_manager_get_widget (cong_primary_window_get_ui_manager (primary_window), "/MainToolBar"));
 	gnome_app_set_toolbar(GNOME_APP(primary_window->window), primary_window->toolbar);
 	gtk_widget_show(GTK_WIDGET(primary_window->toolbar));
 
-	/* --- Toolbar icons --- */
-
-	cong_primary_window_toolbar_populate(primary_window);
-
 	primary_window->doc = NULL;
 
 	/* --- Main window -> status area --- */
@@ -699,8 +627,15 @@ void cong_primary_window_make_gui(CongPrimaryWindow *primary_window)
 #endif
 }
 
-
-CongPrimaryWindow *cong_primary_window_new(CongDocument *doc)
+/**
+ * cong_primary_window_new:
+ * @doc:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+CongPrimaryWindow *
+cong_primary_window_new(CongDocument *doc)
 {
 
 	CongPrimaryWindow *prwin = (CongPrimaryWindow *)g_list_nth_data(cong_app_singleton()->primary_windows, 0);
@@ -715,6 +650,9 @@ CongPrimaryWindow *cong_primary_window_new(CongDocument *doc)
 	
 	primary_window = g_new0(CongPrimaryWindow, 1);
 
+	primary_window->ui_manager = gtk_ui_manager_new ();
+	cong_menus_setup_action_groups (primary_window);
+
 	cong_primary_window_make_gui(primary_window);
 	gtk_window_set_default_size(GTK_WINDOW(primary_window->window), 400, 460);
 
@@ -724,6 +662,7 @@ CongPrimaryWindow *cong_primary_window_new(CongDocument *doc)
 
 	cong_app_singleton()->primary_windows = g_list_prepend(cong_app_singleton()->primary_windows, primary_window);
 
+
 	if (doc) {
 		cong_primary_window_add_doc (primary_window, doc);
 	};
@@ -734,7 +673,14 @@ CongPrimaryWindow *cong_primary_window_new(CongDocument *doc)
 	
 }
 
-void cong_primary_window_free(CongPrimaryWindow *primary_window)
+/**
+ * cong_primary_window_free:
+ * @primary_window:
+ *
+ * TODO: Write me
+ */
+void 
+cong_primary_window_free(CongPrimaryWindow *primary_window)
 {
 	g_return_if_fail(primary_window);
 
@@ -749,14 +695,29 @@ void cong_primary_window_free(CongPrimaryWindow *primary_window)
 	g_free(primary_window);
 }
 
-CongDocument *cong_primary_window_get_document(CongPrimaryWindow *primary_window)
+/**
+ * cong_primary_window_get_document:
+ * @primary_window:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+CongDocument *
+cong_primary_window_get_document(CongPrimaryWindow *primary_window)
 {
 	g_return_val_if_fail(primary_window, NULL);
 
 	return primary_window->doc;
 }
 
-void cong_primary_window_update_title(CongPrimaryWindow *primary_window)
+/**
+ * cong_primary_window_update_title:
+ * @primary_window:
+ *
+ * TODO: Write me
+ */
+void 
+cong_primary_window_update_title(CongPrimaryWindow *primary_window)
 {
 	gchar *title;
 	gchar *filename;
@@ -785,9 +746,96 @@ void cong_primary_window_update_title(CongPrimaryWindow *primary_window)
 	g_free(title);
 }
 
-GtkWindow *cong_primary_window_get_toplevel(CongPrimaryWindow *primary_window)
+/**
+ * cong_primary_window_get_toplevel:
+ * @primary_window:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+GtkWindow *
+cong_primary_window_get_toplevel(CongPrimaryWindow *primary_window)
 {
 	g_return_val_if_fail(primary_window, NULL);
 
 	return GTK_WINDOW(primary_window->window);
 }
+
+/**
+ * cong_primary_window_get_ui_manager:
+ * @primary_window:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+GtkUIManager*
+cong_primary_window_get_ui_manager (CongPrimaryWindow *primary_window)
+{
+	g_return_val_if_fail (primary_window, NULL);
+
+	return primary_window->ui_manager;
+}
+
+GtkActionGroup*
+cong_primary_window_get_action_group (CongPrimaryWindow *primary_window,
+				      enum CongActionGroup action_group)
+{
+	g_return_val_if_fail (primary_window, NULL);
+	g_return_val_if_fail (action_group<NUM_CONG_ACTION_GROUPS, NULL);
+
+	return primary_window->action_group[action_group];
+}
+
+/**
+ * cong_primary_window_action_set_sensitive:
+ * @action_name: name of Action item
+ * @state: TRUE for sensitive, FALSE for insensitive
+ * @primary_window: the primary window containing the action
+ *
+ * Makes the given Action (@action_name) sensitive (@state=TRUE)
+ * or insensitive (@state=FALSE) for the primary window
+ * @primary_window.
+ */
+void
+cong_primary_window_action_set_sensitive (CongPrimaryWindow *primary_window,
+					  gchar *action_name,
+					  gboolean state) {
+	GtkActionGroup *action_group;
+	GtkAction *action;
+
+	action_group = cong_primary_window_get_action_group (primary_window, CONG_ACTION_GROUP_DOCUMENT);
+	g_assert (action_group);
+
+	action = gtk_action_group_get_action (action_group, action_name);
+	g_assert (action);
+
+	g_object_set (G_OBJECT (action), "sensitive", state, NULL);
+}
+
+/**
+ * cong_primary_window_action_set_label:
+ * @action_name: name of Action item
+ * @label: label for the Action
+ * @primary_window: the primary window containing the action
+ *
+ * Sets the label for the Action (@action_name) whose
+ * primary window is @primary_window.
+ *
+ * NOTE: should we make a copy of label?
+ */
+void
+cong_primary_window_action_set_label (CongPrimaryWindow *primary_window,
+				      gchar *action_name,
+				      gchar *label) {
+	GtkActionGroup *action_group;
+	GtkAction *action;
+
+	action_group = cong_primary_window_get_action_group (primary_window, CONG_ACTION_GROUP_DOCUMENT);
+	g_assert (action_group);
+
+	action = gtk_action_group_get_action (action_group, action_name);
+	g_assert (action);
+
+	g_object_set (G_OBJECT (action), "label", (gpointer)label, NULL);
+}
+
