@@ -32,7 +32,14 @@
 #include "cong-app.h"
 #include "cong-util.h"
 #include "cong-fake-plugin-hooks.h"
-#include "cong-attribute-editor.h"
+#include "cong-glade.h"
+#include "cong-dialog.h"
+#include "cong-vfs.h"
+
+#if ENABLE_PRINTING
+#include <libgnomeprint/gnome-print-config.h>
+#include <libgnomeprintui/gnome-print-paper-selector.h>
+#endif
 
 /* Splits input UTF8 into a GList of nul-terminated GUnichar strings */
 static GList*
@@ -56,7 +63,18 @@ struct DocBookCreationInfo
 };
 #endif
 
-void factory_page_creation_callback_unified(CongServiceDocumentFactory *factory, CongNewFileAssistant *assistant, gpointer user_data)
+/**
+ * factory_page_creation_callback_unified:
+ * @factory:
+ * @assistant:
+ * @user_data:
+ *
+ * TODO: Write me
+ */
+void 
+factory_page_creation_callback_unified(CongServiceDocumentFactory *factory,
+				       CongNewFileAssistant *assistant,
+				       gpointer user_data)
 {
 #if 0
 	GnomeDruidPageStandard *which_settings_page;
@@ -159,10 +177,19 @@ static xmlDtdPtr add_docbook_declaration(xmlDocPtr xml_doc, const xmlChar *root_
 	return xml_dtd;
 }
 
-xmlDocPtr make_article(const xmlChar *title)
+/**
+ * make_article:
+ * @title:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+xmlDocPtr 
+make_article(const xmlChar *title)
 {
 	xmlDocPtr xml_doc;
 	xmlNodePtr root_node;
+	xmlNodePtr section_node;
 
 	/* Build up the document and its content: */
 	xml_doc = xmlNewDoc("1.0");
@@ -184,7 +211,20 @@ xmlDocPtr make_article(const xmlChar *title)
 				  title)
 		    );
 
-	xmlAddChild(root_node, 
+	section_node = xmlNewDocNode(xml_doc,
+				     NULL,
+				     "section",
+				     "");
+	xmlAddChild(root_node, section_node);
+
+	xmlAddChild(section_node, 
+		    xmlNewDocNode(xml_doc,
+				  NULL,
+				  "title",
+				  _("Untitled section"))
+		    );
+
+	xmlAddChild(section_node, 
 		    xmlNewDocNode(xml_doc,
 				  NULL,
 				  "para",
@@ -194,7 +234,16 @@ xmlDocPtr make_article(const xmlChar *title)
 	return xml_doc;
 }
 
-void factory_action_callback_article(CongServiceDocumentFactory *factory, CongNewFileAssistant *assistant, gpointer user_data)
+/**
+ * factory_action_callback_article:
+ * @factory:
+ * @assistant:
+ * @user_data:
+ *
+ * TODO: Write me
+ */
+void 
+factory_action_callback_article(CongServiceDocumentFactory *factory, CongNewFileAssistant *assistant, gpointer user_data)
 {
 	xmlDocPtr xml_doc;
 
@@ -204,8 +253,15 @@ void factory_action_callback_article(CongServiceDocumentFactory *factory, CongNe
 						   cong_new_file_assistant_get_toplevel(assistant));
 }
 
-
-xmlDocPtr make_book(const xmlChar *title)
+/**
+ * make_book:
+ * @title:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+xmlDocPtr 
+make_book(const xmlChar *title)
 {
 	xmlDocPtr xml_doc;
 	xmlNodePtr root_node;
@@ -237,6 +293,13 @@ xmlDocPtr make_book(const xmlChar *title)
 				     "");
 	xmlAddChild(root_node, chapter_node);
 
+	xmlAddChild(chapter_node, 
+		    xmlNewDocNode(xml_doc,
+				  NULL,
+				  "title",
+				  _("Untitled chapter"))
+		    );
+
 	xmlAddChild(chapter_node, 
 		    xmlNewDocNode(xml_doc,
 				  NULL,
@@ -247,7 +310,16 @@ xmlDocPtr make_book(const xmlChar *title)
 	return xml_doc;
 }
 
-void factory_action_callback_book(CongServiceDocumentFactory *factory, CongNewFileAssistant *assistant, gpointer user_data)
+/**
+ * factory_action_callback_book:
+ * @factory:
+ * @assistant:
+ * @user_data:
+ *
+ * TODO: Write me
+ */
+void 
+factory_action_callback_book(CongServiceDocumentFactory *factory, CongNewFileAssistant *assistant, gpointer user_data)
 {
 	xmlDocPtr xml_doc;
 
@@ -257,11 +329,19 @@ void factory_action_callback_book(CongServiceDocumentFactory *factory, CongNewFi
 						   cong_new_file_assistant_get_toplevel(assistant));
 }
 
-
-xmlDocPtr make_set(const xmlChar *title)
+/**
+ * make_set:
+ * @title:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+xmlDocPtr 
+make_set(const xmlChar *title)
 {
 	xmlDocPtr xml_doc;
 	xmlNodePtr root_node;
+	xmlNodePtr book_node;
 
 	/* Build up the document and its content: */
 	xml_doc = xmlNewDoc("1.0");
@@ -283,10 +363,33 @@ xmlDocPtr make_set(const xmlChar *title)
 				  title)
 		    );
 
+	book_node = xmlNewDocNode(xml_doc,
+				     NULL,
+				     "book",
+				     "");
+
+	xmlAddChild(root_node, book_node);
+
+	xmlAddChild(book_node, 
+		    xmlNewDocNode(xml_doc,
+				  NULL,
+				  "title",
+				  _("Untitled book"))
+		    );
+
 	return xml_doc;
 }
 
-void factory_action_callback_set(CongServiceDocumentFactory *factory, CongNewFileAssistant *assistant, gpointer user_data)
+/**
+ * factory_action_callback_set:
+ * @factory:
+ * @assistant:
+ * @user_data:
+ *
+ * TODO: Write me
+ */
+void 
+factory_action_callback_set(CongServiceDocumentFactory *factory, CongNewFileAssistant *assistant, gpointer user_data)
 {
 	xmlDocPtr xml_doc;
 
@@ -296,27 +399,32 @@ void factory_action_callback_set(CongServiceDocumentFactory *factory, CongNewFil
 						   cong_new_file_assistant_get_toplevel(assistant));
 }
 
-gboolean text_importer_mime_filter(CongServiceImporter *importer, const gchar *mime_type, gpointer user_data)
+static GtkFileFilter*
+text_importer_filter_factory_callback (CongServiceImporter *importer)
 {
-	g_return_val_if_fail(importer, FALSE);
-	g_return_val_if_fail(mime_type, FALSE);
+	GtkFileFilter *filter;
 
-	if (0==strcmp(mime_type,"text/plain")) {
-		return TRUE;
-	} else if (0==strcmp(mime_type,"text/x-readme")) {
-		return TRUE;
-	} else if (0==strcmp(mime_type,"text/x-install")) {
-		return TRUE;
-	} else if (0==strcmp(mime_type,"text/x-copying")) {
-		return TRUE;
-	} else if (0==strcmp(mime_type,"application/octet-stream")) {
-		return TRUE;
-	} else {
-		return FALSE;
-	}
+	g_return_val_if_fail (importer, NULL);
+
+	filter = cong_service_importer_make_basic_filter (importer);
+
+	gtk_file_filter_add_mime_type (filter, "text/plain");
+	gtk_file_filter_add_mime_type (filter, "text/x-readme");
+	gtk_file_filter_add_mime_type (filter, "text/x-install");
+	gtk_file_filter_add_mime_type (filter, "text/x-copying");
+	gtk_file_filter_add_mime_type (filter, "application/octet-stream");
+
+	return filter;
 }
 
 /* Internal stuff for the text importer: */
+/**
+ * cong_unichar_is_line_break:
+ * @c:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean
 cong_unichar_is_line_break(gunichar c)
 {
@@ -463,6 +571,13 @@ cong_ucs4_is_list_item (gunichar *ucs4_input,
 	return FALSE;
 }
 
+/**
+ * should_merge:
+ * @ucs4_input_1:
+ * @ucs4_input_2:
+ *
+ * TODO: Write me
+ */
 gboolean
 should_merge (gunichar *ucs4_input_1,
 	      gunichar *ucs4_input_2)
@@ -486,6 +601,13 @@ should_merge (gunichar *ucs4_input_1,
 	return TRUE;
 }
 
+/**
+ * cong_ucs4_concat:
+ * @ucs4_input_1:
+ * @ucs4_input_2:
+ *
+ * TODO: Write me
+ */
 gunichar*
 cong_ucs4_concat (gunichar *ucs4_input_1,
 		  gunichar *ucs4_input_2)
@@ -514,7 +636,12 @@ cong_ucs4_concat (gunichar *ucs4_input_1,
 	return ucs4_result;
 }
 
-
+/**
+ * merge_lines:
+ * @list:
+ *
+ * TODO: Write me
+ */
 GList*
 merge_lines (GList *list)
 {
@@ -676,11 +803,13 @@ parse_text_buffer_into_docbook (CongDocument *doc,
 					if (current_sect) {
 						cong_document_private_node_set_parent (doc, 
 									       current_list,
-									       current_sect);
+									       current_sect,
+										       TRUE);
 					} else {
 						cong_document_private_node_set_parent (doc, 
 									       current_list, 
-									       root_node);
+									       root_node,
+										       TRUE);
 					}
 				}
 
@@ -690,15 +819,18 @@ parse_text_buffer_into_docbook (CongDocument *doc,
 					
 				cong_document_private_node_set_parent (doc, 
 							       listitem,
-							       current_list);
+							       current_list,
+								       TRUE);
 
 				cong_document_private_node_set_parent (doc, 							       
-							       para,
-							       listitem);
+								       para,
+								       listitem,
+								       TRUE);
 
 				cong_document_private_node_set_parent (doc,
 							       text_node,
- 							       para);
+ 							       para,
+								       TRUE);
 			} else {
 				CongNodePtr text_node;
 
@@ -727,15 +859,18 @@ parse_text_buffer_into_docbook (CongDocument *doc,
 					
 					cong_document_private_node_set_parent (doc, 
 								       current_sect, 
-								       root_node);
+								       root_node,
+									       TRUE);
 					
 					cong_document_private_node_set_parent (doc, 
 								       title,
-								       current_sect);
+								       current_sect,
+									       TRUE);
 					
 					cong_document_private_node_set_parent (doc, 							       
 								       text_node,
-								       title);
+								       title,
+									       TRUE);
 				} else {
 					/* FIXME: spot text of the form: "fubar: more text"; turn it into a <formalpara> tag instead */
 					CongNodePtr para_node;
@@ -746,15 +881,18 @@ parse_text_buffer_into_docbook (CongDocument *doc,
 					if (current_sect) {
 						cong_document_private_node_set_parent (doc, 
 									       para_node, 
-									       current_sect);
+									       current_sect,
+										       TRUE);
 					} else {
 						cong_document_private_node_set_parent (doc, 
 									       para_node, 
-									       root_node);
+									       root_node,
+										       TRUE);
 					}
 					cong_document_private_node_set_parent (doc, 
 								       text_node, 
-								       para_node);
+								       para_node,
+									       TRUE);
 				}
 			}
 		}
@@ -776,7 +914,18 @@ parse_text_buffer_into_docbook (CongDocument *doc,
 	cong_document_end_edit (doc);
 }
 
-void text_importer_action_callback(CongServiceImporter *importer, const gchar *uri, const gchar *mime_type, gpointer user_data, GtkWindow *toplevel_window)
+/**
+ * text_importer_action_callback:
+ * @importer:
+ * @uri:
+ * @mime_type:
+ * @user_data:
+ * @toplevel_window:
+ *
+ * TODO: Write me
+ */
+void 
+text_importer_action_callback(CongServiceImporter *importer, const gchar *uri, const gchar *mime_type, gpointer user_data, GtkWindow *toplevel_window)
 {
 	char* buffer;
 	GnomeVFSFileSize size;
@@ -820,7 +969,17 @@ void text_importer_action_callback(CongServiceImporter *importer, const gchar *u
 	}
 }
 
-gboolean sourcecode_importer_mime_filter(CongServiceImporter *importer, const gchar *mime_type, gpointer user_data)
+/**
+ * sourcecode_importer_mime_filter:
+ * @importer:
+ * @mime_type:
+ * @user_data:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+gboolean 
+sourcecode_importer_mime_filter(CongServiceImporter *importer, const gchar *mime_type, gpointer user_data)
 {
 	g_return_val_if_fail(importer, FALSE);
 	g_return_val_if_fail(mime_type, FALSE);
@@ -834,7 +993,18 @@ gboolean sourcecode_importer_mime_filter(CongServiceImporter *importer, const gc
 	}
 }
 
-void sourcecode_importer_action_callback(CongServiceImporter *importer, const gchar *uri, const gchar *mime_type, gpointer user_data, GtkWindow *toplevel_window)
+/**
+ * sourcecode_importer_action_callback:
+ * @importer:
+ * @uri:
+ * @mime_type:
+ * @user_data:
+ * @toplevel_window:
+ *
+ * TODO: Write me
+ */
+void 
+sourcecode_importer_action_callback(CongServiceImporter *importer, const gchar *uri, const gchar *mime_type, gpointer user_data, GtkWindow *toplevel_window)
 {
 	char* buffer;
 	GnomeVFSFileSize size;
@@ -880,7 +1050,16 @@ void sourcecode_importer_action_callback(CongServiceImporter *importer, const gc
 	}
 }
 
-gboolean docbook_exporter_document_filter(CongServiceExporter *exporter, CongDocument *doc, gpointer user_data)
+/**
+ * docbook_exporter_document_filter:
+ * @exporter:
+ * @doc:
+ * @user_data:
+ *
+ * TODO: Write me
+ */
+gboolean 
+docbook_exporter_document_filter(CongServiceExporter *exporter, CongDocument *doc, gpointer user_data)
 {
 	g_return_val_if_fail(exporter, FALSE);
 	g_return_val_if_fail(doc, FALSE);
@@ -888,7 +1067,107 @@ gboolean docbook_exporter_document_filter(CongServiceExporter *exporter, CongDoc
 	return cong_util_is_docbook(doc);
 }
 
-void html_exporter_action_callback(CongServiceExporter *exporter, CongDocument *doc, const gchar *uri, gpointer user_data, GtkWindow *toplevel_window)
+/*
+ * Returns the location of the DocBook stylesheet used to convert
+ * a DocBook file to the type argument (e.g. "html" or "fo"). The
+ * return value must be free-ed by the user. The label argument is
+ * used in the error dialog to describe the format being converted
+ * to (e.g. "HTML" or "XSL:FO").
+ *
+ * If no stylesheet can be found then an error dialog is created
+ * and NULL is returned.
+ *
+ */
+static gchar *
+get_and_check_stylesheet_path(gchar *type,
+			      gchar *label,
+			      GtkWindow *toplevel_window)
+{
+	GtkDialog *error_dialog;
+	gchar *msg;
+	gchar *tail;
+	gchar *stylesheet_path;
+
+	tail = g_strdup_printf ("%s/docbook.xsl", type);
+	stylesheet_path = cong_utils_get_norman_walsh_stylesheet (tail);
+	g_free (tail);
+	if (stylesheet_path!=NULL) {
+		return stylesheet_path;
+	}
+
+	/*
+	 * The text may not be very helpful.
+	 * It would be nice to make the URLs be actual links.
+	 */
+	msg = g_strdup_printf( _("Conglomerate can not find the stylesheet used to convert DocBook to %s"),
+			       label );
+	error_dialog = cong_error_dialog_new (toplevel_window,
+					      _("Unable to locate the DocBook stylesheets"),
+					      msg,
+					      _("Ensure that Norman Walsh's DocBook XSL stylesheets, available at http://docbook.sourceforge.net/projects/xsl/, are installed and are added to your XML catalog file (see http://www.xmlsoft.org/catalog.html for more information on catalogs)."));
+	
+	gtk_dialog_run (error_dialog);
+	gtk_widget_destroy (GTK_WIDGET(error_dialog));
+	g_free (msg);
+	return NULL;
+}
+
+/*
+ * perhaps this should be part of cong_ui_transform_doc_to_uri() ?
+ */
+static void
+do_transform(gchar *type,
+	     CongDocument *doc,
+	     const gchar *stylesheet_path,
+	     GList *stylesheet_args,
+	     const gchar *string_uri,
+	     GtkWindow *toplevel_window)
+{
+	/*
+	 * If the call fails then we assume there will have been an
+	 * appropriate error dialog created, so we only need to create
+	 * a dialog on a successfull transform.
+	 */
+	if (cong_ui_transform_doc_to_uri(doc,
+					 stylesheet_path,
+					 stylesheet_args, 
+					 string_uri,
+					 toplevel_window)) {
+
+		GtkDialog *dialog;
+		gchar *filename_alone;
+		gchar *path;
+		gchar *message;
+		
+		cong_vfs_split_string_uri (string_uri, &filename_alone, &path);
+		message = g_strdup_printf (_("%s conversion complete.\n\nConverted %s to %s in %s"),
+					   type,
+					   cong_document_get_filename (doc),
+					   filename_alone,
+					   path);
+		dialog = cong_dialog_information_alert_new (toplevel_window,
+							    message);
+		
+		gtk_dialog_run (dialog);
+		gtk_widget_destroy (GTK_WIDGET(dialog));
+		g_free (message);
+		g_free (filename_alone);
+		g_free (path);
+	}
+}
+
+/**
+ * html_exporter_action_callback:
+ * @exporter:
+ * @doc:
+ * @uri:
+ * @user_data:
+ * @toplevel_window:
+ *
+ * TODO: Write me
+ */
+void 
+html_exporter_action_callback(CongServiceExporter *exporter, CongDocument *doc, const gchar *uri, gpointer user_data, GtkWindow *toplevel_window)
 {
 	gchar *stylesheet_path;
 
@@ -897,25 +1176,80 @@ void html_exporter_action_callback(CongServiceExporter *exporter, CongDocument *
 	g_return_if_fail(uri);
 
 	g_message("html_exporter_action_callback");
-	
-	stylesheet_path = cong_utils_get_norman_walsh_stylesheet("html/docbook.xsl");
-	g_assert(stylesheet_path);
 
-	cong_ui_transform_doc_to_uri(doc,
-				     stylesheet_path,
-				     uri,
-				     toplevel_window);
+	stylesheet_path = get_and_check_stylesheet_path ("html","HTML",toplevel_window);
+	if (stylesheet_path==NULL) {
+		return;
+	}
 
-	g_free(stylesheet_path);
+	/* apply the transformation */
+	do_transform ("HTML",
+		      doc,
+		      stylesheet_path,
+		      NULL, 
+		      uri,
+		      toplevel_window);
+	g_free (stylesheet_path);
 }
 
-void pdf_exporter_action_callback(CongServiceExporter *exporter, CongDocument *doc, const gchar *uri, gpointer user_data, GtkWindow *toplevel_window)
+GList*
+make_fo_export_params (void)
+{
+	GList *result = NULL;
+
+	/* 
+In file:
+/usr/share/sgml/docbook/xsl-stylesheets-1.65.1-1/fo/
+<xsl:param name="paper.type" select="'USletter'"/>
+	*/
+
+#if 1
+	/* FIXME: this is a test; we probably should set this based on GnomePrint settings; or present a dialog */
+	result = g_list_append (result, cong_stylesheet_parameter_new ("paper.type", "A4"));
+#endif
+
+	return result;
+}
+
+static GtkWidget*
+make_pagination_options_widget (CongServiceExporter *exporter)
+{
+#if ENABLE_PRINTING
+	/* FIXME: none of this is actually hooked up to anything */
+	GnomePrintConfig *gpc = gnome_print_config_default ();
+	return gnome_paper_selector_new (gpc);
+#else
+	return NULL;
+#endif
+}
+
+static GtkWidget* 
+pdf_exporter_options_callback (CongServiceExporter *exporter, 
+			       CongDocument *doc, 
+			       gpointer user_data)
+{
+	return make_pagination_options_widget (exporter);
+}
+
+/**
+ * pdf_exporter_action_callback:
+ * @exporter:
+ * @doc:
+ * @uri:
+ * @user_data:
+ * @toplevel_window:
+ *
+ * TODO: Write me
+ */
+void 
+pdf_exporter_action_callback(CongServiceExporter *exporter, CongDocument *doc, const gchar *uri, gpointer user_data, GtkWindow *toplevel_window)
 {
 #if 1
 	GtkWidget *progress_checklist_dialog;
 	CongProgressChecklist *progress_checklist;
 	xmlDocPtr fo_doc;
 	gchar *stylesheet_path;
+	GList *list_of_parameters;
 
 	g_message("pdf_exporter_action_callback");
 
@@ -923,8 +1257,12 @@ void pdf_exporter_action_callback(CongServiceExporter *exporter, CongDocument *d
 	g_return_if_fail(doc);
 	g_return_if_fail(uri);
 
-	stylesheet_path = cong_utils_get_norman_walsh_stylesheet("fo/docbook.xsl");
-	g_assert(stylesheet_path);
+	stylesheet_path = get_and_check_stylesheet_path ("fo","XSL:FO",toplevel_window);
+	if (stylesheet_path==NULL) {
+		return;
+	}
+
+	list_of_parameters = make_fo_export_params ();
 
 	progress_checklist_dialog = cong_progress_checklist_dialog_new(_("Exporting PDF file"), toplevel_window);
 	progress_checklist = cong_progress_checklist_dialog_get_progress_checklist(CONG_PROGRESS_CHECKLIST_DIALOG(progress_checklist_dialog));
@@ -938,9 +1276,11 @@ void pdf_exporter_action_callback(CongServiceExporter *exporter, CongDocument *d
 
 	fo_doc = cong_ui_transform_doc(doc,
 				       stylesheet_path,
+				       list_of_parameters,				       
 				       toplevel_window);
 
 	g_free(stylesheet_path);
+	cong_stylesheet_parameter_list_free (list_of_parameters);
 
 	if (fo_doc) {
 		cong_progress_checklist_complete_stage(progress_checklist);
@@ -960,9 +1300,30 @@ void pdf_exporter_action_callback(CongServiceExporter *exporter, CongDocument *d
 #endif
 }
 
-void fo_exporter_action_callback(CongServiceExporter *exporter, CongDocument *doc, const gchar *uri, gpointer user_data, GtkWindow *toplevel_window)
+
+static GtkWidget* 
+fo_exporter_options_callback (CongServiceExporter *exporter, 
+			      CongDocument *doc, 
+			      gpointer user_data)
+{
+	return make_pagination_options_widget (exporter);
+}
+
+/**
+ * fo_exporter_action_callback:
+ * @exporter:
+ * @doc:
+ * @uri:
+ * @user_data:
+ * @toplevel_window:
+ *
+ * TODO: Write me
+ */
+void 
+fo_exporter_action_callback(CongServiceExporter *exporter, CongDocument *doc, const gchar *uri, gpointer user_data, GtkWindow *toplevel_window)
 {
 	gchar *stylesheet_path;
+	GList *list_of_parameters;
 
 	g_return_if_fail(exporter);
 	g_return_if_fail(doc);
@@ -970,18 +1331,33 @@ void fo_exporter_action_callback(CongServiceExporter *exporter, CongDocument *do
 
 	g_message("fo_exporter_action_callback");
 
-	stylesheet_path = cong_utils_get_norman_walsh_stylesheet("fo/docbook.xsl");
-	g_assert(stylesheet_path);
-
-	cong_ui_transform_doc_to_uri(doc,
-				     stylesheet_path,
-				     uri,
-				     toplevel_window);
+	stylesheet_path = get_and_check_stylesheet_path ("fo","XSL:FO",toplevel_window);
+	if (stylesheet_path==NULL) {
+		return;
+	}
 
+	/* apply the transformation */
+	list_of_parameters = make_fo_export_params ();
+	do_transform ("XSL:FO",
+		      doc,
+		      stylesheet_path,
+		      list_of_parameters, 
+		      uri,
+		      toplevel_window);
 	g_free(stylesheet_path);
+	cong_stylesheet_parameter_list_free (list_of_parameters);
 }
 
-#if (ENABLE_PRINTING &&ENABLE_LIBFO)
+#if (ENABLE_PRINTING && ENABLE_LIBFO)
+/**
+ * docbook_print_method_document_filter:
+ * @print_method:
+ * @doc:
+ * @user_data:
+ *
+ * TODO: Write me
+ * Returns:
+ */
 gboolean 
 docbook_print_method_document_filter (CongServicePrintMethod *print_method, 
 				      CongDocument *doc, 
@@ -993,6 +1369,16 @@ docbook_print_method_document_filter (CongServicePrintMethod *print_method,
 	return cong_util_is_docbook(doc);
 }
 
+/**
+ * docbook_print_method_action_callback:
+ * @print_method:
+ * @doc:
+ * @gpc:
+ * @user_data:
+ * @toplevel_window:
+ *
+ * TODO: Write me
+ */
 void 
 docbook_print_method_action_callback (CongServicePrintMethod *print_method, 
 				      CongDocument *doc, 
@@ -1017,34 +1403,46 @@ docbook_print_method_action_callback (CongServicePrintMethod *print_method,
 	gtk_widget_show(progress_checklist_dialog);
 
 	{
-		gchar *stylesheet_path = cong_utils_get_norman_walsh_stylesheet("fo/docbook.xsl");
-		g_assert(stylesheet_path);
+		GList *list_of_parameters;	
+		gchar *stylesheet_path = get_and_check_stylesheet_path ("fo","XSL:FO",toplevel_window);
+		if (stylesheet_path==NULL) {
+			/* what should we do on error here? */
+			return;
+		}
 
+		list_of_parameters = make_fo_export_params ();
 		fo_doc = cong_ui_transform_doc(doc,
 					       stylesheet_path,
+					       list_of_parameters,
 					       toplevel_window);
-
 		g_free(stylesheet_path);
+		cong_stylesheet_parameter_list_free (list_of_parameters);
 	}
 
 	if (fo_doc) {
 		cong_progress_checklist_complete_stage(progress_checklist);
-
 		cong_util_print_xslfo(toplevel_window, 
 				      gpc, 
 				      fo_doc);
-
 		xmlFreeDoc(fo_doc);
 	}
 
-
 	gtk_widget_destroy(progress_checklist_dialog);
 
-
 }
 #endif /* #if ENABLE_PRINTING */
 
-GtkWidget* docbook_generic_node_factory_method(CongServiceNodePropertyDialog *custom_property_dialog, CongDocument *doc, CongNodePtr node)
+/**
+ * docbook_generic_node_factory_method:
+ * @custom_property_dialog:
+ * @doc:
+ * @node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+GtkWidget* 
+docbook_generic_node_factory_method(CongServiceNodePropertyDialog *custom_property_dialog, CongDocument *doc, CongNodePtr node)
 {
 	gchar* glade_filename;
 	GladeXML *xml;
@@ -1058,7 +1456,7 @@ GtkWidget* docbook_generic_node_factory_method(CongServiceNodePropertyDialog *cu
 
 	glade_filename = gnome_program_locate_file (cong_app_get_gnome_program (cong_app_singleton()),
 						    GNOME_FILE_DOMAIN_APP_DATADIR,
-						    "glade/docbook-common-properties.glade",
+						    "conglomerate/glade/docbook-common-properties.glade",
 						    FALSE,
 						    NULL);
 
@@ -1084,7 +1482,19 @@ GtkWidget* docbook_generic_node_factory_method(CongServiceNodePropertyDialog *cu
 	return glade_xml_get_widget(xml, "common_dialog");
 }
 
-GtkWidget* docbook_orderedlist_properties_factory_method(CongServiceNodePropertyDialog *custom_property_dialog, CongDocument *doc, CongNodePtr node)
+/**
+ * docbook_orderedlist_properties_factory_method:
+ * @custom_property_dialog:
+ * @doc:
+ * @node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+GtkWidget* 
+docbook_orderedlist_properties_factory_method(CongServiceNodePropertyDialog *custom_property_dialog,
+					      CongDocument *doc,
+					      CongNodePtr node)
 {
 	gchar* glade_filename;
 	GladeXML *xml;
@@ -1098,7 +1508,7 @@ GtkWidget* docbook_orderedlist_properties_factory_method(CongServiceNodeProperty
 
 	glade_filename = gnome_program_locate_file (cong_app_get_gnome_program (cong_app_singleton()),
 						    GNOME_FILE_DOMAIN_APP_DATADIR,
-						    "glade/docbook-orderedlist-properties.glade",
+						    "conglomerate/glade/docbook-orderedlist-properties.glade",
 						    FALSE,
 						    NULL);
 
@@ -1119,26 +1529,31 @@ GtkWidget* docbook_orderedlist_properties_factory_method(CongServiceNodeProperty
 		cong_bind_radio_button (GTK_RADIO_BUTTON (glade_xml_get_widget(xml, "arabic")),
 					doc,
 					node,
+					NULL,
 					"numeration",
 					"arabic");
 		cong_bind_radio_button (GTK_RADIO_BUTTON (glade_xml_get_widget(xml, "loweralpha")),
 					doc,
 					node,
+					NULL,
 					"numeration",
 					"loweralpha");
 		cong_bind_radio_button (GTK_RADIO_BUTTON (glade_xml_get_widget(xml, "lowerroman")),
 					doc,
 					node,
+					NULL,
 					"numeration",
 					"lowerroman");
 		cong_bind_radio_button (GTK_RADIO_BUTTON (glade_xml_get_widget(xml, "upperalpha")),
 					doc,
 					node,
+					NULL,
 					"numeration",
 					"upperalpha");
 		cong_bind_radio_button (GTK_RADIO_BUTTON (glade_xml_get_widget(xml, "upperroman")),
 					doc,
 					node,
+					NULL,
 					"numeration",
 					"upperroman");
 		
@@ -1146,18 +1561,21 @@ GtkWidget* docbook_orderedlist_properties_factory_method(CongServiceNodeProperty
 		cong_bind_check_button (GTK_CHECK_BUTTON (glade_xml_get_widget(xml, "inheritnum")),
 					doc,
 					node,
+					NULL,
 					"inheritnum",
 					"ignore",
 					"inherit");
 		cong_bind_check_button (GTK_CHECK_BUTTON (glade_xml_get_widget(xml, "spacing")),
 					doc,
 					node,
+					NULL,
 					"spacing",
 					"normal",
 					"compact");
 		cong_bind_check_button (GTK_CHECK_BUTTON (glade_xml_get_widget(xml, "continuation")),
 					doc,
 					node,
+					NULL,
 					"continuation",
 					"restart",
 					"continues");
@@ -1174,11 +1592,9 @@ GtkWidget* docbook_orderedlist_properties_factory_method(CongServiceNodeProperty
 }
 
 static void
-on_test_link_pressed (GtkButton *button,
-		      gpointer user_data)
+open_ulink_in_browser (CongNodePtr node)
 {
-	CongNodePtr node = user_data;
-	gchar *url = cong_node_get_attribute (node, "url");
+	gchar *url = cong_node_get_attribute (node, NULL, "url");
 
 	if (url) {	
 		/* FIXME: should we have some error handling? */
@@ -1189,7 +1605,28 @@ on_test_link_pressed (GtkButton *button,
 	g_free (url);
 }
 
-GtkWidget* docbook_ulink_properties_factory_method(CongServiceNodePropertyDialog *custom_property_dialog, CongDocument *doc, CongNodePtr node)
+static void
+on_test_link_pressed (GtkButton *button,
+		      gpointer user_data)
+{
+	CongNodePtr node = user_data;
+
+	open_ulink_in_browser (node);
+}
+
+/**
+ * docbook_ulink_properties_factory_method:
+ * @custom_property_dialog:
+ * @doc:
+ * @node:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+GtkWidget* 
+docbook_ulink_properties_factory_method(CongServiceNodePropertyDialog *custom_property_dialog,
+					CongDocument *doc,
+					CongNodePtr node)
 {
 	gchar* glade_filename;
 	GladeXML *xml;
@@ -1203,7 +1640,7 @@ GtkWidget* docbook_ulink_properties_factory_method(CongServiceNodePropertyDialog
 
 	glade_filename = gnome_program_locate_file (cong_app_get_gnome_program (cong_app_singleton()),
 						    GNOME_FILE_DOMAIN_APP_DATADIR,
-						    "glade/docbook-ulink-properties.glade",
+						    "conglomerate/glade/docbook-ulink-properties.glade",
 						    FALSE,
 						    NULL);
 
@@ -1232,7 +1669,40 @@ GtkWidget* docbook_ulink_properties_factory_method(CongServiceNodePropertyDialog
 	return glade_xml_get_widget(xml, "common_dialog");
 }
 
-gboolean 
+static enum NodeToolFilterResult 
+node_filter_browse_url (CongServiceNodeTool *node_tool, 
+			CongDocument *doc, 
+			CongNodePtr node,
+			gpointer user_data)
+{
+	if (cong_node_is_element (node,
+				  NULL,
+				  "ulink")) {
+		gchar *url = cong_node_get_attribute (node, NULL, "url");
+		
+		if (url) {
+			g_free (url);
+
+			return NODE_TOOL_AVAILABLE;
+		} else {
+			return NODE_TOOL_INSENSITIVE;
+		}
+	} else {
+		return NODE_TOOL_HIDDEN;
+	}
+}
+
+static void 
+action_callback_browse_url (CongServiceNodeTool *tool, 
+			    CongDocument *doc, 
+			    CongNodePtr node,
+			    GtkWindow *parent_window,
+			    gpointer user_data)
+{
+	open_ulink_in_browser (node);
+}
+
+static enum NodeToolFilterResult
 node_filter_promote (CongServiceNodeTool *node_tool, 
 		     CongDocument *doc, 
 		     CongNodePtr node,
@@ -1240,33 +1710,36 @@ node_filter_promote (CongServiceNodeTool *node_tool,
 {
 	if (cong_util_is_docbook(doc)) {
 		if (cong_node_is_element(node, NULL, "sect2")) {
-			return TRUE;
+			return NODE_TOOL_AVAILABLE;
 		} else if (cong_node_is_element(node, NULL, "sect3")) {
-			return TRUE;
+			return NODE_TOOL_AVAILABLE;
 		} else if (cong_node_is_element(node, NULL, "sect4")) {
-			return TRUE;
+			return NODE_TOOL_AVAILABLE;
 		} else if (cong_node_is_element(node, NULL, "sect5")) {
-			return TRUE;
+			return NODE_TOOL_AVAILABLE;
 		}
 
 		/* FIXME: handle <sect> tags */
 	}
 
-	return FALSE;
+	return NODE_TOOL_HIDDEN;
 }
 
-void
+static void
 action_callback_promote (CongServiceNodeTool *tool, 
-			 CongPrimaryWindow *primary_window, 
+			 CongDocument *doc, 
 			 CongNodePtr node,
+			 GtkWindow *parent_window,
 			 gpointer user_data)
 {
 	g_message ("action_callback_promote");
 
 	/* Unwritten */
+	CONG_DO_UNIMPLEMENTED_DIALOG (parent_window,
+				      "promote DocBook");
 }
 
-gboolean 
+static enum NodeToolFilterResult
 node_filter_demote (CongServiceNodeTool *node_tool, 
 		    CongDocument *doc, 
 		    CongNodePtr node,
@@ -1274,36 +1747,47 @@ node_filter_demote (CongServiceNodeTool *node_tool,
 {
 	if (cong_util_is_docbook(doc)) {
 		if (cong_node_is_element(node, NULL, "sect1")) {
-			return TRUE;
+			return NODE_TOOL_AVAILABLE;
 		} else if (cong_node_is_element(node, NULL, "sect2")) {
-			return TRUE;
+			return NODE_TOOL_AVAILABLE;
 		} else if (cong_node_is_element(node, NULL, "sect3")) {
-			return TRUE;
+			return NODE_TOOL_AVAILABLE;
 		} else if (cong_node_is_element(node, NULL, "sect4")) {
-			return TRUE;
+			return NODE_TOOL_AVAILABLE;
 		}
 
 		/* FIXME: handle <sect> tags */
 	}
 
-	return FALSE;
+	return NODE_TOOL_HIDDEN;
 }
 
-void
+static void
 action_callback_demote (CongServiceNodeTool *tool, 
-			CongPrimaryWindow *primary_window, 
+			CongDocument *doc, 
 			CongNodePtr node,
+			GtkWindow *parent_window,
 			gpointer user_data)
 {
 	g_message ("action_callback_demote");
 
 	/* Unwritten */
+	CONG_DO_UNIMPLEMENTED_DIALOG (parent_window,
+				      "demote DocBook");
 }
 
 
 
- /* would be exposed as "plugin_register"? */
-gboolean plugin_docbook_plugin_register(CongPlugin *plugin)
+/* would be exposed as "plugin_register"? */
+/**
+ * plugin_docbook_plugin_register:
+ * @plugin:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+gboolean 
+plugin_docbook_plugin_register(CongPlugin *plugin)
 {
 	g_return_val_if_fail(plugin, FALSE);
 	
@@ -1336,7 +1820,8 @@ gboolean plugin_docbook_plugin_register(CongPlugin *plugin)
 				      _("Import text as a DocBook article"), 
 				      _("Import a plain text file into the \"DocBook\" format, as an article."),
 				      "docbook-plaintext-import",
-				      text_importer_mime_filter,
+				      text_importer_filter_factory_callback,
+				      NULL,
 				      text_importer_action_callback,
 				      NULL);
 
@@ -1355,6 +1840,7 @@ gboolean plugin_docbook_plugin_register(CongPlugin *plugin)
 				      _("Use Norman Walsh's DocBook stylesheets to create a webpage from this DocBook file"),
 				      "docbook-HTML-export",
 				      docbook_exporter_document_filter,
+				      NULL,
 				      html_exporter_action_callback,
 				      NULL);
 
@@ -1363,6 +1849,7 @@ gboolean plugin_docbook_plugin_register(CongPlugin *plugin)
 				      _("Use Norman Walsh's DocBook stylesheets to create a PDF file from this DocBook file"),
 				      "docbook-PDF-export",
 				      docbook_exporter_document_filter,
+				      pdf_exporter_options_callback,
 				      pdf_exporter_action_callback,
 				      NULL);
 
@@ -1371,6 +1858,7 @@ gboolean plugin_docbook_plugin_register(CongPlugin *plugin)
 				      _("Use Norman Walsh's DocBook stylesheets to create an XSL:FO file from this DocBook file that can be printed or converted to PDF at a later date"),
 				      "docbook-XSLFO-export",
 				      docbook_exporter_document_filter,
+				      fo_exporter_options_callback,
 				      fo_exporter_action_callback,
 				      NULL);
 
@@ -1403,7 +1891,18 @@ gboolean plugin_docbook_plugin_register(CongPlugin *plugin)
 								 docbook_ulink_properties_factory_method,
 								 NULL);
 
-#if 0
+	cong_plugin_register_node_tool (plugin,
+					_("Open Link in Browser"), 
+					"",
+					"docbook-browse-to-url",
+					_("Open Link in Browser"),
+					NULL,
+					NULL,
+					node_filter_browse_url,
+					action_callback_browse_url,
+					NULL);
+
+#if 1
 	cong_plugin_register_node_tool (plugin,
 					_("Promote Section"), 
 					_("Promotes a DocBook section to a higher organisational level within the document"),
@@ -1431,7 +1930,15 @@ gboolean plugin_docbook_plugin_register(CongPlugin *plugin)
 }
 
 /* exposed as "plugin_configure"? legitimate for it not to be present */
-gboolean plugin_docbook_plugin_configure(CongPlugin *plugin)
+/**
+ * plugin_docbook_plugin_configure:
+ * @plugin:
+ *
+ * TODO: Write me
+ * Returns:
+ */
+gboolean 
+plugin_docbook_plugin_configure(CongPlugin *plugin)
 {
 	g_return_val_if_fail(plugin, FALSE);
 
